Package: heatmaply
Type: Package
Title: Interactive Cluster Heat Maps Using 'plotly' and 'ggplot2'
Version: 1.6.0
Date: 2025-07-12
Authors@R: c(
    person("Tal", "Galili", role = c("aut", "cre", "cph"), email = "tal.galili@gmail.com", comment = "https://www.r-statistics.com"),
    person("Alan", "O'Callaghan", comment = "https://github.com/Alanocallaghan",role = "aut"),
    person("Jonathan", "Sidi", email = "yonis@metrumrg.com", comment = "https://github.com/yonicd",role = "ctb"),
    person("Jaehyun", "Joo", comment = "https://github.com/jaehyunjoo",role = "ctb"),
    person("Yoav", "Benjamini", email = "ybenja@tau.ac.il",role = "ths"),
    person("Mathew", "Simenc", role = "ctb", comment = "https://gitlab.com/mcsimenc, https://github.com/mcsimenc"))
Description: Create interactive cluster 'heatmaps' that can be saved as a stand-
    alone HTML file, embedded in 'R Markdown' documents or in a 'Shiny' app, and
    available in the 'RStudio' viewer pane. Hover the mouse pointer over a cell to
    show details or drag a rectangle to zoom. A 'heatmap' is a popular graphical
    method for visualizing high-dimensional data, in which a table of numbers
    are encoded as a grid of colored cells. The rows and columns of the matrix
    are ordered to highlight patterns and are often accompanied by 'dendrograms'.
    'Heatmaps' are used in many fields for visualizing observations, correlations,
    missing values patterns, and more. Interactive 'heatmaps' allow the inspection
    of specific value by hovering the mouse over a cell, as well as zooming into
    a region of the 'heatmap' by dragging a rectangle around the relevant area.
    This work is based on the 'ggplot2' and 'plotly.js' engine. It produces
    similar 'heatmaps' to 'heatmap.2' with the advantage of speed
    ('plotly.js' is able to handle larger size matrix), the ability to zoom from
    the 'dendrogram' panes, and the placing of factor variables in the sides of the
    'heatmap'.
Depends: R (>= 3.0.0), plotly (>= 4.7.1), viridis
Imports: ggplot2 (>= 2.2.0), dendextend (>= 1.12.0), magrittr (>=
        1.0.1), reshape2, scales, seriation, utils, stats, grDevices,
        methods, colorspace, RColorBrewer, htmlwidgets, webshot,
        assertthat, egg
Suggests: knitr, covr, gplots, tidyselect, rmarkdown, testthat
VignetteBuilder: knitr
License: GPL-2 | GPL-3
URL: https://talgalili.github.io/heatmaply/,
        https://cran.r-project.org/package=heatmaply,
        https://github.com/talgalili/heatmaply/,
        https://www.r-statistics.com/tag/heatmaply/
BugReports: https://github.com/talgalili/heatmaply/issues
RoxygenNote: 7.3.2
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-07-12 16:09:42 UTC; talgalili
Author: Tal Galili [aut, cre, cph] (https://www.r-statistics.com),
  Alan O'Callaghan [aut] (https://github.com/Alanocallaghan),
  Jonathan Sidi [ctb] (https://github.com/yonicd),
  Jaehyun Joo [ctb] (https://github.com/jaehyunjoo),
  Yoav Benjamini [ths],
  Mathew Simenc [ctb] (https://gitlab.com/mcsimenc,
    https://github.com/mcsimenc)
Maintainer: Tal Galili <tal.galili@gmail.com>
Repository: CRAN
Date/Publication: 2025-07-12 21:20:02 UTC
Built: R 4.6.0; ; 2026-04-30 07:40:47 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(heatmaply,default)
S3method(heatmaply,heatmapr)
S3method(normalize,data.frame)
S3method(normalize,default)
S3method(normalize,matrix)
S3method(percentize,data.frame)
S3method(percentize,default)
S3method(percentize,matrix)
export(Blues)
export(BrBG)
export(BuGn)
export(BuPu)
export(GnBu)
export(Greens)
export(Greys)
export(OrRd)
export(Oranges)
export(PRGn)
export(PiYG)
export(PuBu)
export(PuBuGn)
export(PuOr)
export(PuRd)
export(Purples)
export(RdBu)
export(RdGy)
export(RdPu)
export(RdYlBu)
export(RdYlGn)
export(Reds)
export(Spectral)
export(YlGn)
export(YlGnBu)
export(YlOrBr)
export(YlOrRd)
export(cool_warm)
export(ggheatmap)
export(heatmaply)
export(heatmaply_cor)
export(heatmaply_na)
export(heatmapr)
export(is.heatmapr)
export(is.na10)
export(is.plotly)
export(normalize)
export(percentize)
import(RColorBrewer)
import(dendextend)
import(ggplot2)
import(magrittr)
import(webshot)
importFrom(assertthat,assert_that)
importFrom(dendextend,as.ggdend)
importFrom(dendextend,color_branches)
importFrom(dendextend,find_k)
importFrom(dendextend,is.dendrogram)
importFrom(dendextend,is.hclust)
importFrom(dendextend,rotate)
importFrom(dendextend,seriate_dendrogram)
importFrom(dendextend,set)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.new)
importFrom(grDevices,rgb)
importFrom(htmlwidgets,saveWidget)
importFrom(methods,as)
importFrom(plotly,add_segments)
importFrom(plotly,add_trace)
importFrom(plotly,config)
importFrom(plotly,ggplotly)
importFrom(plotly,hide_colorbar)
importFrom(plotly,layout)
importFrom(plotly,plot_ly)
importFrom(plotly,plotly)
importFrom(plotly,plotly_empty)
importFrom(plotly,style)
importFrom(plotly,subplot)
importFrom(scales,col_factor)
importFrom(scales,col_numeric)
importFrom(seriation,get_order)
importFrom(seriation,seriate)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,dendrapply)
importFrom(stats,dist)
importFrom(stats,ecdf)
importFrom(stats,hclust)
importFrom(stats,is.leaf)
importFrom(stats,na.omit)
importFrom(stats,order.dendrogram)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(viridis,viridis)

<a href="https://github.com/talgalili/heatmaply/actions"
class="pkgdown-devel"><img
src="https://github.com/talgalili/heatmaply/workflows/R-CMD-check/badge.svg"
alt="R-CMD-check" /></a> 
[![codecov](https://codecov.io/gh/talgalili/heatmaply/branch/master/graph/badge.svg?token=cl7mOAKWTI)](https://app.codecov.io/gh/talgalili/heatmaply)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/heatmaply)](https://cran.r-project.org/package=heatmaply)
![](https://cranlogs.r-pkg.org/badges/heatmaply?color=yellow)
![](https://cranlogs.r-pkg.org/badges/grand-total/heatmaply?color=yellowgreen)

# heatmaply

**Table of contents:**

- [heatmaply](#heatmaply)
  - [Screenshot demo](#screenshot-demo)
  - [Introduction](#introduction)
    - [Please submit features requests](#please-submit-features-requests)
    - [Latest news](#latest-news)
    - [Code of conduct](#code-of-conduct)
  - [Installation](#installation)
  - [Usage](#usage)
    - [Saving your heatmaply into a file](#saving-your-heatmaply-into-a-file)
    - [Replacing d3heatmap with heatmaply](#replacing-d3heatmap-with-heatmaply)
  - [Acknowledgements](#acknowledgements)
  - [How to cite the heatmaply package](#how-to-cite-the-heatmaply-package)
  - [Contact](#contact)


## Screenshot demo

![](https://i.imgur.com/qdUCKlg.gif)



## Introduction

A heatmap is a popular graphical method for visualizing high-dimensional data, in which a table of numbers are encoded as a grid of colored cells. The rows and columns of the matrix are ordered to highlight patterns and are often accompanied by dendrograms. Heatmaps are used in many fields for visualizing observations, correlations, missing values patterns, and more.

Interactive heatmaps allow the inspection of specific value by hovering the mouse over a cell, as well as zooming into a region of the heatmap by dragging a rectangle around the relevant area.

This work is based on the ggplot2 and plotly.js engine. It produces similar heatmaps as d3heatmap (or the static heatmap.2 from gplots), with the advantage of more features such as speed (plotly.js is able to handle larger size matrix), sidebar annotation, and the ability to zoom from the dendrogram.


### Please submit features requests

This package is still under active development. If you have features you would like to have added, please submit your suggestions (and bug-reports) at: <https://github.com/talgalili/heatmaply/issues>


### Latest news

You can see the most recent changes to the package in the [NEWS.md file](https://talgalili.github.io/heatmaply/news/index.html)



### Code of conduct

Please note that this project is released with a [Contributor Code of Conduct](https://github.com/talgalili/heatmaply/blob/master/CONDUCT.md). By participating in this project you agree to abide by its terms.



## Installation

To install the stable version on CRAN:

```r
install.packages('heatmaply')
```

To install the latest ("cutting-edge") GitHub version run:

```R

# good packages to install for this to work smoothly:

install.packages(c("Rcpp","ggplot2","munsell","htmltools","DBI","assertthat",
"gridExtra","digest","fpc","TSP","registry","gclus","gplots","RColorBrewer",
"stringr","labeling","yaml"))

# You'll need devtools
install.packages.2 <- function (pkg) if (!require(pkg)) install.packages(pkg);
install.packages.2('devtools')
# make sure you have Rtools installed first! if not, then run:
#install.packages('installr'); install.Rtools()

devtools::install_github("ropensci/plotly") # you will probably benefit from the latest version of plotly
devtools::install_github('talgalili/heatmaply')
```

And then you may load the package using:

```R
library("heatmaply")
```

## Usage

Quick example:

```r
library(heatmaply)
heatmaply(mtcars, k_row = 3, k_col = 2)
```

**For more (interactive) examples see the [online vignette on CRAN](https://CRAN.R-project.org/package=heatmaply/vignettes/heatmaply.html)**

There are also more complex biological examples of using heatmaply in the R package [heatmaplyExamples](https://github.com/talgalili/heatmaplyExamples) (hosted on github). Here are direct links for available examples:


* [Introduction to heatmaply](https://CRAN.R-project.org/package=heatmaply/vignettes/heatmaply.html)
* General biological examples
  - [Using heatmaply with the measles data set](https://cdn.jsdelivr.net/gh/talgalili/heatmaplyExamples@master/inst/doc/measles.html)
  - [Using heatmaply with famous data sets](https://cdn.jsdelivr.net/gh/talgalili/heatmaplyExamples@master/inst/doc/heatmaply_examples.html)
* Reproducing heatmaps from papers published in Nature
  - [Using heatmaply to reproduce Nature (2015) Kotsyfakis et al.](https://cdn.jsdelivr.net/gh/talgalili/heatmaplyExamples@master/inst/doc/reproducing_Nature_2015_Kotsyfakis.html)
  - [Using heatmaply to reproduce Nature (2015) Alfano et al.](https://cdn.jsdelivr.net/gh/talgalili/heatmaplyExamples@master/inst/doc/reproducing_Nature_2015_Alfano.html)
* Using heatmaply with gene expression data  
  - Visualization of raw and voom-transformed data (all genes) (url offline, you could combile this on your own)
  - Visualization of raw data (median-centered data, PAM50 genes only) (url offline, you could combile this on your own)
  - Visualization of voom-transformed data (median-centered data, PAM50 genes only) (url offline, you could combile this on your own)
  - [Using heatmaply with non-centred RNAseq heatmaps (PAM50 genes) ](https://cdn.jsdelivr.net/gh/talgalili/heatmaplyExamples@master/inst/doc/non_centred_heatmaps.html)
* General examples
  - [Using heatmaply for visualizing glmnet coefficient path](https://cdn.jsdelivr.net/gh/talgalili/heatmaplyExamples@master/inst/doc/glmnet.html)



### Saving your heatmaply into a file

You can save an interactive version of your heatmaply into an HTML file using the following code:

```r
dir.create("folder")
library(heatmaply)
heatmaply(mtcars, file = "folder/heatmaply_plot.html")
browseURL("folder/heatmaply_plot.html")
```

Similar code can be used for saving a static file (png/jpeg/pdf)

```r
dir.create("folder")
library(heatmaply)
# Before the first time using this code you may need to first run:
# webshot::install_phantomjs()
heatmaply(mtcars, file = "folder/heatmaply_plot.png")
browseURL("folder/heatmaply_plot.png")
```

### Replacing d3heatmap with heatmaply

The package d3heatmap (https://github.com/talgalili/d3heatmap) is no not actively maintained.

If users are interested in replacing their `d3heatmap` functions, this is easily done with `heatmaply`:

* Function calls `d3heatmap::d3heatmap()` can be replaced with `heatmaply::heatmaply()`
* Users of `d3heatmap` for `shiny` applications can use the following functions:
    * `d3heatmap::d3heatmapOutput()` can be replaced with `plotly::plotlyOutput()`
    * `d3heatmap::renderD3heatmap()` can be replaced with `plotly::renderPlotly()`


## Acknowledgements


This package is thanks to the amazing work done by MANY people in the open source community. Beyond the many people working on the pipeline of R, thanks should go to the people working on ggplot2 (Hadley Wickham, etc.) and plotly (Carson Sievert, etc.). Also, many of the design elements were inspired by the work done on heatmap, heatmap.2 and d3heatmap, so special thanks goes to the R core team, Gregory R. Warnes, and Joe Cheng from RStudio. The dendrogram side of the package is based on the work in dendextend, in which special thanks should go to Andrie de Vries for his original work on bringing dendrograms to ggplot2 (which evolved into the richer ggdend objects, as implemented in dendextend). 

The work on heatmaply was done by Tal Galili, Alan O'Callaghan, and Jonathan Sidi (mostly on shinyHeatmaply).


**Funding**: This work was supported in part by the European Union Seventh Framework Programme (FP7/2007-2013) under grant agreement no. 604102 (Human Brain Project).  



## How to cite the heatmaply package

The methods within the package can be cited as:

    Tal Galili, Alan O'Callaghan, Jonathan Sidi, Carson Sievert; heatmaply: an R package for creating
    interactive cluster heatmaps for online publishing, Bioinformatics, , btx657,
    https://doi.org/10.1093/bioinformatics/btx657

A BibTeX entry for LaTeX users is

    @Article{,
      author = {{Galili} and {Tal} and {O'Callaghan} and {Alan} and {Sidi} and {Jonathan} and {Sievert} and {Carson}},
      title = {heatmaply: an R package for creating interactive cluster heatmaps for online publishing},
      journal = {Bioinformatics},
      year = {2017},
      doi = {10.1093/bioinformatics/btx657},
      url = {https://dx.doi.org/10.1093/bioinformatics/btx657},
      eprint = {https://academic.oup.com/bioinformatics/article-pdf/doi/10.1093/bioinformatics/btx657/21358327/btx657.pdf},
    }

This free open-source software implements academic research by the authors and co-workers. If you use
it, please support the project by citing the appropriate journal articles.




## Contact

You are welcome to:

* Ask questions on: <https://stackoverflow.com/questions/tagged/heatmaply>
* Submit suggestions and bug-reports at: <https://github.com/talgalili/heatmaply/issues>
* Send a pull request on: <https://github.com/talgalili/heatmaply/>
* Compose a friendly e-mail to: <tal.galili@gmail.com>

# Copyright (C) Tal Galili
#
# This file is part of heatmaply.
#
# heatmaply is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# heatmaply is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#





# ### add new branches:
# ### http://r-pkgs.had.co.nz/git.html#git-branch
# git checkout -b faster_01
# git push --set-upstream origin faster_01






#' @import dendextend
#' @import ggplot2
#' @import magrittr
#' @import RColorBrewer

# http://stackoverflow.com/questions/10325231/when-writing-my-own-r-package-i-cant-seem-to-get-other-packages-to-import-corr

#' @importFrom plotly plotly ggplotly plotly_empty subplot layout hide_colorbar config style add_trace
#' @importFrom scales col_factor col_numeric
#' @importFrom viridis viridis
#' @importFrom stats as.dendrogram dendrapply dist hclust is.leaf order.dendrogram reorder sd
#' @importFrom seriation seriate get_order
#' @importFrom dendextend set is.dendrogram is.hclust seriate_dendrogram rotate color_branches find_k as.ggdend
#' @importFrom stats ecdf na.omit as.dist cor
#' @importFrom htmlwidgets saveWidget
#' @importFrom plotly plot_ly add_segments
#' @importFrom assertthat assert_that
#' @importFrom grDevices dev.new
#' @importFrom assertthat assert_that

#'
NULL







#
#
#
#
#
#
#
#
# .onLoad <- function(libname, pkgname){
#    # Thanks for Romain: http://stackoverflow.com/questions/4369334/first-lib-idiom-in-r-packages
#
#    # adding and removing menus from the Rgui when loading and detaching the library
#    # setHook(packageEvent("installr", "attach"), {function(pkgname, libpath) {add.installr.GUI()}  } )
#    # setHook(packageEvent("heatmaply", "detach"), {function(pkgname, libpath) {remove_heatmaply_options()}  } )
#
#    # set default options for heatmaply
#    setHook(packageEvent("heatmaply", "onLoad"), {function(pkgname, libpath) {assign_heatmaply_options()}  } )
#
#    # Does NOT work!
#    # remove_heatmaply_options is currently an empty function. In the future, it should
#    # remove default options for heatmaply when unloading
#    setHook(packageEvent("heatmaply", "onUnload"), {function(pkgname, libpath) {remove_heatmaply_options()}  } )
#    setHook(packageEvent("heatmaply", "detach"), {function(pkgname, libpath) {remove_heatmaply_options()}  } )
#
#    # heatmaply::heatmaply_options()
#
#    # set default options for d3 dendrogram.
#    d3dendro_defaults(D3DENDRODEFAULTS)
# }
#
# # menus are added and removed as needed: !!
#

.onAttach <- function(lib, pkg, ...) {
  packageStartupMessage(heatmaplyWelcomeMessage())
}

#
#


heatmaplyWelcomeMessage <- function() {
  # library(utils)

  paste0(
    "\n",
    "======================\n",
    "Welcome to heatmaply version ", utils::packageDescription("heatmaply")$Version, "\n",
    # "\n",
    "\n",
    "Type citation('heatmaply') for how to cite the package.\n",
    "Type ?heatmaply for the main documentation.\n",
    "\n",
    "The github page is: https://github.com/talgalili/heatmaply/\n",
    "Please submit your suggestions and bug-reports at: https://github.com/talgalili/heatmaply/issues\n",
    "You may ask questions at stackoverflow, use the r and heatmaply tags: \n\t https://stackoverflow.com/questions/tagged/heatmaply\n",
    # "\n",
    # "\tTo suppress this message use:  ", "suppressPackageStartupMessages(library(heatmaply))\n",
    "======================\n"
  )
}




# using "zzz.r" like in devtools...

# When adding new files, make sure they are listed in DESCRIPTION:
# Collate:
#    'create.global.library.r'
# 'install.r'
# 'updateR.r'




# library(heatmaply)
# environmentIsLocked(as.environment("package:heatmaply"))
# lockEnvironment(env=as.environment("package:heatmaply"), bindings = FALSE)
# lockEnvironment(env=as.environment("package:heatmaply"), bindings = TRUE)


# IMPORTANT NOTICE: this will add Collate to the DESCRIPTION file, and if any new r file is added - it will need to be updated.
# Collate:
# +    'create.global.library.r'
# +    'install.r'
# +    'updateR.r'
# +    'zzz.r'


############
## OLD
# Steps:
# http://r.789695.n4.nabble.com/vignettes-problems-with-PDF-compaction-td4664909.html
# 1) install gs - http://www.ghostscript.com/download/gsdnld.html
# 2) find where it is, and update R_GSCMD:
# Sys.setenv(R_GSCMD="C:\\Program Files\\gs\\gs9.10\\bin\\gswin64c.exe")
# Sys.setenv(R_GSCMD="C:\\Program Files\\gs\\gs9.14\\bin\\gswin64c.exe")
# Sys.setenv(R_GSCMD="D:\\temp\\qpdf-5.1.2\\bin\\qpdf.exe")
# Sys.getenv("R_GSCMD")
# 3) Check that it works:
# system2(Sys.getenv("R_GSCMD"), args="--version")
# 4) use:
# library(tools)
# tools::compactPDF("inst\\doc\\heatmaply-tutorial.pdf", gs_quality="printer")
# tools::compactPDF("inst\\doc\\heatmaply-tutorial.pdf",
# qpdf = "D:\\temp\\qpdf-5.1.2\\bin\\qpdf.exe", gs_cmd = "C:\\Program Files\\gs\\gs9.14\\bin\\gswin64c.exe")
#### tools::compactPDF("inst\\doc\\heatmaply-tutorial.pdf", gs_quality="ebook")
#### tools::compactPDF("inst\\doc\\heatmaply-tutorial.pdf", gs_quality="screen")
#### tools::compactPDF("vignettes\\heatmaply-tutorial.pdf")
###   compacted 'heatmaply-tutorial.pdf' from 964Kb to 737Kb
#### tools::compactPDF("vignettes\\heatmaply-tutorial.pdf", gs_quality="ebook")

# For checking:
# 1) get qpdf
#     http://sourceforge.net/projects/qpdf/files/
# 2) put it somewhere
# 3) set R_QPDF
#  Sys.setenv(R_QPDF="C:\\Rtools\\qpdf-5.1.1\\bin\\qpdf.exe")
#  Sys.which(Sys.getenv("R_QPDF", "qpdf"))

# Also, make sure to add:
# options(repos=c("http://cran.rstudio.com", "http://www.stats.ox.ac.uk/pub/RWin" ))
# to D:\R\R-devel\etc\Rprofile.site

##############







##########
##########
##########
##########
## NEW
# How to deal with compression:
# 1) Download the latest qpdf: http://sourceforge.net/projects/qpdf/files/  (and place it somewhere)
# 2) Install gs - http://www.ghostscript.com/download/gsdnld.html
# 3) Run the following:

# tools::compactPDF("inst\\doc\\heatmaply-tutorial.pdf",
#                   qpdf = "C:\\Program Files (x86)\\qpdf-5.1.2\\bin\\qpdf.exe",
#                   gs_cmd = "C:\\Program Files\\gs\\gs9.14\\bin\\gswin64c.exe",
#                   gs_quality="ebook")

##########
##########
##########



# https://stat.ethz.ch/pipermail/r-help/2010-September/251194.html
# tools::showNonASCII( readLines("vignettes\\FAQ.Rmd"))
#





####### Cool stuff to add:
# library(devtools)
# use_code_of_conduct()
# use_cran_badge()
# use_coveralls()
# #



# install.packages("C:\\Dropbox\\aaaa good R code\\AA - My packages\\heatmaply_1.0.0.tar.gz", repos = NULL, type="source")

#
# # Run once:
# shell('set PATH=%PATH%;"C:\\Program%20Files%20(x86)\\Git\\bin"', intern = TRUE)
# shell("echo %PATH% ", intern= TRUE)
#
# system('set PATH=%PATH%;C:\\xampp\\php')
#
#
#
# # Creating a changelog using git
# First make sure git is in the path. Run the
# following using cmd.exe, as admin:
# setx PATH "C:\\Program Files (x86)\\Git\\bin"
#
# Then - run the script to create the ChangeLog before shipping the package.
# # http://stackoverflow.com/questions/10330425/how-do-i-export-a-git-log-to-a-text-file
# # http://stackoverflow.com/questions/3523534/good-ways-to-manage-a-changelog-using-git
# # http://www.commandlinefu.com/commands/view/12420/generate-a-change-log-with-git
# shell("git log --decorate > ChangeLog", intern = T)

# Modify it using: http://git-scm.com/book/en/Git-Basics-Viewing-the-Commit-History
#         http://stackoverflow.com/questions/9007181/custom-log-format-omits-newline-at-end-of-output
# shell('git log --graph --stat --date=iso > ChangeLog', intern = TRUE)
# use this:
# shell('git log --graph --stat --date=short --pretty=format:"%ad(%an) %s |%h" > ChangeLog', intern = TRUE)
#
# system.PATH()
# shell("path")


# shell("echo %PATH% ", intern= TRUE)
# library(rmarkdown)
# rmarkdown::render("NEWS",clean = TRUE,output_format = "html_document")



# https://github.com/r-lib/styler
# # install.packages("remotes")
# remotes::install_github("krlmlr/styler")
# styler::style_pkg()

# devtools::spell_check()




# Search [^\x00-\x7F]+
# with notepad++ with regex, in order to find non-ascii values.
# Source: http://stackoverflow.com/questions/20889996/notepad-how-to-remove-all-non-ascii-characters-with-regex




# when a function is renamed, its document in man must be removed - otherwise it may cause problems with the built check (it will try to run the code in the example, and will fail.)
# When all is done, run:
# library(devtools)
# check()
# browseURL(tempdir())
### http://www.rstudio.com/ide/docs/packages/build_options
#
# devtools::check(build_args="--no-build-vignettes --no-manual", args = "--no-examples --no-build-vignettes --no-manual",  cran = FALSE, cleanup = FALSE)
# check(build_args="--no-build-vignettes --no-manual", args = "--no-build-vignettes --no-manual",  cran = FALSE, cleanup = FALSE)
# check(build_args="--no-build-vignettes ", args = "--no-build-vignettes",  cran = FALSE, cleanup = FALSE)
# devtools::check(args="--as-cran")
# devtools::check("C:/Dropbox/aaaa good R code/AA - My packages/heatmaply", args="--as-cran")
#                 Thanks to: http://stackoverflow.com/questions/10017702/r-cmd-check-options-for-more-rigorous-testing-2-15-0


# shell('git log --graph --stat --date=short --pretty=format:"%ad(%an) %s |%h" > ChangeLog', intern = TRUE)
# system('git log --graph --stat --date=short --pretty=format:"%ad(%an) %s |%h" > ChangeLog', intern = TRUE)
# file.copy("NEWS", "NEWS.md",overwrite = TRUE)
# devtools::check_win_devel()
# pkgdown::build_home()
# pkgdown::build_site(run_dont_run = F)
# pkgdown::build_news()
# release()

# prepare the heatmapr object.

`%||%` <- function(a, b) {
  if (!is.null(a)) {
    a
  } else {
    b
  }
}



# Check if all values in vector x are unique
all_unique <- function(x, ...) {
  length(x) == length(unique(x))
}

fix_not_all_unique <- function(x, ...) {
  if(all_unique(x)) {
    return(x)
  } else {
    warning("Not all the values are unique - manually added prefix numbers")
    return(paste0(seq_along(x), "_", x))
  }
}





#' Creates a heatmapr object
#'
#' An object of class heatmapr includes all the needed information
#' for producing a heatmap. The goal is to separate the pre-processing of the
#' heatmap elements from the graphical rendering of the object, which could be done
#' using plotly (but potentially also with other graphical devices).
#'
#' @param x A numeric matrix
#'   Defaults to \code{TRUE} unless \code{x} contains any \code{NA}s.
#' @param theme A custom CSS theme to use. Currently the only valid values are
#'   \code{""} and \code{"dark"}. \code{"dark"} is primarily intended for
#'   standalone visualizations, not R Markdown or Shiny.
#' @param colors Either a colorbrewer2.org palette name (e.g. \code{"YlOrRd"} or
#'   \code{"Blues"}), or a vector of colors to interpolate in hexadecimal
#'   \code{"#RRGGBB"} format, or a color interpolation function like
#'   \code{\link[grDevices]{colorRamp}}.
#' @param width Width in pixels (optional, defaults to automatic sizing).
#' @param height Height in pixels (optional, defaults to automatic sizing).
#'
#' @param xaxis_height Size of axes, in pixels.
#' @param yaxis_width Size of axes, in pixels.
#' @param xaxis_font_size Font size of axis labels, as a CSS size (e.g. "14px" or "12pt").
#' @param yaxis_font_size Font size of axis labels, as a CSS size (e.g. "14px" or "12pt").
#'
#' @param brush_color The base color to be used for the brush. The brush will be
#'   filled with a low-opacity version of this color. \code{"#RRGGBB"} format
#'   expected.
#' @param show_grid \code{TRUE} to show gridlines, \code{FALSE} to hide them, or
#'   a numeric value to specify the gridline thickness in pixels (can be a
#'   non-integer).
#' @param anim_duration Number of milliseconds to animate zooming in and out.
#'   For large \code{x} it may help performance to set this value to \code{0}.
#'
#' @param Rowv determines if and how the row dendrogram should be reordered.	By default, it is TRUE, which implies dendrogram is computed and reordered based on row means. If NULL or FALSE, then no dendrogram is computed and no reordering is done. If a \link{dendrogram} (or \link{hclust}), then it is used "as-is", ie without any reordering. If a vector of integers, then dendrogram is computed and reordered based on the order of the vector.
#' @param Colv determines if and how the column dendrogram should be reordered.	Has the options as the Rowv argument above and additionally when x is a square matrix, Colv = "Rowv" means that columns should be treated identically to the rows.
#' @param distfun function used to compute the distance (dissimilarity) between both rows and columns. Defaults to dist.
#' @param hclustfun function used to compute the hierarchical clustering when Rowv or Colv are not dendrograms. Defaults to hclust.
#'
#' @param dist_method default is NULL (which results in "euclidean" to be used). Can accept alternative character strings indicating the
#' method to be passed to distfun. By default distfun. is \link{dist} hence
#' this can be one of "euclidean", "maximum", "manhattan", "canberra", "binary" or "minkowski".
#' @param hclust_method default is NULL (which results in "complete" to be used). Can accept alternative character strings indicating the
#' method to be passed to hclustfun By default hclustfun is \link{hclust} hence
#' this can be one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
#'
#' @param distfun_row distfun for row dendrogram only.
#' @param hclustfun_row hclustfun for col dendrogram only.
#' @param distfun_col distfun for row dendrogram only.
#' @param hclustfun_col hclustfun for col dendrogram only.
#'
#' @param dendrogram character string indicating whether to compute 'none',
#' 'row', 'column' or 'both' dendrograms. Defaults to 'both'. However, if Rowv
#' (or Colv) is FALSE or NULL and dendrogram is 'both', then a warning is issued
#' and Rowv (or Colv) arguments are honoured.
#' @param show_dendrogram Logical vector of length controlling whether the row
#' and column dendrograms are displayed. If a logical scalar is
#' provided, it is repeated to become a logical vector of length two.
#' @param reorderfun function(d, w) of dendrogram and weights for reordering the row and column dendrograms. The default uses stats{reorder.dendrogram}
#'
#' @param k_row an integer scalar with the desired number of groups by which to color the dendrogram's branches in the rows (uses \link[dendextend]{color_branches})
#' If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.
#' @param k_col an integer scalar with the desired number of groups by which to color the dendrogram's branches in the columns (uses \link[dendextend]{color_branches})
#' If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.
#'
#' @param symm logical indicating if x should be treated symmetrically; can only be true when x is a square matrix.
#' @param revC logical indicating if the column order should be reversed for plotting.
#' Default (when missing) - is FALSE, unless symm is TRUE.
#' This is useful for cor matrix.
#'
#' @param scale character indicating if the values should be centered and scaled in either the row direction or the column direction, or none. The default is "none".
#' @param na.rm logical indicating whether NA's should be removed.
#'
#' @param digits integer indicating the number of decimal places to be used by \link{round} for 'label'.
#' @param cellnote (optional) matrix of the same dimensions as \code{x}
#' that has the human-readable version of each value,
#' for displaying on top of the heatmap cells.
#'
#' @param cexRow positive numbers. If not missing, it will override \code{xaxis_font_size}
#' and will give it a value cexRow*14
#' @param cexCol positive numbers. If not missing, it will override \code{yaxis_font_size}
#' and will give it a value cexCol*14
#'
#' @param labRow character vectors with row labels to use (from top to bottom); default to rownames(x).
#' @param labCol character vectors with column labels to use (from left to right); default to colnames(x).
#'
#' @param row_side_colors,col_side_colors data.frame of factors to produce
#'    row/column side colors in the style of heatmap.2/heatmap.3.
#'    col_side_colors should be "wide", ie be the same dimensions
#'    as the column side colors it will produce.
#'
#' @param seriate character indicating the method of matrix sorting (default: "OLO").
#' Implemented options include:
#' "OLO" (Optimal leaf ordering, optimizes the Hamiltonian path length that is restricted by the dendrogram structure - works in O(n^4) )
#' "mean" (sorts the matrix based on the reorderfun using marginal means of the matrix. This is the default used by \link[gplots]{heatmap.2}),
#' "none" (the default order produced by the dendrogram),
#' "GW" (Gruvaeus and Wainer heuristic to optimize the Hamiltonian path length that is restricted by the dendrogram structure)
#' @param point_size_mat A matrix of values which can be mapped to point size
#' @param custom_hovertext Custom hovertext matrix (the same dimensions as the input).
#' @param ... currently ignored
#'
#' @export
#' @source
#' The interface was designed based on \link{heatmap}, \link[gplots]{heatmap.2}, and (the also d3heatmap).
#'
#' @seealso
#' \link{heatmap}, \link[gplots]{heatmap.2}
#'
#' @examples
#'
#' \dontrun{
#' library(heatmaply)
#' hm <- heatmapr(mtcars, scale = "column", colors = "Blues")
#' heatmaply(hm)
#' }
#'
heatmapr <- function(x,

                     ## dendrogram control
                     Rowv,
                     Colv,
                     distfun = dist,
                     hclustfun = hclust,
                     dist_method = NULL,
                     hclust_method = NULL,

                     distfun_row,
                     hclustfun_row,
                     distfun_col,
                     hclustfun_col,

                     dendrogram = c("both", "row", "column", "none"),
                     show_dendrogram = c(TRUE, TRUE),
                     reorderfun = function(d, w) reorder(d, w),

                     k_row = 1,
                     k_col = 1,

                     symm = FALSE,
                     revC,

                     ## data scaling
                     scale = c("none", "row", "column"),
                     na.rm = TRUE,

                     labRow = rownames(x),
                     labCol = colnames(x),

                     cexRow,
                     cexCol,

                     ## value formatting
                     digits = 3L,
                     cellnote = NULL,

                     ## TODO: decide later which names/conventions to keep
                     theme = NULL,
                     colors = "RdYlBu",
                     width = NULL, height = NULL,
                     xaxis_height = 80,
                     yaxis_width = 120,
                     xaxis_font_size = NULL,
                     yaxis_font_size = NULL,
                     brush_color = "#0000FF",
                     show_grid = TRUE,
                     anim_duration = 500,
                     row_side_colors,
                     col_side_colors,
                     seriate = c("OLO", "mean", "none", "GW"),
                     point_size_mat = NULL,
                     custom_hovertext = NULL,
                     ...) {

  ## update hclust/dist functions?
  ## ====================
  distfun <- match.fun(distfun)
  if (!is.null(dist_method)) {
    distfun_old <- distfun
    distfun <- function(x) {
      distfun_old(x, method = dist_method)
    }
  }
  if (!is.null(hclust_method)) {
    if (is.na(hclust_method)) {
      hclustfun <- find_dend
    } else {
      hclustfun_old <- hclustfun
      hclustfun <- function(x) {
        hclustfun_old(x, method = hclust_method)
      }
    }
  }

  if (!(is.logical(show_dendrogram) & length(show_dendrogram) %in% c(1, 2))) {
    stop("show_dendrogram must be a logical vector of length 1 or 2")
  }
  if (length(show_dendrogram) == 1) {
    show_dendrogram <- rep(show_dendrogram, 2)
  }

  if (missing(distfun_row)) distfun_row <- distfun
  if (missing(hclustfun_row)) hclustfun_row <- hclustfun
  if (missing(distfun_col)) distfun_col <- distfun
  if (missing(hclustfun_col)) hclustfun_col <- hclustfun

  distfun_row <- match.fun(distfun_row)
  distfun_col <- match.fun(distfun_col)

  ## x is a matrix!
  ## ====================
  if (!is.matrix(x)) {
    x <- as.matrix(x)
  }
  if (!is.matrix(x)) stop("x must be a matrix")


  seriate <- match.arg(seriate)

  nr <- nrow(x)
  nc <- ncol(x)
  ### TODO: debating if to include this or not:
  #   if(nr <= 1 || nc <= 1)
  #     stop("`x' must have at least 2 rows and 2 columns")


  ## Labels for Row/Column
  ## ======================
  rownames(x) <- labRow %||% as.character(1:nrow(x))
  colnames(x) <- labCol %||% as.character(1:ncol(x))


  # fix a case where the row/col names are non unique (this assume the users does NOT supply a dendrogram)
  rownames(x) <- fix_not_all_unique(rownames(x))
  colnames(x) <- fix_not_all_unique(colnames(x))

  if (!missing(cexRow)) {
    if (is.numeric(cexRow)) {
      xaxis_font_size <- cexRow * 14
    } else {
      warning("cexRow is not numeric. It is ignored")
    }
  }
  if (!missing(cexCol)) {
    if (is.numeric(cexCol)) {
      yaxis_font_size <- cexCol * 14
    } else {
      warning("cexCol is not numeric. It is ignored")
    }
  }

  ## Scale the data?
  ## ====================
  scale <- match.arg(scale)

  if (scale == "row") {
    x <- sweep(x, 1, rowMeans(x, na.rm = na.rm))
    x <- sweep(x, 1, apply(x, 1, sd, na.rm = na.rm), "/")
  }
  else if (scale == "column") {
    x <- sweep(x, 2, colMeans(x, na.rm = na.rm))
    x <- sweep(x, 2, apply(x, 2, sd, na.rm = na.rm), "/")
  }

  ## Dendrograms for Row/Column
  ## =======================
  dendrogram <- match.arg(dendrogram)

  # Use dendrogram argument to set defaults for Rowv/Colv
  if (missing(Rowv)) {
    Rowv <- dendrogram %in% c("both", "row")
  }
  if (missing(Colv)) {
    if (dendrogram %in% c("both", "column")) {
      Colv <- if (symm) "Rowv" else TRUE
    } else {
      Colv <- FALSE
    }
  }

  if (isTRUE(Rowv)) {
    Rowv <- create_dend(x, seriate, distfun_row, hclustfun_row, na.rm)
  }
  if (is.numeric(Rowv)) {
    Rowv <- reorderfun(as.dendrogram(hclustfun_row(distfun_row(x))), Rowv)
    Rowv <- rev(Rowv) # I would rather the matrix will be with the first row at the top
  }

  if (is.hclust(Rowv)) Rowv <- as.dendrogram(Rowv)

  if (is.dendrogram(Rowv)) {
    # Rowv <- rev(Rowv)
    rowInd <- order.dendrogram(Rowv)
    if (nr != length(rowInd)) {
      stop("Row dendrogram is the wrong size")
    }
  } else {
    if (!is.null(Rowv) && !is.na(Rowv) && !identical(Rowv, FALSE)) {
      warning("Invalid value for Rowv, ignoring")
    }
    Rowv <- NULL
    rowInd <- 1:nr
  }

  # making the order of the matrix rows comparable with heatmap.2
  Rowv <- rev(Rowv)
  rowInd <- rev(rowInd)

  if (identical(Colv, "Rowv")) {
    # i.e.: if symm=TRUE
    Colv <- Rowv
  }
  if (isTRUE(Colv)) {
    Colv <- create_dend(t(x), seriate, distfun_col, hclustfun_col, na.rm)
  }
  if (is.numeric(Colv)) {
    Colv <- reorderfun(as.dendrogram(hclustfun_col(distfun_col(t(x)))), rev(Colv))
  }

  if (is.hclust(Colv)) Colv <- as.dendrogram(Colv)

  if (is.dendrogram(Colv)) {
    Colv <- rev(Colv)
    colInd <- order.dendrogram(Colv)
    if (nc != length(colInd)) {
      stop("Col dendrogram is the wrong size")
    }
  } else {
    if (!is.null(Colv) && !is.na(Colv) && !identical(Colv, FALSE)) {
      warning("Invalid value for Colv, ignoring")
    }
    Colv <- NULL
    colInd <- 1:nc
  }


  # TODO:  We may wish to change the defaults a bit in the future
  ## revC
  ## =======================
  if (missing(revC)) {
    if (symm) {
      revC <- TRUE
    } else if (is.dendrogram(Colv) & is.dendrogram(Rowv) & identical(Rowv, rev(Colv))) {
      revC <- TRUE
    } else {
      revC <- FALSE
    }
  }
  if (revC) {
    Colv <- rev(Colv)
    colInd <- rev(colInd)
  }

  ## reorder x (and others)
  ## =======================
  if (is.null(cellnote)) cellnote <- x
  if (is.numeric(cellnote)) {
    cellnote <- round(cellnote, digits = digits)
  }
  x <- x[rowInd, colInd, drop = FALSE]
  cellnote <- cellnote[rowInd, colInd, drop = FALSE]
  if (!is.null(point_size_mat)) {
    point_size_mat <- point_size_mat[rowInd, colInd, drop = FALSE]
  }
  if (!is.null(custom_hovertext)) {
    custom_hovertext <- custom_hovertext[rowInd, colInd, drop = FALSE]
  }
  if (!missing(row_side_colors)) {
    if (!(is.data.frame(row_side_colors) | is.matrix(row_side_colors))) {
      row_side_colors <- data.frame("row_side_colors" = row_side_colors)
    }
    assert_that(nrow(row_side_colors) == nrow(x))
    row_side_colors <- row_side_colors[rowInd, , drop = FALSE]
  }
  if (!missing(col_side_colors)) {
    if (!(is.data.frame(col_side_colors) | is.matrix(col_side_colors))) {
      col_side_colors <- data.frame(col_side_colors)
      colnames(col_side_colors) <- "col_side_colors"
    }
    assert_that(nrow(col_side_colors) == ncol(x))
    col_side_colors <- col_side_colors[colInd, , drop = FALSE]
  }

  ## Dendrograms - Update the labels and change to dendToTree
  ## =======================

  # color branches?
  # ----------------
  # Due to the internal working of dendextend, in order to use it we first need
  # to populate the dendextend::dendextend_options() space:
  # if(!missing(k_row) | !missing(k_col))  # Setting k_row and k_col to 1 by default
  dendextend::assign_dendextend_options()

  if (is.dendrogram(Rowv)) {
    if (is.na(k_row)) k_row <- find_k(Rowv)$k

    if (k_row > 1) {
      Rowv <- color_branches(
        Rowv, k = k_row,
        col = k_colors(k_row)
      )
    }
  }
  if (is.dendrogram(Colv)) {
    if (is.na(k_col)) k_col <- find_k(Colv)$k

    if (k_col > 1) {
      Colv <- color_branches(
        Colv, k = k_col,
        col = k_colors(k_col)
      )
    }
  }

  rowDend <- if (is.dendrogram(Rowv) & show_dendrogram[[1]]) Rowv else NULL
  colDend <- if (is.dendrogram(Colv) & show_dendrogram[[2]]) Colv else NULL


  ## cellnote
  ## ====================
  # Check that cellnote is o.k.:
  if (!is.null(cellnote)) {
    if (is.null(dim(cellnote))) {
      if (length(cellnote) != nr * nc) {
        stop("Incorrect number of cellnote values")
      }
      dim(cellnote) <- dim(x)
    }
    if (!identical(dim(x), dim(cellnote))) {
      stop("cellnote matrix must have same dimensions as x")
    }
  }

  ## Final touches before exporting the object
  ## =======================

  # if(!is.null(custom_hovertext) && !is.matrix(custom_hovertext)) {
  if(is.data.frame(custom_hovertext)) {
      custom_hovertext <- as.matrix(custom_hovertext)
  }

  mtx <- list(
    data = as.matrix(x),
    cellnote = cellnote,
    dim = dim(x),
    rows = rownames(x),
    cols = colnames(x),
    point_size_mat = point_size_mat,
    custom_hovertext = custom_hovertext
  )


  if (is.factor(x)) {
    colors <- col_factor(colors, x, na.color = "transparent")
  } else {
    rng <- range(x, na.rm = TRUE)
    if (scale %in% c("row", "column")) {
      rng <- c(max(abs(rng)), -max(abs(rng)))
    }

    colors <- col_numeric(colors, rng, na.color = "transparent")
  }

  # imgUri <- base64enc::encodeAsPNG(t(x), colors)

  options <- NULL

  options <- c(options, list(
    xaxis_height = xaxis_height,
    yaxis_width = yaxis_width,
    xaxis_font_size = xaxis_font_size,
    yaxis_font_size = yaxis_font_size,
    brush_color = brush_color,
    show_grid = show_grid,
    anim_duration = anim_duration
  ))

  if (is.null(rowDend)) {
    c(options, list(yclust_width = 0))
  }
  if (is.null(colDend)) {
    c(options, list(xclust_height = 0))
  }

  heatmapr <- list(
    rows = rowDend, cols = colDend, matrix = mtx,
    theme = theme, options = options
  )

  if (!missing(row_side_colors)) heatmapr[["row_side_colors"]] <- row_side_colors
  if (!missing(col_side_colors)) heatmapr[["col_side_colors"]] <- col_side_colors

  class(heatmapr) <- "heatmapr"

  heatmapr
}




#' @title Is the object of class heatmapr
#' @description Is the object of class heatmapr.
#' @export
#' @param x an object.
#' @return logical - is the object of class heatmapr.
is.heatmapr <- function(x) {
  inherits(x, "heatmapr")
}


create_dend <- function(x, seriate, distfun, hclustfun, na.rm) {
  switch(seriate,
    "mean" = rowMeans(x, na.rm = na.rm),
    "none" = 1:nrow(x),
    "OLO" = {
      dist <- distfun(x)
      hc <- hclustfun(dist)
      dend <- as.dendrogram(hc)
      dend <- seriate_dendrogram(dend, dist, method = "OLO")
      dend
    },
    "GW" = {
      dist <- distfun(x)
      hc <- hclustfun(dist)
      dend <- as.dendrogram(hc)
      dend <- seriate_dendrogram(dend, dist, method = "GW")
      dend
    }
  )
}

context("heatmapr")

test_that("seriate", {
  expect_is(heatmapr(mtcars, seriate = "GW"), "heatmapr")
})

test_that("cellnote", {
  expect_is(heatmapr(mtcars, seriate = "GW", cellnote = mtcars), "heatmapr")
})


test_that("Rowv, Colv", {
  hmapr <- heatmapr(mtcars, Colv = 1:ncol(mtcars), Rowv = 1:ncol(mtcars))
  expect_true(is.heatmapr(hmapr))
})

test_that("symm, revc", {
  expect_is(heatmapr(cor(mtcars), revC = TRUE), "heatmapr")
})

test_that("scale", {
  expect_is(h1 <- heatmapr(cor(mtcars), scale = "row", symm = TRUE), "heatmapr")
  expect_is(h2 <- heatmapr(cor(mtcars), scale = "col", symm = TRUE), "heatmapr")
})

test_that("show_dendrogram error conditions", {
  expect_error(heatmapr(mtcars, show_dendrogram = 1))
  expect_error(heatmapr(mtcars, show_dendrogram = rep(TRUE, 3)))
  expect_error(heatmapr(mtcars, show_dendrogram = NULL))
  expect_error(heatmapr(mtcars, show_dendrogram = TRUE), NA)
  expect_error(heatmapr(mtcars, show_dendrogram = c(TRUE, FALSE)), NA)

})


test_that("create_dend works", {
  for (ser in c("mean", "none", "OLO", "GW")) {
    expect_error(create_dend(as.matrix(mtcars), ser, dist, hclust, TRUE), NA)
    expect_error(create_dend(as.matrix(mtcars), ser, dist, hclust, FALSE), NA)
  }
})

# heatmaply:::ggplot_heatmap(as.matrix(mtcars))
# heatmaply:::plotly_heatmap(as.matrix(mtcars))
# style(plotly_heatmap(as.matrix(mtcars)), xgap = 5, ygap = 5)




#
# library(ggplot2)
# library(plotly)
# # library(heatmaply)
# ggplot_heatmap <- heatmaply:::ggplot_heatmap
# class_to <- function(x, new_class) {
#   class(x) <- new_class
#   x
# }
# na_mat <- function(x) {
#   x %>% is.na %>% class_to("numeric")
# }
#
# p <- heatmaply:::ggplot_heatmap(na_mat(airquality),
#                     scale_fill_gradient_fun = scale_fill_gradientn(colors= c("white","black")) ,
#                     grid_color = "grey", grid_size = 1)
# plot(p)
# ggplotly(p)
# p <- ggplot_heatmap(mtcars,
#                     grid_color = "white")
# p
#


# heatmaply:::ggplot_heatmap(as.matrix(mtcars))


# xx is a data matrix
ggplot_heatmap <- function(xx,
                           row_text_angle = 0,
                           column_text_angle = 45,
                           scale_fill_gradient_fun =
                           scale_fill_gradientn(
                             colors = viridis(
                               n = 256, alpha = 1, begin = 0,
                               end = 1, option = "viridis"
                             ),
                             na.value = "grey50", limits = NULL
                           ),
                           grid_color = NA,
                           grid_size = 0.1,
                           key.title = NULL,
                           layers,
                           row_dend_left = FALSE,
                           label_names = NULL,
                           fontsize_row = 10,
                           fontsize_col = 10,
                           type = c("heatmap", "scatter"),
                           pointsize = 5,
                           point_size_mat = NULL,
                           point_size_name = "Point size",
                           label_format_fun = function(...) format(..., digits = 4),
                           custom_hovertext = NULL,
                           showticklabels = c(TRUE, TRUE),
                           ...) {
  theme_clear_grid_heatmap <- theme(
    axis.line = element_line(color = "black"),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    panel.border = element_blank(),
    panel.background = element_blank()
  )

  type <- match.arg(type)
  # heatmap
  # xx <- x$matrix$data

  df <- xx
  if (!is.data.frame(df)) df <- as.data.frame(df, check.rows = FALSE)

  if (is.null(label_names)) {
    if (is.null(dim_names <- names(dimnames(df)))) {
      label_names <- c("row", "column", "value")
    } else {
      label_names <- dim_names
    }
  } else {
    assert_that(length(label_names) == 3)
  }

  mdf <- melt_df(xx, label_names)
  if (!is.null(point_size_mat)) {
    ps_label_names <- label_names
    ps_label_names[[3]] <- point_size_name
    point_size_mat <- melt_df(point_size_mat, ps_label_names)
    mdf <- cbind(mdf, point_size_mat[point_size_name])
  }
  row <- label_names[[1]]
  col <- label_names[[2]]
  val <- label_names[[3]]

  mdf[["text"]] <- paste0(
    row, ": ", mdf[[1]], "<br>",
    col, ": ", mdf[[2]], "<br>",
    val, ": ", label_format_fun(mdf[[3]])
  )

  if (type == "heatmap") {
    geom <- "geom_tile"
    aes_args <- list(fill = paste_aes(val), text = "text")
    geom_args <- list(
      # mapping = aes_string(fill = paste_aes(val)),
      color = grid_color,
      size = grid_size
    )
  } else if (type == "scatter") {
    geom <- "geom_point"
    geom_args <- list()
    if (!is.null(point_size_mat)) {
      mdf[["text"]] <- paste(
        mdf[["text"]], "<br>",
        point_size_name, ": ", label_format_fun(mdf[[4]])
      )
      aes_args <- list(color = paste_aes(val),
        text = "text",
        size = paste_aes(point_size_name))

      # geom_args[["mapping"]] <- aes_string(
      #   color = paste_aes(val),
      #   text = "text",
      #   size = paste_aes(point_size_name)
      # )
    } else {
      geom_args[["size"]] <- grid_size
      aes_args <- list(color = paste_aes(val),
        text = "text")
      # geom_args[["mapping"]] <- aes_string(color = paste_aes(val), text = "text")
    }
  }
  if (!is.null(custom_hovertext)) {
    mdf[["text"]] <- paste0(mdf[["text"]], "<br>", custom_hovertext)
  }
  geom_args[["mapping"]] <- do.call(aes_string, aes_args)

  # TODO:
  # http://stackoverflow.com/questions/15921799/draw-lines-around-specific-areas-in-geom-tile
  # https://cran.r-project.org/web/packages/viridis/vignettes/intro-to-viridis.html
  p <- ggplot(mdf, aes_string(x = paste_aes(col), y = paste_aes(row))) +
    ## Using the text aes produces a warning... Not ideal!
    suppressWarnings(do.call(geom, geom_args)) +
    scale_fill_gradient_fun +
    theme_bw() +
    theme_clear_grid_heatmap +
    theme(
      axis.title = element_blank(),
      axis.text.x = if (showticklabels[[1]]) {
        element_text(angle = column_text_angle, size = fontsize_col, hjust = 1)
      } else element_blank(),
      axis.ticks.x = if (showticklabels[[1]]) element_line() else element_blank(),
      axis.text.y = if (showticklabels[[2]]) {
        element_text(angle = row_text_angle, size = fontsize_row, hjust = 1)
      } else element_blank(),
      axis.ticks.y = if (showticklabels[[2]]) element_line() else element_blank(),
    )

  if (type == "scatter") {
    p <- p +
      coord_cartesian(xlim = c(1, ncol(xx)), ylim = c(1, nrow(xx)))
  } else {
    p <- p + coord_cartesian(expand = FALSE)
  }
  ## Passed in to allow users to alter (courtesy of GenVisR)
  if (!missing(layers)) {
    p <- p + layers
  }

  # p <- p + scale_x_discrete(limits = unique(mdf))
  # http://stats.stackexchange.com/questions/5007/how-can-i-change-the-title-of-a-legend-in-ggplot2
  p <- p + labs(fill = key.title)

  # until this bug is fixed: https://github.com/ropensci/plotly/issues/699
  # we are forced to use geom_hline and geom_vline
  if (!is.na(grid_color)) {
    p <- p + geom_hline(yintercept = c(0:nrow(xx)) + 0.5, color = grid_color) # , size = grid_size # not implemented since it doesn't work with plotly
    p <- p + geom_vline(xintercept = c(0:ncol(xx)) + 0.5, color = grid_color) # , size = grid_size # not implemented since it doesn't work with plotly
  }

  if (row_dend_left) {
    p <- p + scale_y_discrete(position = "right") # possible as of ggplot 2.1.0 !
  }

  p
}

melt_df <- function(x, label_names) {
  # heatmap
  # xx <- x$matrix$data
  df <- as.data.frame(x)

  row <- label_names[[1]]
  col <- label_names[[2]]
  val <- label_names[[3]]

  # colnames(df) <- x$matrix$cols
  if (!is.null(rownames(x))) {
    df[[row]] <- rownames(x)
  } else {
    df[[row]] <- 1:nrow(x)
  }

  df[[row]] <- factor(
    df[[row]],
    levels = df[[row]],
    ordered = TRUE
  )

  mdf <- reshape2::melt(df, id.vars = row)
  colnames(mdf)[2:3] <- c(col, val) # rename "variable" and "value"
  mdf
}

paste_aes <- function(x) {
  paste0("`", x, "`")
}

plotly_heatmap <- function(x,
                           limits = range(x),
                           colors = viridis(n = 256, alpha = 1, begin = 0, end = 1, option = "viridis"),
                           row_text_angle = 0,
                           column_text_angle = 45,
                           grid.color,
                           grid.size,
                           row_dend_left = FALSE,
                           fontsize_row = 10,
                           fontsize_col = 10,
                           key_title = "",
                           colorbar_xanchor = "left",
                           colorbar_yanchor = "bottom",
                           label_names = NULL,
                           colorbar_xpos = 1.1,
                           colorbar_ypos = 1,
                           colorbar_len = 0.3,
                           colorbar_thickness = 30,
                           custom_hovertext = NULL,
                           point_size_mat = NULL,
                           point_size_name = "Point size",
                           showticklabels = c(TRUE, TRUE)) {

  if (is.function(colors)) colors <- colors(256)

  if (is.null(label_names)) {
    if (is.null(dim_names <- names(dimnames(x)))) {
      label_names <- c("row", "column", "value")
    } else {
      label_names <- dim_names
    }
  } else {
    assert_that(length(label_names) == 3)
  }

  if (!is.null(custom_hovertext)) {
    text_mat <- custom_hovertext
  } else {
    text_mat <- as.data.frame(x)
    text_mat[] <- lapply(
      seq_along(text_mat),
      function(i) {
        paste0(
          label_names[1], ": ", rownames(x), "<br>",
          label_names[2], ": ", colnames(x)[i], "<br>",
          label_names[3], ": ", x[, i]
        )
      }
    )
    text_mat <- as.matrix(text_mat)
    if (!is.null(point_size_mat)) {
      point_size_name
    }
  }

  if (is.null(point_size_mat)) {
    p <- plot_ly(
      z = x, x = 1:ncol(x), y = 1:nrow(x), text = text_mat,
      type = "heatmap",
      showlegend = FALSE,
      colors = colors,
      hoverinfo = "text",
      zmin = limits[1], zmax = limits[2]
    )
  } else {

    melt <- function(x, cn=colnames(x), rn=rownames(x)) {
      xdf <- reshape2::melt(x)
      xdf$Var1 <- factor(xdf$Var1, levels = rn)
      xdf$Var2 <- factor(xdf$Var2, levels = cn)
      xdf
    }
    xdf <- melt(x)
    tdf <- melt(text_mat)
    pdf <- melt(point_size_mat)

    p <- plot_ly(
      x = as.numeric(xdf$Var1),
      y = as.numeric(xdf$Var2),
      text = tdf$value,
      color = xdf$value,
      size = pdf$value,
      type = "scatter",
      mode = "markers",
      showlegend = FALSE,
      colors = colors,
      hoverinfo = "text"
    )
  }
  p <- p  %>%
    layout(
      xaxis = list(
        tickfont = list(size = fontsize_col),
        tickangle = column_text_angle,
        tickvals = 1:ncol(x), ticktext = colnames(x),
        linecolor = "#ffffff",
        range = c(0.5, ncol(x) + 0.5),
        showticklabels = showticklabels[[1]]
      ),
      yaxis = list(
        tickfont = list(size = fontsize_row),
        tickangle = row_text_angle,
        tickvals = 1:nrow(x), ticktext = rownames(x),
        linecolor = "#ffffff",
        range = c(0.5, nrow(x) + 0.5),
        showticklabels = showticklabels[[2]]
      )
    )
  p <- plotly::colorbar(
    p,
    lenmode = "fraction",
    title = key_title,
    xanchor = colorbar_xanchor,
    x = colorbar_xpos,
    y = colorbar_ypos,
    yanchor = colorbar_yanchor,
    len = colorbar_len,
    thickness = colorbar_thickness
  )
  p
}


plotly_dend <- function(dend,
                        side = c("row", "col"),
                        flip = FALSE,
                        dend_hoverinfo = TRUE) {

  if (is.hclust(dend)) {
    dend <- as.dendrogram(dend)
  }

  side <- match.arg(side)
  dend_data <- as.ggdend(dend)
  segs <- dend_data$segments

  ## Have to get colors back from dendrogram otherwise plotly will make some up
  if (is.null(segs$col) || all(is.na(segs$col))) {
    segs$col <- rep(1, length(segs$col))
  }
  segs$col[is.na(segs$col)] <- "black" # default value for NA is "black"

  if (is.numeric(segs$col)) {
    segs$col <- factor(segs$col)
  }

  ## Need to somehow convert to colors that plotly will understand
  colors <- sort(unique(segs$col))
  if (is.numeric(colors)) {
    colors <- col2hex(grDevices::palette()[seq_along(colors)])
  }

  lab_max <- nrow(dend_data$labels)
  if (side == "row") {
    lab_max <- lab_max + 0.5
  }

  axis1 <- list(
    title = "",
    range = c(0, max(segs$y)),
    linecolor = "#ffffff",
    showgrid = FALSE
  )
  axis2 <- list(
    title = "",
    range = c(0, lab_max),
    linecolor = "#ffffff",
    showgrid = FALSE
  )

  ## Have to change x and y depending on which orientation
  if (side == "row") {
    add_plot_lines <- function(p) {
      p %>%
        add_segments(
          x = ~y, xend = ~yend, y = ~x, yend = ~xend, color = ~col,
          showlegend = FALSE,
          colors = colors,
          hoverinfo = if (dend_hoverinfo) "x" else "none"
        ) %>%
        layout(
          hovermode = "closest",
          xaxis = axis1,
          yaxis = axis2
        )
    }
  }
  else {
    add_plot_lines <- function(p) {
      p %>%
        add_segments(
          x = ~x, xend = ~xend, y = ~y, yend = ~yend, color = ~col,
          showlegend = FALSE,
          colors = colors,
          hoverinfo = if (dend_hoverinfo) "y" else "none"
        ) %>%
        layout(
          hovermode = "closest",
          xaxis = axis2,
          yaxis = axis1
        )
    }
  }

  p <- plot_ly(segs) %>% add_plot_lines()

  if (flip) {
    p <- layout(p, xaxis = list(autorange = "reversed"))
  }
  p
}

#' @title Side color plots for heatmaps
#' @description
#' Important for creating annotation.
#'
#' @param df A "molten" data.frame as produced by (eg) reshape2::melt
#' @param palette A function which can return colors to be used in the sidebar
#' plot
#' @param scale_title Title of the color scale. Not currently used.
#' @param type Horizontal or vertical plot? Valid values are "column" and "row"
#' @param text_angle the angle of the text of the rows/columns.
#' @param is_colors Use if the values in df are valid colours and should not be mapped
#'  to a color scheme, and instead should be plotted directly.
#' @param fontsize Font size (currently unused)
#' @param label_name Name for the mouseover label, usually "row" or "column"
#'
#' @return A ggplot geom_tile object
#'
ggplot_side_color_plot <- function(df,
                                   palette = NULL,
                                   scale_title = paste(type, "side colors"),
                                   type = c("column", "row"),
                                   text_angle = if (type == "column") 0 else 90,
                                   is_colors = FALSE,
                                   fontsize = 10,
                                   label_name = NULL) {

  type <- match.arg(type)
  if (is.matrix(df)) df <- as.data.frame(df)
  assert_that(is.data.frame(df))

  if (is.null(label_name)) label_name <- type

  ## Cooerce to character
  df[] <- lapply(df, as.character)

  ## TODO: Find out why names are dropped when ncol(df) == 1 (Not any more?)
  original_dim <- dim(df)

  if (is.null(palette)) palette <- default_side_colors

  ## Custom label
  if (type %in% colnames(df)) {
    stop("Having", type, "in the colnames of the side_color df will drop data!")
  }

  df[[type]] <- if (!is.null(rownames(df))) rownames(df) else 1:nrow(df)

  df[[type]] <- factor(df[[type]], levels = df[[type]], ordered = TRUE)
  df <- reshape2::melt(df, id.vars = type)
  df[["value"]] <- factor(df[["value"]])

  id_var <- colnames(df)[1]

  common_theme <- theme(
    panel.background = element_blank(),
    axis.ticks = element_blank()
  )

  ## Don't need this hack any more?
  # if(original_dim[2] > 1) {
  text_element <- element_text(angle = text_angle, hjust = 1, size = fontsize)
  # } else text_element <- element_blank()

  if (type == "column") {
    mapping <- aes_string(x = paste_aes(id_var), y = "variable", fill = "value")
    specific_theme <- theme(
      axis.text.x = element_blank(),
      axis.text.y = text_element
    )
  } else {
    mapping <- aes_string(x = "variable", y = paste_aes(id_var), fill = "value")
    specific_theme <- theme(
      axis.text.x = text_element,
      axis.text.y = element_blank(),
      legend.position = "top"
    )
  }
  theme <- list(common_theme, specific_theme)

  if (is_colors) {
    palette <- setNames(unique(df[["value"]]), unique(df[["value"]]))
  }

  if (is.function(palette)) {
    palette <- setNames(palette(nlevels(df[["value"]])), levels(df[["value"]]))
  } else if (!all(levels(factor(df[["value"]])) %in% names(palette))) {
    stop(paste0(
      "Not all levels of the ", type,
      "_side_colors are mapped in the ", type, "_side_palette"
    ))
  }

  g <- ggplot(df, mapping = mapping) +
    geom_tile() +
    xlab("") +
    ylab("") +
    scale_fill_manual(
      name = scale_title,
      breaks = levels(df[["value"]]),
      values = as.character(palette[levels(df[["value"]])])
    ) +
    theme
  return(g)
}



default_side_colors <- function(n) {
  ## Warning if n < 3 in any case
  suppressWarnings(
    if (n <= 12) {
      RColorBrewer::brewer.pal(n, "Paired")[seq_len(n)]
    } else if (n <= 20) {
      c(
        RColorBrewer::brewer.pal(12, "Paired"),
        RColorBrewer::brewer.pal(n - 12, "Set2")
      )[seq_len(n)]
    } else if (n <= 32) {
      c(
        RColorBrewer::brewer.pal(12, "Paired"),
        RColorBrewer::brewer.pal(8, "Set2"),
        RColorBrewer::brewer.pal(n - 20, "Set3")
      )[seq_len(n)]
    } else {
      colorspace::rainbow_hcl(n)
    }
  )
}

## Predict luminosity of cells and change text based on that
## http://stackoverflow.com/questions/12043187/how-to-check-if-hex-color-is-too-black
predict_colors <- function(p,
                           colorscale_df = p$x$data[[1]]$colorscale %||% p$x$data[[2]]$marker$colorscale,
                           cell_values = p$x$data[[1]]$z,
                           plot_method = c("ggplot", "plotly")) {

  plot_method <- match.arg(plot_method)

  if (is.null(cell_values)) {
    cell_values <- as.data.frame(
      matrix(
        p$x$data[[1]]$marker$size,
        ncol = length(unique(p$x$data[[1]]$x)),
        nrow = length(unique(p$x$data[[1]]$y))
      )
    )
  }
  if (is.null(colorscale_df)) {
    stop("Internal error in predicting colours!")
  }
  cell_values <- as.data.frame(cell_values)
  cell_values$row <- seq_len(nrow(cell_values))
  cell_values_m <- reshape2::melt(cell_values, id.vars = "row")
  cell_values_vector <- cell_values_m$value

  ## Need to normalise to (0, 1) scale as this is what plotly
  ## uses internally
  if (plot_method == "plotly") {
    ## Need to convert plotly colors to hex colors
    colorscale_df[, 2] <- parse_plotly_color(colorscale_df[, 2])

    cell_values_vector <- normalize(as.numeric(cell_values_vector))
    ## interpolate to 256 colors because that's probably enough
    colorscale_df <- data.frame(
      stats::approx(as.numeric(colorscale_df[, 1]), n = 256)$y,
      grDevices::colorRampPalette(colorscale_df[, 2])(256)
    )

    ## Then need to sort, find nearest neighbour, and map across
    cell_values_vector_sort <- sort(cell_values_vector)
    nearest_neighbours <- sapply(
      cell_values_vector_sort,
      function(val) {
        max(colorscale_df[as.numeric(colorscale_df[, 1]) <= val, 1])
      }
    )
    names(nearest_neighbours) <- cell_values_vector_sort
    cell_values_vector <- nearest_neighbours[as.character(cell_values_vector)]
  }

  cell_values_vector <- as.character(cell_values_vector)
  ind <- match(cell_values_vector, colorscale_df[, 1])
  cell_colors <- unlist(colorscale_df[ind, 2])

  # apply colors only to non-NA cells
  # In the future, it might be worth using na.color
  cell_colors <- as.character(cell_colors)
  cell_colors[is.na(cell_colors)] <- "#ffffff" # make the default white for NA values

  cell_colors_rgb <- colorspace::hex2RGB(cell_colors)
  cell_font_colors <- sapply(
    seq_len(nrow(cell_colors_rgb@coords)),
    function(i) {
      col <- cell_colors_rgb@coords[i, ]
      luma <- (0.2126 * col[1]) +
        (0.7152 * col[2]) +
        (0.0722 * col[3])
      ifelse(luma < 0.4, "white", "black")
    }
  )
  cell_font_colors
}


parse_plotly_color <- function(color) {
  r <- gsub(
    "rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
    "\\1", color
  )
  g <- gsub(
    "rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
    "\\2", color
  )
  b <- gsub(
    "rgb[a]?\\((\\d+),(\\d+),(\\d+)(,\\d+)?)",
    "\\3", color
  )
  rgb(r, g, b, maxColorValue = 255)
}


## Helper function to generate "normal" colors for dendrograms
## ie black if one k or rainbow_hcl otherwise
k_colors <- function(k) {
  if (k > 1) {
    colorspace::rainbow_hcl(k)
  } else {
    "black"
  }
}


# Create a plotly colorscale from a list of colors in any format.
discrete_colorscale <- function(colors) {
  colors <- rep(colors, each = 100)
  seq <- seq(0, 1, length.out = length(colors))
  setNames(data.frame(seq, colors), NULL)
}

#' @importFrom stats setNames
plotly_side_color_plot <- function(df,
                                   palette = NULL,
                                   scale_title = paste(type, "side colors"),
                                   type = c("column", "row"),
                                   text_angle = if (type == "column") 0 else 90,
                                   is_colors = FALSE,
                                   label_name = NULL,
                                   colorbar_len = 0.3,
                                   fontsize = 10,
                                   show_legend = TRUE) {

  type <- match.arg(type)

  if (is.null(label_name)) label_name <- type

  data <- df
  data[] <- lapply(df, factor)
  # if (type == "column") {
  #   data <- t(data)
  # }
  data <- as.data.frame(data, stringsAsFactors = TRUE)
  data_vals <- unlist(data)
  levels <- levels(data_vals)
  levels <- levels[!is.na(levels)]

  if (is.null(palette)) palette <- default_side_colors

  if (is.function(palette)) {
    palette <- setNames(palette(length(levels)), levels)
  } else {
    palette <- setNames(col2hex(palette), names(palette))
    if (!all(levels %in% names(palette))) {
      stop(paste0(
        "Not all levels of the ", type,
        "_side_colors are mapped in the ", type, "_side_palette"
      ))
    }
  }

  levs2colors <- palette[as.character(levels)]
  levs2nums <- setNames(seq_along(levels), levels)

  df_nums <- data
  df_nums[] <- lapply(
    data,
    function(col) as.numeric(levs2nums[as.character(col)])
  )
  df_nums <- as.matrix(df_nums)
  if (type == "column") {
    df_nums <- t(df_nums)
  }
  if (ncol(df) == 1) {
    key_title <- colnames(df)
  } else {
    key_title <- paste(
      gsub("^(\\w)", "\\U\\1", type, perl = TRUE), "annotation"
    )
  }

  text_mat <- data
  text_mat[] <- lapply(
    seq_along(text_mat),
    function(i) {
      if (type == "row") {
        paste0(
          "value: ", data[, i], "<br>",
          "variable: ", colnames(data)[i], "<br>",
          label_name, ": ", rownames(data)
        )
      } else {
        paste0(
          "value: ", data[, i], "<br>",
          label_name, ": ", colnames(data)[i], "<br>",
          "variable: ", rownames(data)
        )
      }
    }
  )

  ## Ensure tickvals are in right position when n = 2
  offset <- ((length(levels) - 1) / length(levels)) / 2

  ## https://stackoverflow.com/questions/42524450/using-discrete-custom-color-in-a-plotly-heatmap
  p <- plot_ly(
    z = df_nums, x = 1:ncol(df_nums), y = 1:nrow(df_nums),
    text = as.matrix(text_mat), hoverinfo = "text",
    type = "heatmap", showlegend = FALSE, colors = levs2colors,
    colorscale = discrete_colorscale(levs2colors),
    colorbar = list(
      # Capitalise first letter
      title = key_title,
      tickmode = "array",
      ## Issue #137
      tickvals = seq(
        1 + offset,
        length(levels) - offset,
        length.out = length(levels)
      ),
      ticktext = levels,
      len = colorbar_len
    )
  )
  if (type == "row") {
    p <- p %>% layout(
      xaxis = list(
        tickfont = list(size = fontsize),
        tickangle = text_angle,
        tickvals = 1:ncol(df_nums), ticktext = colnames(df_nums),
        linecolor = "#ffffff",
        range = c(0.5, ncol(df_nums) + 0.5),
        showticklabels = TRUE
      ),
      yaxis = list(showticklabels = FALSE)
    )
  } else {
    p <- p %>% layout(
      yaxis = list(
        tickfont = list(size = fontsize),
        tickangle = text_angle,
        tickvals = 1:nrow(df_nums), ticktext = rownames(df_nums),
        linecolor = "#ffffff",
        range = c(0.5, nrow(df_nums) + 0.5),
        showticklabels = TRUE
      ),
      xaxis = list(showticklabels = FALSE)
    )
  }
  p
}





#' @import webshot
NULL
# just so to have an excuse for why webshot is in import (the real reason is that plotly has it as suggests while it is used there by plotly::export)
# webshot <- webshot::webshot



# TODO: add roxydoc

# This function gets a heatmaply object and a file, and writes that to a file
# we should later add control over width...
hmly_to_file_1file <- function(hmly, file, width = NULL, height = NULL, ...) {

  # tools::file_ext("hithere.png")
  # tools::file_ext("hithere.html")
  # tools::file_ext("hithere.html.png")
  file_extension <- tolower(tools::file_ext(file))
  if (!(file_extension %in% c("html", "pdf", "png", "jpeg"))) {
    stop(paste(
      "file extension must be one of the following:",
      "\"html\", \"pdf\", \"png\", or \"jpeg\".",
      "Since it was not, your heatmaply was not saved to a file."
    ))
  } else {
    if (file_extension == "html") {
      # solution to dealing with the folder:
      # https://stackoverflow.com/questions/41399795/savewidget-from-htmlwidget-in-r-cannot-save-html-file-in-another-folder
      tmp_fp <- file
      tmp_fp <- file.path(normalizePath(dirname(tmp_fp)), basename(tmp_fp))
      hmly %>% htmlwidgets::saveWidget(file = tmp_fp, selfcontained = TRUE)
    }
    if (file_extension %in% c("pdf", "png", "jpeg")) {
      if (is.null(width)) {
        width <- size_default(file_extension, "width")
      }
      if (is.null(height)) {
        height <- size_default(file_extension, "height")
      }
      tryCatch(
        export_orca(hmly, file, width, height),
        error = function(e) {
          warning <- paste("plotly::orca failed:\n", e)
          warning(warning)
          export(hmly, file, width, height)
        }
      )
    }
  }
  invisible(hmly)
}

export_orca <- function(x, file, width, height) {
  plotly::orca(
    x,
    file = file,
    width = width,
    height = height
  )
}

export <- function(x, file, width, height) {
  plotly::export(
    x,
    file = file,
    vwidth = width,
    vheight = height,
    cliprect = "viewport"
  )
}

size_default <- function(file_extension, direction=c("width", "height")) {
  direction <- match.arg(direction)
  ## webshot uses viewport size in pixels to control file size, so
  ## all sizes in pixels
  # switch(direction,
  #   "width" = if (file_extension %in% bitmap_types) 800 else 8,
  #   "height" = if (file_extension %in% bitmap_types) 500 else 5
  # )
  switch(direction,
    "width" = 800,
    "height" = 500
  )
}


bitmap_types <- c("png", "jpeg")

hmly_to_file <- Vectorize(hmly_to_file_1file, vectorize.args = "file")

## Copied from gplots
#' @importFrom grDevices col2rgb
col2hex <- function(col) {
    colMat <- col2rgb(col)
    rgb(red = colMat[1, ] / 255,
        green = colMat[2, ] / 255,
        blue = colMat[3, ] / 255)
}

context("plots")

expect_ggplotly <- function(g) {
  expect_is(g, "ggplot")
  expect_error(ggplot_build(g), NA)
  expect_error(ggplotly(g), NA)
}

iris_plot <- iris[, -5]
iris_category <- iris[, 5, drop = FALSE]

test_that("ggplot_heatmap works", {
  g <- heatmaply:::ggplot_heatmap(as.matrix(iris_plot))
  expect_ggplotly(g)

  g <- heatmaply:::ggplot_heatmap(as.matrix(iris_plot), node_type = "scatter")
  expect_ggplotly(g)

  g <- heatmaply:::ggplot_heatmap(
    as.matrix(iris_plot),
    label_names = c("a", "b", "c")
  )
  expect_ggplotly(g)
  expect_error(heatmaply:::ggplot_heatmap(
    as.matrix(iris_plot),
    label_names = c("a")
  ))
})

test_that("plotly_heatmap works", {
  p <- heatmaply:::plotly_heatmap(as.matrix(iris_plot))
  expect_is(p, "plotly")
})

test_that("plotly_dend works", {
  hc <- hclust(dist(iris_plot))
  p <- heatmaply:::plotly_dend(hc)
  expect_is(p, "plotly")
})

test_that("ggplot_side_color_plot works", {
  g <- heatmaply:::ggplot_side_color_plot(
    iris_category,
    type = "row"
  )
  expect_ggplotly(g)
  g <- heatmaply:::ggplot_side_color_plot(
    iris_category,
    type = "column"
  )
  expect_ggplotly(g)
})

test_that("plotly_side_color_plot works", {
  p <- heatmaply:::plotly_side_color_plot(
    iris_category,
    type = "row"
  )
  expect_is(p, "plotly")
  expect_error(ggplotly(p), NA)
  p <- heatmaply:::plotly_side_color_plot(
    iris_category,
    type = "column"
  )
  expect_is(p, "plotly")
  expect_error(plotly_build(p), NA)
})

test_that("predict_colors works", {
  expect_error(heatmaply:::predict_colors("#ffffff"))
  p <- heatmaply:::plotly_heatmap(as.matrix(iris_plot))
  g <- heatmaply:::ggplot_heatmap(as.matrix(iris_plot))
  expect_ggplotly(g)
  expect_error(heatmaply:::predict_colors(p, plot_method = "plotly"), NA)
  expect_error(
    heatmaply:::predict_colors(p, plot_method = "plotly"),
    NA
  )
  expect_error(
    heatmaply:::predict_colors(ggplotly(g), plot_method = "ggplot"),
    NA
  )
})

test_that("parse_plotly_color works", {
  expect_equal(
    heatmaply:::parse_plotly_color("rgb(1,1,1,0)"),
    heatmaply:::parse_plotly_color("rgb(1,1,1)"),
    "#010101"
  )
  expect_equal(
    toupper(heatmaply:::parse_plotly_color("rgb(255,255,255)")),
    "#FFFFFF"
  )
  expect_equal(
    heatmaply:::parse_plotly_color("rgb(0,0,0)"),
    "#000000"
  )
})

test_that("k_colors works", {
  expect_equal(heatmaply:::k_colors(1), "black")
  expect_equal(heatmaply:::k_colors(5), colorspace::rainbow_hcl(5))
})

test_that("default colors works", {
  for (i in 1:50) {
    expect_equal(length(heatmaply:::default_side_colors(i)), i)
  }
})

test_that("numeric cols on dendrograms", {
  dend <- as.dendrogram(hclust(dist(mtcars)))
  dend <- dendextend::set(dend, "branches_k_color", 1:32)
  expect_silent(heatmaply:::plotly_dend(dend))
})

test_that("side colors fail when bad palette", {
  expect_error(
    heatmaply:::ggplot_side_color_plot(iris[, 5, drop = FALSE]),
    NA
  )
  expect_error(
    heatmaply:::ggplot_side_color_plot(
      iris[, 5, drop = FALSE],
      palette = setNames(
        RColorBrewer::brewer.pal(3, "Set3"),
        unique(iris[, 5])
      )
    ),
    NA
  )
  expect_error(
    heatmaply:::ggplot_side_color_plot(
      iris[, 5, drop = FALSE],
      palette = c("Nope" = "Nope")
    )
  )

  expect_error(
    heatmaply:::plotly_side_color_plot(iris[, 5, drop = FALSE]),
    NA
  )
  expect_error(
    heatmaply:::plotly_side_color_plot(
      iris[, 5, drop = FALSE],
      palette = setNames(
        RColorBrewer::brewer.pal(3, "Set3"),
        unique(iris[, 5])
      )
    ),
    NA
  )
  expect_error(
    heatmaply:::plotly_side_color_plot(
      iris[, 5, drop = FALSE],
      palette = c("Nope" = "Nope")
    )
  )
})

test_that("plotly dend uses colors", {
  dend <- hclust(dist(mtcars))
  dend <- dendextend::color_branches(dend, k = 2)
  p <- heatmaply:::plotly_dend(dend)
  expect_is(p, "plotly")
  dend <- dendextend::color_branches(dend, k = 2, col = c(1, 2))
  p <- heatmaply:::plotly_dend(dend)
  expect_is(p, "plotly")
})

#' ggplot heatmap equivalent to heatmaply
#'
#' This function produces a ggplot analogue of heatmaply figures
#' using \link[egg]{ggarrange}. This function may not always support the same
#' set of features as , and exporting the heatmaply object with, for example,
#' \link[plotly]{orca} or \code{heatmaply(mtcars, file = "foo.png")}.
#'
#' @param ... Passed to \link{heatmaply}
#' @param widths,heights Relative widths and heights of plots.
#' @param row_dend_left Logical argument controlling whether the row
#'  dendrogram is placed on the left of the plot.
#' @param hide_colorbar Logical argument controlling whether the color bar (i.e.:
#'   the legend) is hidden.
#' @examples
#' ggheatmap(mtcars)
#' @export
ggheatmap <- function(..., widths = NULL, heights = NULL,
                      row_dend_left = FALSE, hide_colorbar = FALSE) {
  plots <- heatmaply(
    ...,
    row_dend_left = row_dend_left,
    hide_colorbar = hide_colorbar,
    return_ppxpy = TRUE,
    plot_method = "ggplot"
  )
  arrange_plots(
    plots,
    widths = widths,
    heights = heights,
    row_dend_left = row_dend_left,
    hide_colorbar = hide_colorbar
  )
}


## TODO: duplication with heatmap_subplot_from_ggplotly
arrange_plots <- function(
    plots,
    widths = NULL,
    heights = NULL,
    row_dend_left = FALSE,
    hide_colorbar = FALSE) {

  plots <- plots[!sapply(plots, is.null)]
  if (!row_dend_left) {
    plots$p <- plots$p + theme(legend.position = "left")
  }
  if (hide_colorbar) {
    plots$p <- plots$p + theme(legend.position = "none")
  }
  plots <- lapply(plots, function(x) x + theme(plot.margin = unit(c(0, 0, 0, 0), "npc")))

  column_list <- list(plots$py, plots$pc, plots$p)
  ind_null_col <- sapply(column_list, is.null)

  row1_list <- list(plots$py, ggplot_empty(), ggplot_empty())
  row2_list <- list(plots$pc, ggplot_empty(), ggplot_empty())
  row3_list <- list(plots$p, plots$pr, plots$px)

  if (row_dend_left) {
    row3_list <- rev(row3_list)
    row2_list <- rev(row2_list)
    row1_list <- rev(row1_list)
  }
  plotlist <- c(
    row1_list,
    row2_list,
    row3_list
  )

  nrows <- sum(!ind_null_col)
  ind_remove_col <- rep(ind_null_col, each = length(plotlist) / 3)

  ind_null_row <- sapply(row3_list, is.null)
  ncols <- sum(!ind_null_row)
  ind_remove_row <- rep(ind_null_row, length.out = length(plotlist))
  plotlist <- plotlist[!(ind_remove_row | ind_remove_col)]

  widths <- widths %||% default_dims(plots$px, plots$pr)
  if (row_dend_left) {
    widths <- rev(widths)
  }
  egg::ggarrange(
    plots = plotlist,
    ncol = ncols,
    widths = widths,
    heights = heights %||% rev(default_dims(plots$py, plots$pc))
  )
}

ggplot_empty <- function() {
  ggplot() + theme_void() + theme(plot.margin = unit(c(0, 0, 0, 0), "npc"))
}

context("ggheatmap")

test_that("ggheatmap works", {
  g <- ggheatmap(mtcars, dendrogram = "none")
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, dendrogram = "row")
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, dendrogram = "column")
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, row_dend_left = TRUE)
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, dendrogram = "row", row_dend_left = TRUE)
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, row_side_colors = mtcars[, 2])
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, col_side_colors = t(mtcars[1, ]))
  expect_is(g, "egg")
  g <- ggheatmap(mtcars, row_side_colors = mtcars[, 2], col_side_colors = t(mtcars[1, ]))
  expect_is(g, "egg")
})

# Copyright (C) Tal Galili
#
# This file is part of heatmaply.
#
# heatmaply is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# heatmaply is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
#  A copy of the GNU General Public License is available at
#  http://www.r-project.org/Licenses/
#


# library(RColorBrewer)
#
# The sequential palettes names are
# Blues BuGn BuPu GnBu Greens Greys Oranges OrRd PuBu PuBuGn PuRd Purples RdPu Reds YlGn YlGnBu YlOrBr YlOrRd
#
# All the sequential palettes are available in variations from 3 different values up to 9 different values.
#
# The diverging palettes are
# BrBG PiYG PRGn PuOr RdBu RdGy RdYlBu RdYlGn Spectral
#
# All the diverging palettes are available in variations from 3 different values up to 11 different values.
#




# importFrom("grDevices", "rgb")
# importFrom("methods", "as")


#' @importFrom grDevices rgb
#' @importFrom methods as






#' @title RColorBrewer color Ramp Palette
#' @name RColorBrewer_colors
#'
#' @description
#' Functions for getting the colors of RColorBrewer (i.e.: \link{brewer.pal}) without the limitation of only 9/11
#' color values, based on \link{colorRampPalette}.
#'
#' For sequential palettes this is not essential since we have \link{viridis}. But for diverging palettes
#' this is quit essential.
#'
#' The sequential palettes names are
#' Blues BuGn BuPu GnBu Greens Greys Oranges OrRd PuBu PuBuGn PuRd Purples RdPu Reds YlGn YlGnBu YlOrBr YlOrRd
#'
#' The diverging palettes are
#' BrBG PiYG PRGn PuOr RdBu RdGy RdYlBu RdYlGn Spectral
#' And also cool_warm. The cool_warm palette is based on Kenneth Moreland's proposal (see ref).
#' It goes from blue (cool) to ref (warm), based on well thought-out design elements.
#'
#' @references
#' * Moreland, Kenneth. "Diverging color maps for scientific visualization." Advances in Visual Computing (2009): 92-103.
#' url: http://www.kennethmoreland.com/color-maps/
#' The code was provided here: http://stackoverflow.com/a/44073011/256662
#' Thanks to the user YAK, who relied on the code from the Rgnuplot package
#' (which is duplicated here, in order to save the need to import the entire package)
#'
#'
#'
#' @param n the number of colors (>= 1) to be in the palette.
#'
#' @return
#' A character vector of color names.
#'
#' @examples
#' \dontrun{
#'
#' library(RColorBrewer)
#' display.brewer.all(n=11,type="div"); title(main = "Divergent color palette")
#' display.brewer.all(n=9,type=c("seq")); title(main = "Sequential color palette")
#'
#'
#'
#' img <- function(obj, nam) {
#'   image(1:length(obj), 1, as.matrix(1:length(obj)), col=obj,
#'         main = nam, ylab = "", xaxt = "n", yaxt = "n",  bty = "n")
#' }
#'
#' par(mfrow = c(10,1))
#' img(rev(cool_warm(500)), "cool_warm, (Moreland 2009)")
#' img(RdBu(500), "RdBu")
#' img(BrBG(500), "BrBG")
#' img(PiYG(500), "PiYG")
#' img(PRGn(500), "PRGn")
#' img(PuOr(500), "PuOr")
#' img(RdGy(500), "RdGy")
#' img(RdYlBu(500), "RdYlBu")
#' img(RdYlGn(500), "RdYlGn")
#' img(Spectral(500), "Spectral")
#'
#'
#' library(heatmaply)
#' heatmaply(cor(mtcars), colors = PiYG, limits = c(-1,1))
#' heatmaply(cor(mtcars), colors = RdBu, limits = c(-1,1))
#'
#' }
NULL



#' @export
#' @rdname RColorBrewer_colors
BrBG <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "BrBG"))

#' @export
#' @rdname RColorBrewer_colors
PiYG <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "PiYG"))

#' @export
#' @rdname RColorBrewer_colors
PRGn <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "PRGn"))

#' @export
#' @rdname RColorBrewer_colors
PuOr <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "PuOr"))

#' @export
#' @rdname RColorBrewer_colors
RdBu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "RdBu"))

#' @export
#' @rdname RColorBrewer_colors
RdGy <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "RdGy"))

#' @export
#' @rdname RColorBrewer_colors
RdYlBu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "RdYlBu"))

#' @export
#' @rdname RColorBrewer_colors
RdYlGn <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "RdYlGn"))

#' @export
#' @rdname RColorBrewer_colors
Spectral <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(11, "Spectral"))



#' @export
#' @rdname RColorBrewer_colors
Blues <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "Blues"))


#' @export
#' @rdname RColorBrewer_colors
BuGn <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "BuGn"))


#' @export
#' @rdname RColorBrewer_colors
BuPu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "BuPu"))


#' @export
#' @rdname RColorBrewer_colors
GnBu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "GnBu"))


#' @export
#' @rdname RColorBrewer_colors
Greens <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "Greens"))


#' @export
#' @rdname RColorBrewer_colors
Greys <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "Greys"))


#' @export
#' @rdname RColorBrewer_colors
Oranges <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "Oranges"))




#' @export
#' @rdname RColorBrewer_colors
OrRd <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "OrRd"))


#' @export
#' @rdname RColorBrewer_colors
PuBu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "PuBu"))


#' @export
#' @rdname RColorBrewer_colors
PuBuGn <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "PuBuGn"))


#' @export
#' @rdname RColorBrewer_colors
PuRd <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "PuRd"))


#' @export
#' @rdname RColorBrewer_colors
Purples <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "Purples"))


#' @export
#' @rdname RColorBrewer_colors
RdPu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "RdPu"))


#' @export
#' @rdname RColorBrewer_colors
Reds <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "Reds"))




#' @export
#' @rdname RColorBrewer_colors
YlGn <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "YlGn"))


#' @export
#' @rdname RColorBrewer_colors
YlGnBu <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "YlGnBu"))


#' @export
#' @rdname RColorBrewer_colors
YlOrBr <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "YlOrBr"))


#' @export
#' @rdname RColorBrewer_colors
YlOrRd <- grDevices::colorRampPalette(RColorBrewer::brewer.pal(9, "YlOrRd"))









# install.packages("Rgnuplot")
# Rgnuplot:::GpdivergingColormap

GpdivergingColormap <- function(s, rgb1, rgb2, outColorspace = "sRGB") {
  LabToMsh <- function(Lab) {
    L <- Lab@coords[1]
    a <- Lab@coords[2]
    b <- Lab@coords[3]
    M <- sqrt(L * L + a * a + b * b)
    s <- (M > 0.001) * acos(L / M)
    h <- (s > 0.001) * atan2(b, a)
    c(M, s, h)
  }
  MshToLab <- function(Msh) {
    M <- Msh[1]
    s <- Msh[2]
    h <- Msh[3]
    L <- M * cos(s)
    a <- M * sin(s) * cos(h)
    b <- M * sin(s) * sin(h)
    colorspace::LAB(L, a, b)
  }
  AngleDiff <- function(a1, a2) {
    v1 <- matrix(c(cos(a1), sin(a1)), 1, 2, byrow = TRUE)
    v2 <- matrix(c(cos(a2), sin(a2)), 1, 2, byrow = TRUE)
    x <- acos(sum(v1 * v2))
    x
  }
  AdjustHue <- function(msh, unsatM) {
    if (msh[1] >= unsatM - 0.1) {
      h <- msh[3]
    } else {
      hueSpin <- (msh[2] * sqrt(unsatM ^ 2 - msh[1] ^ 2) / (msh[1] *
        sin(msh[2])))
      if (msh[3] > -0.3 * pi) {
        h <- msh[3] + hueSpin
      } else {
        h <- msh[3] - hueSpin
      }
    }
    h
  }
  divergingMap1val <- function(s, rgb1, rgb2, outColorspace = "sRGB") {
    msh1 <- LabToMsh(as(rgb1, "LAB"))
    msh2 <- LabToMsh(as(rgb2, "LAB"))
    if (msh1[2] > 0.05 & msh2[2] > 0.05 & AngleDiff(
      msh1[3],
      msh2[3]
    ) > pi / 3) {
      Mmid <- max(88, msh1[1], msh2[1])
      if (s < 0.5) {
        msh2[1] <- Mmid
        msh2[2] <- 0
        msh2[3] <- 0
        s <- 2 * s
      }
      else {
        msh1[1] <- Mmid
        msh1[2] <- 0
        msh1[3] <- 0
        s <- 2 * s - 1
      }
    }
    if ((msh1[2] < 0.05) & (msh2[2] > 0.05)) {
      msh1[3] <- AdjustHue(msh2, msh1[1])
    } else if ((msh2[2] < 0.05) & (msh1[2] > 0.05)) {
      msh2[3] <- AdjustHue(msh1, msh2[1])
    }
    mshTmp <- msh1
    mshTmp[1] <- (1 - s) * msh1[1] + s * msh2[1]
    mshTmp[2] <- (1 - s) * msh1[2] + s * msh2[2]
    mshTmp[3] <- (1 - s) * msh1[3] + s * msh2[3]
    as(MshToLab(mshTmp), outColorspace)
  }
  dvmap <- matrix(0, length(s), 3)
  for (n in 1:length(s)) dvmap[n, ] <- divergingMap1val(
      s[n],
      rgb1, rgb2, outColorspace
    )@coords
  dvmap
}



#' @export
#' @rdname RColorBrewer_colors
cool_warm <- function(n) {
  colormap <- GpdivergingColormap(
    seq(0, 1, length.out = n),
    rgb1 = colorspace::sRGB(0.230, 0.299, 0.754),
    rgb2 = colorspace::sRGB(0.706, 0.016, 0.150),
    outColorspace = "sRGB"
  )
  colormap[colormap > 1] <- 1 # sometimes values are slightly larger than 1
  colormap <- grDevices::rgb(colormap[, 1], colormap[, 2], colormap[, 3])
  colormap
}

# img <- function(obj, nam) {
#   image(1:length(obj), 1, as.matrix(1:length(obj)), col=obj,
#         main = nam, ylab = "", xaxt = "n", yaxt = "n",  bty = "n")
# }
#
# par(mfrow = c(2,1))
# img(red_blue_diverging_colormap(500), "Cool-warm, (Moreland 2009)")
#
# library(heatmaply)
# img(rev(RdBu(500)), "RdBu")
#

context("RColorBrewer")

test_that("cool_warm", {
  expect_equal(length(cool_warm(10)), 10)
  expect_equal(length(cool_warm(20)), 20)
})


test_that("colorbrewer funs", {
  expect_equal(length(BrBG(10)), 10)
  expect_equal(length(PiYG(10)), 10)
  expect_equal(length(PRGn(10)), 10)
  expect_equal(length(PuOr(10)), 10)
  expect_equal(length(RdBu(10)), 10)
  expect_equal(length(RdGy(10)), 10)
  expect_equal(length(RdYlBu(10)), 10)
  expect_equal(length(RdYlGn(10)), 10)
  expect_equal(length(Spectral(10)), 10)
  expect_equal(length(Blues(10)), 10)
  expect_equal(length(BuGn(10)), 10)
  expect_equal(length(BuPu(10)), 10)
  expect_equal(length(GnBu(10)), 10)
  expect_equal(length(Greens(10)), 10)
  expect_equal(length(Greys(10)), 10)
  expect_equal(length(Oranges(10)), 10)
  expect_equal(length(OrRd(10)), 10)
  expect_equal(length(PuBu(10)), 10)
  expect_equal(length(PuBuGn(10)), 10)
  expect_equal(length(PuRd(10)), 10)
  expect_equal(length(Purples(10)), 10)
  expect_equal(length(RdPu(10)), 10)
  expect_equal(length(Reds(10)), 10)
  expect_equal(length(YlGn(10)), 10)
  expect_equal(length(YlGnBu(10)), 10)
  expect_equal(length(YlOrBr(10)), 10)
  expect_equal(length(YlOrRd(10)), 10)
})

# kCDF_fun, ecdf
percentize_predict <- function(x, ecdf_fun = ecdf, ...) {
  # http://stackoverflow.com/questions/25130531/how-to-select-only-numeric-columns-from-a-data-table
  # x must be a data.frame
  ss_numeric <- sapply(x, is.numeric)

  # ecdf_fun <- kCDF_fun # ecdf

  ecdf_list <- list()
  for (i in 1:ncol(x)) {
    ecdf_list[[i]] <- if (ss_numeric[i]) {
      ecdf_fun(na.omit(x[, i]))
    } else {
      NA
    }
  }

  fun <- function(new_x) {
    if (any(colnames(new_x) != colnames(x))) stop("The column names (or order) of the new x are different than that of the old x. Please fix and try again.")

    for (i in 1:ncol(x)) {
      ecdf_fun <- ecdf_list[[i]]
      if (!(is.vector(ecdf_fun) && is.na(ecdf_fun))) {
        ss_no_NA <- !is.na(new_x[, i])
        new_x[ss_no_NA, i] <- ecdf_fun(new_x[ss_no_NA, i])
      }
    }

    return(new_x)
  }

  class(fun) <- c("function", "percentize_predict")

  return(fun)
}






#' @title  Empirical Percentile Transformation
#' @export
#'
#' @description
#' An Empirical Percentile Transformation (percentize) is similar to taking the rank
#' of a variable. The difference is that it is simpler to compare and interpret
#' the transformed variables.
#'
#' This is helpful for comparing several variables in a heatmap (e.g.: \link{heatmaply}).
#'
#' @param x a vector or a data.frame.
#'
#' @param ... Currently ignored.
#'
#' @return
#' A vector (or data.frame) after \link{ecdf} was used on that vector.
#' If x is a \link{data.frame} then only the numeric variables are transformed.
#'
#' @aliases
#' percentize.default
#' percentize.data.frame
#' percentize.matrix
#' @seealso \link{normalize}
#' @examples
#' \dontrun{
#' x <- mtcars
#' x <- data.frame(x)
#' x$am <- factor(x$am)
#' x$vs <- factor(x$vs)
#' heatmaply(percentize(x))
#'
#'
#' x <- data.frame(a = 1:10, b = 11:20)
#' x[4:6, 1:2] <- NA
#' percentize(x)
#' percentize(x[,1])
#'
#' }
percentize <- function(x, ...) {
  UseMethod("percentize")
}


#' @export
percentize.default <- function(x, ...) {
  ss_no_NA <- !is.na(x)
  x[ss_no_NA] <- ecdf(x[ss_no_NA])(x[ss_no_NA])
  x
}


#' @export
percentize.data.frame <- function(x, ...) {
  # x <- na.omit(x)
  percentize_predict(x)(x)
}


#' @export
percentize.matrix <- function(x, ...) {
  x <- as.data.frame(x)
  percentize(x)
}








#' @title  Normalization transformation (0-1)
#' @export
#'
#' @description
#' An Empirical Normalization Transformation brings data to the 0 to 1 scale
#' by substracting the minimum and dividing by the maximum of all observations.
#' This is similar to \link{percentize} in that it allows to compare variables
#' of different scales, but it also keeps the shape of the distribution.
#'
#' @param x a vector or a data.frame.
#'
#' @param ... Currently ignored.
#'
#' @return
#' A vector (or data.frame) after normalizing the numeric variables.
#'
#' @aliases
#' normalize.default
#' normalize.data.frame
#' normalize.matrix
#' @seealso \link{percentize}
#' @examples
#' \dontrun{
#' x <- mtcars
#' x <- data.frame(x)
#' x$am <- factor(x$am)
#' x$vs <- factor(x$vs)
#' heatmaply(percentize(x))
#' heatmaply(normalize(x))
#'
#'
#' x <- data.frame(a = 1:10, b = 11:20)
#' x[4:6, 1:2] <- NA
#' normalize(x)
#' normalize(x[,1])
#'
#' }
normalize <- function(x, ...) {
  UseMethod("normalize")
}


# http://stats.stackexchange.com/questions/70801/how-to-normalize-data-to-0-1-range
#' @export
normalize.default <- function(x, ...) {
  ss_no_NA <- !is.na(x)
  x[ss_no_NA] <- (x[ss_no_NA] - min(x[ss_no_NA])) / (max(x[ss_no_NA]) - min(x[ss_no_NA]))
  x
}


#' @export
normalize.data.frame <- function(x, ...) {
  ss_numeric <- sapply(x, is.numeric)
  normalize

  x[, ss_numeric] <- lapply(x[, ss_numeric], normalize)

  x
}



#' @export
normalize.matrix <- function(x, ...) {
  x <- as.data.frame(x)
  normalize(x)
}

context("misc")

test_that("is.na10", {
  tf <- rep(c(NA, FALSE), length.out = 10)
  expect_equal(is.na10(tf), as.integer(is.na(tf)))
})

test_that("normalize", {
  a <- seq(0, 1, by = 0.01)
  expect_equal(normalize(a), a)
  b <- 1:1000000
  expect_true(all(normalize(b) <= 1) && all(normalize(b) >= 0))

  d <- data.frame(a = a[1:101], b = b[1:101], c = "c")
  dn <- normalize(d)
  expect_equal(normalize(d[, 1:2]), normalize(as.matrix(d[, 1:2])))
  expect_equal(dn[, 1], a)
  expect_true(all(dn[[3]] == "c"))
})

test_that("percentize", {
  a <- seq(0, 1, by = 0.01)
  expect_equal(percentize(a), unname(unlist(percentize(data.frame(a)))))
})

#' @title  Cluster heatmap based on plotly
#' @name heatmaply
#'
#' @description
#' An object of class heatmapr includes all the needed information
#' for producing a heatmap. The goal is to separate the pre-processing of the
#' heatmap elements from the graphical rendering of the object, which could be done
#'
#' (Please submit an issue on github if you have a feature that you wish to have added)
#'
#' @param x can either be a heatmapr object, or a numeric matrix
#'   Defaults to \code{TRUE} unless \code{x} contains any \code{NA}s.
#'
#' @param colors,col a vector of colors to use for heatmap color.
#' The default uses
#' \code{\link[viridis]{viridis}(n=256, alpha = 1, begin = 0, end = 1, option = "viridis")}
#' It is passed to \link[ggplot2]{scale_fill_gradientn}.
#' If colors is a color function (with the first argument being `n` = the number of colors),
#' it will be used to create 256 colors from that function.
#' (col is there to stay compatible with \link[gplots]{heatmap.2})
#' @param limits a two dimensional numeric vector specifying the data range for the scale.
#' @param na.value color to use for missing values (default is "grey50").
#'
#' @param row_text_angle numeric (Default is 0), the angle of the text of the
#' rows. (this is called srtRow in \link[gplots]{heatmap.2})
#' @param column_text_angle numeric (Default is 45), the angle of the text of
#' the columns. (this is called srtCol in \link[gplots]{heatmap.2})
#'
#' @param subplot_margin Currently not well implemented. It is passed to
#' \link[plotly]{subplot}. Default is 0. Either a single value or
#'  four values (all between 0 and 1). If four values are provided,
#'  the first is used as the left margin, the second is used as the right margin,
#'  the third is used as the top margin, and the fourth is used as the bottom margin.
#'  If a single value is provided, it will be used as all four margins.
#'
#' @param cellnote Values to be shown as annotations atop the heatmap cells.
#' @param draw_cellnote Should the cellnote annotations be drawn? Defaults is FALSE,
#' if cellnote is not supplied, TRUE if cellnote is supplied. If TRUE and
#' cellnote is not supplied, x will be used for cellnote.
#' @param cellnote_color The color of the cellnote text to be used.
#' @param cellnote_textposition The text positioning/centering of the cellnote.
#' Default is "middle right". Options are
#' "top left", "top center", "top right", "middle left", "middle center",
#' "middle right", "bottom left", "bottom center", "bottom right"
#' @param cellnote_size The font size (HTML/CSS) of the cellnote. Default is 12.
#' @param Rowv determines if and how the row dendrogram should be reordered.
#' By default, it is TRUE, which implies dendrogram is computed and reordered
#' based on row means. If NULL or FALSE, then no dendrogram is computed and
#' no reordering is done. If a \link{dendrogram} (or \link{hclust}),
#' then it is used "as-is", ie without any reordering. If a vector of integers,
#' then dendrogram is computed and reordered based on the order of the vector.
#' @param Colv determines if and how the column dendrogram should be reordered.
#' Has the options as the Rowv argument above and additionally when x is a
#' square matrix, Colv = "Rowv" means that columns should be treated
#' identically to the rows.
#' @param distfun function used to compute the distance (dissimilarity)
#' between both rows and columns. Defaults to dist.
#' The options "pearson", "spearman" and "kendall" can be used to
#' use correlation-based clustering, which uses \code{as.dist(1 - cor(t(x)))}
#' as the distance metric (using the specified correlation method).
#' @param hclustfun function used to compute the hierarchical clustering
#' when Rowv or Colv are not dendrograms. Defaults to hclust.
#'
#' @param dist_method default is NULL (which results in "euclidean" to be used).
#' Can accept alternative character strings indicating the
#' method to be passed to distfun. By default distfun. is \link{dist} hence
#' this can be one of "euclidean", "maximum", "manhattan", "canberra", "binary"
#' or "minkowski".
#' @param hclust_method default is NULL (which results in "complete" to be used).
#' Can accept alternative character strings indicating the
#' method to be passed to hclustfun By default hclustfun is \link{hclust} hence
#' this can be one of "ward.D", "ward.D2", "single", "complete", "average"
#' (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
#' Specifying hclust_method=NA causes heatmaply to use
#' \code{\link[dendextend]{find_dend}} to find the "optimal" dendrogram for
#' the data.
#'
#' @param distfun_row distfun for row dendrogram only.
#' @param hclustfun_row hclustfun for col dendrogram only.
#' @param distfun_col distfun for row dendrogram only.
#' @param hclustfun_col hclustfun for col dendrogram only.
#'
#' @param dendrogram character string indicating whether to compute 'none',
#' 'row', 'column' or 'both' dendrograms. Defaults to 'both'.
#' However, if Rowv (or Colv) is FALSE or NULL and dendrogram is 'both',
#' then a warning is issued and Rowv (or Colv) arguments are honoured.
#' It also accepts TRUE/FALSE as synonyms for "both"/"none".
#' @param show_dendrogram Logical vector of length two, controlling whether
#' the row and/or column dendrograms are displayed. If a logical scalar is
#' provided, it is repeated to become a logical vector of length two.
#' @param reorderfun function(d, w) of dendrogram and weights for reordering the
#' row and column dendrograms. The default uses stats{reorder.dendrogram}
#'
#' @param k_row an integer scalar with the desired number of groups by which to
#' color the dendrogram's branches in the rows (uses \link[dendextend]{color_branches})
#' If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.
#' @param k_col an integer scalar with the desired number of groups by which to
#' color the dendrogram's branches in the columns (uses \link[dendextend]{color_branches})
#' If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.
#'
#' @param symm logical indicating if x should be treated symmetrically; can only
#' be true when x is a square matrix.
#' @param revC logical indicating if the column order should be reversed for plotting.
#' Default (when missing) - is FALSE, unless symm is TRUE.
#' This is useful for cor matrix.
#'
#' @param scale character indicating if the values should be centered and scaled
#' in either the row direction or the column direction, or none. The default is
#' "none".
#' @param na.rm logical (default is TRUE) indicating whether NA's should be
#' removed when scaling (i.e.: when using rowMeans/colMeans). Generally it
#' should always be kept as TRUE, and is included here mainly to stay backward
#' compatible with gplots::heatmap.2. This argument does not effect the presence
#' of NA values in the matrix itself. For removing rows/columns with NAs you
#' should pre-process your matrix using na.omit (or some form of imputation).
#'
#' @param row_dend_left logical (default is FALSE). Should the row dendrogram be
#' plotted on the left side of the heatmap. If false then it will be plotted on
#' the right side.
#'
#' @param margins numeric vector of length 4 (default is c(50,50,NA,0))
#' containing the margins (see \link[plotly]{layout}) for column, row and main
#' title names, respectively. The top margin is NA by default. If main==""
#' then the top margin will be set to 0, otherwise it will get 30.
#' For a multiline title a larger default for the 3rd element should be set.
#' The right margin is NA by default, meaning it will be zero if row_dend_left
#' is FALSE, or 100 if row_dend_left is TRUE.
#'
#' @param ... other parameters passed to \link{heatmapr} (currently, various
#' parameters may be ignored.
#'
#' @param scale_fill_gradient_fun A function that creates a smooth gradient for the heatmap.
#' The default uses \link[ggplot2]{scale_fill_gradientn} with the values of colors, limits, and
#' na.value that are supplied by the user. The user can input a customized function, such as
#' \link{scale_color_gradient}() in order to get other results (although the virids default
#' is quite recommended)
#'
#' @param grid_color control the color of the heatmap grid. Default is NA.
#' Value passed to \link[ggplot2]{geom_tile}. Do not use this parameter on
#' larger matrix sizes, as it can dramatically prolong the build time of the heatmap.
#' (another parameter, grid_color, will be added in the future - once it is implemented in plotly)
#' In the meantime it is MUCH better to use the grid_gap argument.
#'
#' @param grid_gap this is a fast alternative to grid_color. The default is 0, but if a larger value
#' is used (for example, 1), then the resulting heatmap will have a white grid which can
#' help identify different cells. This is implemented using \link[plotly]{style} (with xgap and ygap).
#'
#' @param srtRow if supplied, this overrides row_text_angle (this is to stay compatible with \link[gplots]{heatmap.2})
#' @param srtCol if supplied, this overrides column_text_angle (this is to stay compatible with \link[gplots]{heatmap.2})
#'
#' @param xlab A character title for the x axis.
#' @param ylab A character title for the y axis.
#'
#' @param main A character title for the heatmap.
#'
#' @param titleX logical (TRUE). should x-axis titles be retained? (passed to \link[plotly]{subplot}).
#' @param titleY logical (TRUE). should y-axis titles be retained? (passed to \link[plotly]{subplot}).
#'
#'
#' @param hide_colorbar logical (FALSE). If TRUE, then the color bar (i.e.: the legend) is hidden.
#'
#' @param key.title (character) main title of the color key. If set to NULL (default) no title will be plotted.
#'
#' @param return_ppxpy logical (FALSE). If TRUE, then no plotting is done and the p, px and py objects are
#' returned (before turning into plotly objects). This is a temporary option which might be removed in the
#' future just to make it easy to create a ggplot heatmaps.
#'
#' @param row_side_colors,col_side_colors data.frame of factors to produce
#'    row/column side colors in the style of heatmap.2/heatmap.3.
#'    When a data.frame is provided, the column names are used as the label names for each of the newly added row_side_colors.
#'    When a vector is provided it is coerced into a data.frame and the name of the side color will be just row_side_colors.
#'
#' @param row_side_palette,col_side_palette Color palette functions to be
#'    used for row_side_colors and col_side_colors respectively.
#'
#' @param ColSideColors,RowSideColors passed to row_side_colors,col_side_colors in order
#' to keep compatibility with \link[gplots]{heatmap.2}
#'
#' @param plot_method Use "ggplot" or "plotly" to choose which library produces heatmap
#' and dendrogram plots
#' @param seriate character indicating the method of matrix sorting (default: "OLO").
#' Implemented options include:
#' "OLO" (Optimal leaf ordering, optimizes the Hamiltonian path length that is
#' restricted by the dendrogram structure - works in O(n^4) )
#' "mean" (sorts the matrix based on the reorderfun using marginal means of
#' the matrix. This is the default used by \link[gplots]{heatmap.2}),
#' "none" (the default order produced by the dendrogram),
#' "GW" (Gruvaeus and Wainer heuristic to optimize the Hamiltonian path length
#' that is restricted by the dendrogram structure)
#'
#' @param heatmap_layers ggplot object(s) (eg, list(theme_bw())) to be added to
#'  the heatmap before conversion to a plotly object.
#' @param side_color_layers ggplot2 objects to be added to side color plots,
#'  similar to heatmap_layers.
#' @param dendrogram_layers ggplot2 objects to be added to dendrograms,
#'  similar to heatmap_layers and side_color_layers.
#' @param branches_lwd numeric (default is 0.6). The width of the dendrograms' branches.
#' If NULL then it is ignored. If the "lwd" is already defined in Rowv/Colv then this
#' parameter is ignored (it is checked using \link[dendextend]{has_edgePar}("lwd")).
#'
#'
#' @param file name of the file(s) into which to save the heatmaply output.
#' Should be a character vector of strings ending with ".html" for a dynamic output,
#' or ".png", ".jpeg", ".pdf" for a static output.
#'
#' For example: heatmaply(x, file = "heatmaply_plot.html") or
#' dir.create("folder");
#' heatmaply(x, file = "folder/heatmaply_plot.html")
#'
#' This is based on \link[htmlwidgets]{saveWidget}, and \link[webshot]{webshot} for the static files.
#' For more refined control over the static file output, you should save the heatmaply object using \link[plotly]{export} and pass the
#' arguments you want based on the ones in \link[webshot]{webshot}.
#'
#' Another example: heatmaply(x, file = c("heatmaply_plot.html", "heatmaply_plot.png"))
#'
#' @param width,height The width and height of the image saved. Default is
#' which uses 800 x 500 pixels.
#' @param long_data Data in long format. Replaces x, so both should not be used.
#'  Colnames must be c("name", "variable", "value"). If you do not have a names
#'  column you can simply use a sequence of numbers from 1 to the number of "rows"
#'  in the data.
#'
#' @param label_names Names for labels of x, y and value/fill mouseover.
#' @param fontsize_row,fontsize_col,cexRow,cexCol Font size for row and column labels.
#' @param subplot_widths,subplot_heights The relative widths and heights of each
#'  subplot. The length of these vectors will vary depending on the number of
#'  plots involved.
#'
#' @param colorbar_len The length of the colorbar/color key relative to the total
#' plot height. Only used if plot_method = "plotly"
#'
#' @param colorbar_thickness The thickness (width) of the colorbar/color key
#' in pixels. Only used if plot_method = "plotly".
#'
#' @param colorbar_xanchor,colorbar_yanchor The x and y anchoring points of the
#' colorbar/color legend. Can be "left", "middle" or "right" for colorbar_xanchor,
#' and "top", "middle" or "bottom" for colorbar_yanchor.
#' See \code{\link[plotly]{colorbar}} for more details.
#' @param colorbar_xpos,colorbar_ypos The x and y co-ordinates (in proportion of the plot window)
#' of the colorbar/color legend. See \code{\link[plotly]{colorbar}} for more details.
#'
#' @param showticklabels A logical vector of length two (default is TRUE).
#' If FALSE, then the ticks are removed from the sides of the plot. The first location refers to
#' the x axis and the second to the y axis. If only one value is supplied (TRUE/FALSE) then it is
#' replicated to get to length 2. When using this parameter, it might be worth also adjusting
#' margins.
#' This option should be used when working with medium to large matrix size as it
#' makes the heatmap much faster (and the hover still works).
#'
#' @param dynamicTicks (default: FALSE). passed to \link[plotly]{ggplotly}:
#' should plotly.js dynamically generate axis tick labels?
#' Dynamic ticks are useful for updating ticks in response to zoom/pan interactions; however,
#' they can not always reproduce labels as they would appear in the static ggplot2 image.
#'
#' @param node_type For plot_method = "ggplot", should the heatmap be rendered as
#'  a x-y scatter plot (node_type = "scatter") or a heatmap (node_type = "heatmap").
#'  Default is node_type = "heatmap".
#'
#' @param grid_size When node_type is "scatter", this controls point size. When
#' node_type is "heatmap", this controls the size of the grid between heatmap cells.
#'
#' @param point_size_mat Matrix to map to point size
#' @param point_size_name Name of point size mapping (for hovertext/legend)
#' @param custom_hovertext Custom hovertext matrix (the same dimensions as the input).
#' If plot_method is "plotly" then just this text is displayed; if plot_method
#' if "ggplot" then it is appended to the existing text.
#' @param label_format_fun Function to format hovertext (eg,
#'    \code{function(...) round(..., digits=3)} or
#'    \code{function(...) format(..., digits=3)}
#'
#' @param labRow,labCol character vectors with row and column labels to use;
#' these default to rownames(x) or colnames(x), respectively.
#' if set to NA, they change the value in showticklabels to be FALSE. This is mainly to keep
#' backward compatibility with gplots::heatmap.2.
#' @param dend_hoverinfo Boolean value which controls whether mouseover text
#' is shown for the row and column dendrograms.
#' @param side_color_colorbar_len As with colorbar_len, this controls the
#' length of the colorbar/color key relative to the total plot height.
#' This argument controls the colorbar_len of the side colour plots.
#' Only used if plot_method = "plotly".
#'
#' @export
#' @examples
#'
#' \dontrun{
#'
#' # mtcars
#' # x <- heatmapr(mtcars)
#' library(heatmaply)
#' heatmaply(iris[,-5], k_row = 3, k_col = 2)
#' heatmaply(cor(iris[,-5]))
#' heatmaply(cor(iris[,-5]), limits = c(-1,1))
#' heatmaply(mtcars, k_row = 3, k_col = 2)
#' # heatmaply(mtcars, k_row = 3, k_col = 2, grid_color = "white")
#' heatmaply(mtcars, k_row = 3, k_col = 2, grid_gap = 1)
#'
#' # make sure there is enough room for the labels:
#' heatmaply(mtcars, margins = c(40, 130))
#' # this is the same as using:
#' heatmaply(mtcars) %>% layout(margin = list(l = 130, b = 40))
#'
#' # control text angle
#' heatmaply(mtcars, column_text_angle = 90, margins = c(40, 130))
#' # the same as using srtCol:
#' # heatmaply(mtcars, srtCol = 90) %>% layout(margin = list(l = 130, b = 40))
#'
#'
#'
#' x <- mtcars
#' # different colors
#' heatmaply(x, colors = heat.colors(200))
#' # using special scale_fill_gradient_fun colors
#' heatmaply(x, scale_fill_gradient_fun = scale_color_gradient())
#'
#'
#' # We can join two heatmaps together:
#' library(heatmaply)
#' hm1 <- heatmaply(mtcars, margins = c(40, 130))
#' hm2 <- heatmaply(mtcars, scale = "col", margins = c(40, 130))
#' subplot(hm1, hm2, margin = .2)
#'
#' # If we want to share the Y axis, then it is risky to keep any of the dendrograms:
#' library(heatmaply)
#' hm1 <- heatmaply(mtcars, Colv = FALSE, Rowv = FALSE, margins = c(40, 130))
#' hm2 <- heatmaply(mtcars, scale = "col" , Colv = FALSE, Rowv = FALSE,
#'              margins = c(40, 130))
#' subplot(hm1, hm2, margin = .02, shareY = TRUE)
#'
#'
#' # We can save heatmaply as an HTML file by using:
#' heatmaply(iris[,-5], file = "heatmaply_iris.html")
#' # or a png/pdf/jpeg file using:
#' heatmaply(iris[,-5], file = "heatmaply_iris.png")
#' # or just doing it in one go:
#' heatmaply(iris[,-5], file = c("heatmaply_iris.html", "heatmaply_iris.png") )
#'
#'
#'
#' # If we don't want the HTML to be selfcontained, we can use the following:
#' library(heatmaply)
#' library(htmlwidgets)
#' heatmaply(iris[,-5]) %>%
#'    saveWidget(file="heatmaply_iris.html",selfcontained = FALSE)
#'
#'
#' # Example for using RowSideColors
#'
#' x  <- as.matrix(datasets::mtcars)
#' rc <- colorspace::rainbow_hcl(nrow(x))
#'
#' library(gplots)
#' library(viridis)
#' heatmap.2(x, trace = "none", col = viridis(100),
#'           RowSideColors=rc)
#'
#' heatmaply(x, seriate = "mean",
#'           RowSideColors=rc)
#'
#'
#' heatmaply(x[,-c(8,9)], seriate = "mean",
#'           col_side_colors = c(rep(0,5), rep(1,4)),
#'           row_side_colors = x[,8:9])
#' heatmaply(x[,-c(8,9)], seriate = "mean",
#'           col_side_colors = data.frame(a=c(rep(0,5), rep(1,4))),
#'           row_side_colors = x[,8:9])
#'
#'
#' ## Example of using Rowv And Colv for custumized dendrograms.
#'
#'
#' x  <- as.matrix(datasets::mtcars)
#'
#' # now let's spice up the dendrograms a bit:
#' library(dendextend)
#'
#' row_dend  <- x %>% dist %>% hclust %>% as.dendrogram %>%
#'   set("branches_k_color", k = 3) %>% set("branches_lwd", 4) %>%
#'   ladderize
#' #    rotate_DendSer(ser_weight = dist(x))
#' col_dend  <- x %>% t %>% dist %>% hclust %>% as.dendrogram %>%
#'   set("branches_k_color", k = 2) %>% set("branches_lwd", 4) %>%
#'   ladderize
#' #    rotate_DendSer(ser_weight = dist(t(x)))
#'
#' heatmaply(x, Rowv = row_dend, Colv = col_dend)
#'
#'
#' heatmaply(is.na10(airquality))
#' heatmaply(is.na10(airquality), grid_gap = 1)
#'
#' # grid_gap can handle quite large data matrix
#' heatmaply(matrix(1:10000,100,100), k_row = 3, k_col = 3, grid_gap = 1)
#'
#' # Examples of playing with font size:
#' heatmaply(mtcars, fontsize_col = 20, fontsize_row = 5, margin = c(100,90))
#'
#'
#'
#' # Example for using subplot_width/subplot_height
#'
#' heatmaply(percentize(mtcars),
#'      subplot_widths=c(0.6, 0.4),
#'      subplot_heights=c(0.05, 0.95))
#'
#'
#'
#' # Example of removing labels and thus making the plot faster
#' heatmaply(iris, showticklabels = c(T,F), margins = c(80,10))
#'
#' # this is what allows for a much larger matrix to be printed:
#' set.seed(2017-05-18)
#' large_x <- matrix(rnorm(19), 1000,100)
#' heatmaply(large_x, dendrogram = F, showticklabels = F, margins = c(1,1))
#'
#'
#' }
heatmaply <- function(x, ...) {
  UseMethod("heatmaply")
}


#' @export
#' @description
#' heatmaply_na is a wrapper for `heatmaply` which comes with defaults that are better
#' for exploring missing value (NA) patterns. Specifically, the grid_gap is set to 1, and the
#' colors include two shades of grey. It also calculates the \link{is.na10} automatically.
#' @rdname heatmaply
#' @examples
#' \dontrun{
#' heatmaply_na(airquality)
#' }
heatmaply_na <- function(x,
                         grid_gap = 1,
                         colors = c("grey80", "grey20"),
                         ...) {
  heatmaply(
    is.na10(x), grid_gap = grid_gap,
    colors = colors, ...
  )
}

#' @export
#' @description
#' heatmaply_cor is a wrapper for `heatmaply` which comes with defaults that are better
#' for correlation matrixes. Specifically, the limits are set from -1 to 1, and the color palette is \link{RdBu}.
#' @rdname heatmaply
#' @examples
#' \dontrun{
#' heatmaply_cor(cor(mtcars))
#' }
heatmaply_cor <- function(x,
                          limits = c(-1, 1),
                          colors = cool_warm,
                          ...) {
  heatmaply(
    x, limits = limits, # symm = TRUE,
    colors = colors, ...
  )
}







#' @export
#' @rdname heatmaply
heatmaply.default <- function(x,
                              # elements for scale_fill_gradientn
                              colors = viridis(
                                n = 256, alpha = 1, begin = 0,
                                end = 1, option = "viridis"
                              ),
                              limits = NULL,
                              na.value = "grey50",
                              row_text_angle = 0,
                              column_text_angle = 45,
                              subplot_margin = 0,
                              cellnote = NULL,
                              draw_cellnote = !is.null(cellnote),
                              cellnote_color = "auto",
                              cellnote_textposition = "middle right",
                              cellnote_size = 12,

                              ## dendrogram control
                              Rowv,
                              Colv,
                              distfun = dist,
                              hclustfun = hclust,
                              dist_method = NULL,
                              hclust_method = NULL,

                              distfun_row,
                              hclustfun_row,
                              distfun_col,
                              hclustfun_col,

                              dendrogram = c("both", "row", "column", "none"),
                              show_dendrogram = c(TRUE, TRUE),
                              reorderfun = function(d, w) reorder(d, w),

                              k_row = 1,
                              k_col = 1,

                              symm = FALSE,
                              revC,

                              ## data scaling
                              scale = c("none", "row", "column"),
                              na.rm = TRUE,

                              row_dend_left = FALSE,
                              margins = c(NA, NA, NA, NA),
                              ...,
                              scale_fill_gradient_fun = NULL,
                              grid_color = NA,
                              grid_gap = 0,
                              srtRow, srtCol,
                              xlab = "", ylab = "",
                              main = "",
                              titleX = TRUE, titleY = TRUE,
                              hide_colorbar = FALSE,
                              key.title = NULL,
                              return_ppxpy = FALSE,
                              row_side_colors,
                              row_side_palette = NULL,
                              col_side_colors,
                              col_side_palette = NULL,
                              ColSideColors = NULL,
                              RowSideColors = NULL,
                              seriate = c("OLO", "mean", "none", "GW"),
                              heatmap_layers = NULL,
                              side_color_layers = NULL,
                              dendrogram_layers = NULL,
                              branches_lwd = 0.6,
                              file,
                              width = NULL,
                              height = NULL,
                              long_data,
                              plot_method = c("ggplot", "plotly"),
                              label_names = NULL,
                              fontsize_row = 10,
                              fontsize_col = 10,
                              cexRow, cexCol,
                              subplot_widths = NULL,
                              subplot_heights = NULL,
                              colorbar_len = 0.3,
                              colorbar_thickness = 30,
                              colorbar_xanchor = if (row_dend_left) "right" else "left",
                              colorbar_yanchor = "bottom",
                              colorbar_xpos = if (row_dend_left) -0.1 else 1.1,
                              colorbar_ypos = 0,
                              showticklabels = c(TRUE, TRUE),
                              dynamicTicks = FALSE,
                              grid_size = 0.1,
                              node_type = "heatmap",
                              point_size_mat = NULL,
                              point_size_name = "Point size",
                              label_format_fun = function(...) format(..., digits = 4),
                              labRow = NULL, labCol = NULL,
                              custom_hovertext = NULL,
                              col = NULL,
                              dend_hoverinfo = TRUE,
                              side_color_colorbar_len = 0.3) {
  if (!missing(long_data)) {
    if (!missing(x)) {
      stop("x and long_data should not be used together")
    }
    assert_that(
      ncol(long_data) == 3,
      all(colnames(long_data) == c("name", "variable", "value"))
    )
    x <- reshape2::dcast(long_data, name ~ variable)
    rownames(x) <- x$name
    x$name <- NULL
  }


  # this is to fix the error: "argument * matches multiple formal arguments"
  if (!is.null(col)) colors <- col

  if (is.null(scale_fill_gradient_fun)) {
    if (node_type == "heatmap") {
      scale_fill_gradient_fun <- scale_fill_gradientn(
        colors = if (is.function(colors)) colors(256) else colors,
        na.value = na.value, limits = limits
      )
    } else {
      scale_fill_gradient_fun <- scale_color_gradientn(
        colors = if (is.function(colors)) colors(256) else colors,
        na.value = na.value, limits = limits
      )
    }
  }

  plot_method <- match.arg(plot_method)

  if (plot_method == "ggplot") {

    ## Suppress creation of new graphcis device, but on exit replace it.
    ## TODO: Avoid this or find better method
    old_dev <- options()[["device"]]
    on.exit(options(device = old_dev))
    options(device = names(capabilities()[which(capabilities())])[1])
  }


  if (is.logical(dendrogram)) {
    # Using if and not ifelse to make sure the output is a "scalar".
    dendrogram <- if (dendrogram) "both" else "none"
    # if(T) "both" else "none"
    # if(F) "both" else "none"
  }
  dendrogram <- match.arg(dendrogram)

  if (!(is.data.frame(x) | is.matrix(x))) {
    stop("x must be either a data.frame or a matrix.")
  }

  if (!missing(srtRow)) {
    row_text_angle <- srtRow
  }
  if (!missing(srtCol)) {
    column_text_angle <- srtCol
  }
  if (!is.null(ColSideColors)) {
    col_side_colors <- ColSideColors
  }
  if (!is.null(RowSideColors)) {
    row_side_colors <- RowSideColors
  }

  if (!missing(cexRow)) {
    fontsize_row <- if (is.numeric(cexRow)) cexRow * 10 else cexRow
  }
  if (!missing(cexCol)) {
    fontsize_col <- if (is.numeric(cexCol)) cexCol * 10 else cexCol
  }

  # TODO: maybe create heatmaply.data.frame heatmaply.matrix instead.
  #       But right now I am not sure this would be needed.
  if (is.data.frame(x)) {
    ss_c_numeric <- sapply(x, is.numeric)
  }
  if (is.matrix(x)) {
    ss_c_numeric <- apply(x, 2, is.numeric)
  }

  # We must have some numeric values to be able to make a heatmap
  if (!any(ss_c_numeric)) {
    stop("heatmaply only works for data.frame/matrix which includes some numeric columns.")
  }

  # If we have non-numeric columns, we should move them to row_side_colors
  # TODO: add a parameter to control removing of non-numeric columns without moving them to row_side_colors
  if (!all(ss_c_numeric)) {
    row_side_colors <- if (missing(row_side_colors)) {
      data.frame(x[, !ss_c_numeric, drop = FALSE])
    } else {
      data.frame(row_side_colors, x[, !ss_c_numeric, drop = FALSE])
    }
    x <- x[, ss_c_numeric]
  }
  if (!is.null(labRow)) {
    if (all(is.na(labRow))) {
      showticklabels[[2]] <- FALSE
    }
  } else {
    labRow <- rownames(x)
  }
  if (!is.null(labCol)) {
    if (all(is.na(labCol))) {
      showticklabels[[1]] <- FALSE
    }
  } else {
    labCol <- colnames(x)
  }
  if (!is.logical(showticklabels)) {
    stop("showticklabels must be a logical vector of length 2 or 1")
  }
  if (length(showticklabels) == 1) {
    showticklabels <- rep(showticklabels, 2)
  }

  # help dendrogram work again:
  if (dendrogram == "row") Colv <- FALSE
  if (dendrogram == "column") Rowv <- FALSE
  if (dendrogram == "none") Rowv <- Colv <- FALSE

  # this also occurs in heatmapr, so it may be o.k. to remove the following line.
  seriate <- match.arg(seriate)

  if (is.numeric(cellnote_color)) cellnote_color <- grDevices::palette()[cellnote_color]

  hm <- heatmapr(
    x,
    row_side_colors = row_side_colors,
    col_side_colors = col_side_colors,
    point_size_mat = point_size_mat,
    seriate = seriate,

    cellnote = cellnote,

    ## dendrogram control
    Rowv = Rowv,
    Colv = Colv,
    distfun = distfun,
    hclustfun = hclustfun,
    dist_method = dist_method,
    hclust_method = hclust_method,

    distfun_row = distfun_row,
    hclustfun_row = hclustfun_row,
    distfun_col = distfun_col,
    hclustfun_col = hclustfun_col,

    dendrogram = dendrogram,
    show_dendrogram = show_dendrogram,
    reorderfun = reorderfun,

    k_row = k_row,
    k_col = k_col,

    symm = symm,
    revC = revC,

    ## data scaling
    scale = scale,
    na.rm = na.rm,
    custom_hovertext = custom_hovertext,
    labRow = labRow,
    labCol = labCol,
    ...
  )
  hmly <- heatmaply(
    hm,
    colors = colors, limits = limits,
    scale_fill_gradient_fun = scale_fill_gradient_fun,
    grid_color = grid_color,
    grid_gap = grid_gap,
    row_text_angle = row_text_angle,
    column_text_angle = column_text_angle,
    subplot_margin = subplot_margin,
    row_dend_left = row_dend_left,
    xlab = xlab, ylab = ylab, main = main,
    titleX = titleX, titleY = titleY,
    hide_colorbar = hide_colorbar,
    key.title = key.title,
    return_ppxpy = return_ppxpy,
    margins = margins,
    row_side_palette = row_side_palette,
    col_side_palette = col_side_palette,
    heatmap_layers = heatmap_layers,
    side_color_layers = side_color_layers,
    dendrogram_layers = dendrogram_layers,
    ColSideColors = ColSideColors,
    RowSideColors = RowSideColors,
    branches_lwd = branches_lwd,
    label_names = label_names,
    plot_method = plot_method,
    draw_cellnote = draw_cellnote,
    cellnote_textposition = cellnote_textposition,
    cellnote_size = cellnote_size,
    cellnote_color = cellnote_color,
    fontsize_row = fontsize_row,
    fontsize_col = fontsize_col,
    subplot_widths = subplot_widths,
    subplot_heights = subplot_heights,
    colorbar_len = colorbar_len,
    colorbar_thickness = colorbar_thickness,
    colorbar_xanchor = colorbar_xanchor,
    colorbar_yanchor = colorbar_yanchor,
    colorbar_xpos = colorbar_xpos,
    colorbar_ypos = colorbar_ypos,
    showticklabels = showticklabels,
    dynamicTicks = dynamicTicks,
    grid_size = grid_size,
    node_type = node_type,
    point_size_name = point_size_name,
    label_format_fun = label_format_fun,
    dend_hoverinfo = dend_hoverinfo,
    side_color_colorbar_len = side_color_colorbar_len
  )

  # TODO: think more on what should be passed in "..."


  if (!missing(file)) {
    hmly_to_file(hmly = hmly, file = file, width = width, height = height)
  }

  hmly
}

#' @export
#' @rdname heatmaply
heatmaply.heatmapr <- function(x,
                               # elements for scale_fill_gradientn
                               colors = viridis(
                                 n = 256, alpha = 1, begin = 0,
                                 end = 1, option = "viridis"
                               ),
                               limits = NULL,
                               na.value = "grey50",
                               row_text_angle = 0,
                               column_text_angle = 45,
                               subplot_margin = 0,

                               row_dend_left = FALSE,
                               margins = c(NA, NA, NA, NA),
                               ...,
                               scale_fill_gradient_fun = scale_fill_gradientn(
                                 colors = if (is.function(colors)) colors(256) else colors,
                                 na.value = na.value, limits = limits
                               ),
                               grid_color = NA,
                               grid_gap = 0,
                               srtRow, srtCol,
                               xlab = "", ylab = "",
                               main = "",
                               titleX = TRUE, titleY = TRUE,
                               hide_colorbar = FALSE,
                               key.title = NULL,
                               return_ppxpy = FALSE,
                               draw_cellnote = FALSE,
                               cellnote_color = "auto",
                               cellnote_textposition = "middle right",
                               cellnote_size = 12,
                               row_side_colors = x[["row_side_colors"]],
                               row_side_palette = NULL,
                               col_side_colors = x[["col_side_colors"]],
                               col_side_palette = NULL,
                               plot_method = c("ggplot", "plotly"),
                               ColSideColors = NULL,
                               RowSideColors = NULL,
                               heatmap_layers = NULL,
                               side_color_layers = NULL,
                               dendrogram_layers = NULL,
                               branches_lwd = 0.6,
                               label_names = c("row", "column", "value"),
                               fontsize_row = 10,
                               fontsize_col = 10,
                               subplot_widths = NULL,
                               subplot_heights = NULL,
                               colorbar_xanchor = if (row_dend_left) "right" else "left",
                               colorbar_yanchor = "bottom",
                               colorbar_xpos = if (row_dend_left) -0.1 else 1.1,
                               colorbar_ypos = 0,
                               colorbar_len = 0.3,
                               colorbar_thickness = 30,
                               showticklabels = c(TRUE, TRUE),
                               dynamicTicks = FALSE,
                               node_type = c("scatter", "heatmap"),
                               grid_size = 0.1,
                               point_size_mat = x[["matrix"]][["point_size_mat"]],
                               point_size_name = "Point size",
                               label_format_fun = function(...) format(..., digits = 4),
                               custom_hovertext = x[["matrix"]][["custom_hovertext"]],
                               dend_hoverinfo = TRUE,
                               side_color_colorbar_len = 0.3) {

  node_type <- match.arg(node_type)
  plot_method <- match.arg(plot_method)
  cellnote_textposition <- match.arg(
    cellnote_textposition,
    choices = c(
      "top left", "top center", "top right", "middle left",
      "middle center", "middle right", "bottom left", "bottom center",
      "bottom right"
    )
  )

  is.Rgui <- function(...) {
    .Platform$GUI == "Rgui" # if running on MAC OS, this would likely be "AQUA"
  }

  if (is.Rgui()) {
    # print(p) # solves R crashes - not sure why...
    dev.new() # it seems we need just some device to be open...
  }


  # informative errors for mis-specified limits
  if (!is.null(limits)) {
    if (!is.numeric(limits)) stop("limits must be numeric")
    if (length(limits) != 2L) stop("limits must be of length 2 (i.e.: two dimensional)")

    r <- range(as.matrix(x$matrix$data), na.rm = TRUE)
    limits <- sort(limits)

    ## Warn for broken heatmap colors
    if (limits[1] > r[1]) {
      limits[1] <- r[1]
      warning("Lower limit is not <= lowest value in x, min of limits is set to the min of the range (otherwise, colors will be broken!)")
    }
    if (limits[2] < r[2]) {
      limits[2] <- r[2]
      warning("Upper limit is not >= highest value in x, max of limits is set to the max of the range (otherwise, colors will be broken!)")
    }
  }
  if (!missing(srtRow)) row_text_angle <- srtRow
  if (!missing(srtCol)) column_text_angle <- srtCol



  # x is a heatmapr object.
  # heatmapr <- list(rows = rowDend, cols = colDend, matrix = mtx, image = imgUri,
  #                  theme = theme, options = options)
  # x <- heatmapr(mtcars)
  # source: http://stackoverflow.com/questions/6528180/ggplot2-plot-without-axes-legends-etc
  theme_clear_grid_dends <- theme(
    axis.line = element_blank(),
    axis.text.x = element_blank(),
    axis.text.y = element_blank(),
    axis.ticks = element_blank(),
    axis.title.x = element_blank(),
    axis.title.y = element_blank(),
    legend.position = "none",
    panel.background = element_blank(),
    panel.border = element_blank(),
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    plot.background = element_blank()
  )
  # dendrograms:
  rows <- x$rows
  cols <- x$cols

  if (!is.null(branches_lwd) && branches_lwd != 1) {
    if (is.dendrogram(rows) && !has_edgePar(rows, "lwd")) {
      rows <- set(rows, "branches_lwd", branches_lwd)
    }
    if (is.dendrogram(cols) && !has_edgePar(cols, "lwd")) {
      cols <- set(cols, "branches_lwd", branches_lwd)
    }
  }



  # this is using dendextend
  if (is.null(cols)) {
    py <- NULL
  } else {
    if (plot_method == "ggplot") {
      col_ggdend <- as.ggdend(cols)
      xlims <- c(0.5, nrow(col_ggdend$labels) + 0.5)
      py <- ggplot(cols, labels = FALSE, na.rm = TRUE) +
        theme_bw() +
        coord_cartesian(expand = FALSE, xlim = xlims) +
        theme_clear_grid_dends +
        dendrogram_layers
    } else {
      py <- plotly_dend(cols,
                        side = "col",
                        dend_hoverinfo = dend_hoverinfo)
    }
  }
  if (is.null(rows)) {
    px <- NULL
  } else {
    if (plot_method == "ggplot") {
      row_ggdend <- as.ggdend(rows)
      ylims <- c(0.5, nrow(row_ggdend$labels) + 0.5)

      px <- ggplot(row_ggdend, labels = FALSE, na.rm = TRUE) +
        theme_bw() +
        coord_flip(expand = FALSE, xlim = ylims) +
        theme_clear_grid_dends +
        dendrogram_layers
      if (row_dend_left) {
        px <- px + scale_y_reverse()
      }
    } else {
      px <- plotly_dend(rows,
                        flip = row_dend_left,
                        side = "row",
                        dend_hoverinfo = dend_hoverinfo)
    }
  }
  # create the heatmap
  data_mat <- x$matrix$data

  if (plot_method == "ggplot") {
    p <- ggplot_heatmap(
      data_mat,
      row_text_angle,
      column_text_angle,
      scale_fill_gradient_fun,
      grid_color,
      grid_size = grid_size,
      key.title = key.title,
      layers = heatmap_layers,
      row_dend_left = row_dend_left,
      label_names = label_names,
      type = node_type,
      fontsize_row = fontsize_row,
      fontsize_col = fontsize_col,
      point_size_mat = point_size_mat,
      point_size_name = point_size_name,
      label_format_fun = label_format_fun,
      custom_hovertext = custom_hovertext,
      showticklabels = showticklabels
    )
  } else if (plot_method == "plotly") {
    p <- plotly_heatmap(
      data_mat,
      limits = limits,
      colors = colors,
      key_title = key.title,
      label_names = label_names,
      row_text_angle = row_text_angle, column_text_angle = column_text_angle,
      fontsize_row = fontsize_row, fontsize_col = fontsize_col,
      colorbar_yanchor = colorbar_yanchor, colorbar_xanchor = colorbar_xanchor,
      colorbar_xpos = colorbar_xpos, colorbar_ypos = colorbar_ypos,
      point_size_mat = point_size_mat,
      point_size_name = point_size_name,
      colorbar_len = colorbar_len,
      colorbar_thickness = colorbar_thickness,
      custom_hovertext = custom_hovertext
    )
  }






  # TODO: Add native plotly sidecolor function.
  # TODO: Possibly use function to generate all 3 plots to prevent complex logic here
  if (is.null(row_side_colors)) {
    pr <- NULL
  } else {
    side_color_df <- x[["row_side_colors"]]
    if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
    assert_that(
      nrow(side_color_df) == nrow(data_mat),
      is.data.frame(side_color_df)
    )
    ## Just make sure it's character first
    side_color_df[] <- lapply(side_color_df, as.character)
    if (plot_method == "ggplot") {
      pr <- ggplot_side_color_plot(
        side_color_df,
        type = "row",
        text_angle = column_text_angle,
        palette = row_side_palette,
        fontsize = fontsize_col,
        is_colors = !is.null(RowSideColors),
        label_name = label_names[[1]]
      ) + side_color_layers
    } else {
      pr <- plotly_side_color_plot(
        side_color_df,
        type = "row",
        text_angle = column_text_angle,
        palette = row_side_palette,
        fontsize = fontsize_col,
        is_colors = !is.null(RowSideColors),
        colorbar_len = side_color_colorbar_len,
        label_name = label_names[[1]]
      )
    }
  }

  if (is.null(col_side_colors)) {
    pc <- NULL
  } else {
    side_color_df <- x[["col_side_colors"]]
    if (is.matrix(side_color_df)) side_color_df <- as.data.frame(side_color_df)
    assert_that(
      nrow(side_color_df) == ncol(data_mat),
      is.data.frame(side_color_df)
    )
    if (plot_method == "ggplot") {
      pc <- ggplot_side_color_plot(
        side_color_df,
        type = "column",
        text_angle = row_text_angle,
        palette = col_side_palette,
        is_colors = !is.null(ColSideColors),
        fontsize = fontsize_row,
        label_name = label_names[[2]]
      ) + side_color_layers
    } else {
      pc <- plotly_side_color_plot(
        side_color_df,
        type = "column",
        text_angle = row_text_angle,
        palette = col_side_palette,
        fontsize = fontsize_row,
        is_colors = !is.null(ColSideColors),
        colorbar_len = side_color_colorbar_len,
        label_name = label_names[[2]]
      )
    }
  }

  if (return_ppxpy) {
    return(list(p = p, px = px, py = py, pr = pr, pc = pc))
  } else {
    if (!is.null(pc)) {
      pc <- ggplotly(pc)
      pc <- layout(pc, showlegend = TRUE)
    }
    if (!is.null(pr)) {
      pr <- ggplotly(pr)
      pr <- layout(pr, showlegend = TRUE)
    }
  }

  ## plotly:
  # turn p, px, and py to plotly objects if necessary
  if (!is.plotly(p)) {
    p <- ggplotly(p, dynamicTicks = dynamicTicks, tooltip = "text") %>%
      layout(showlegend = FALSE)
      ## Currently broken, see:
      ##  https://github.com/ropensci/plotly/issues/1701
      # %>%
      # colorbar(
      #   len = colorbar_len,
      #   thickness = colorbar_thickness
      # )
  }

  if (draw_cellnote) {
    ## Predict cell color luminosity based on colorscale
    if (cellnote_color == "auto") {
      cellnote_color <- predict_colors(p, plot_method = plot_method)
    }
    df <- as.data.frame(x[["matrix"]][["cellnote"]])
    df[["_row"]] <- seq_len(nrow(df))
    mdf <- reshape2::melt(df, id.vars = "_row")
    ## TODO: Enforce same dimnames to ensure it's not scrambled?
    # mdf$variable <- factor(mdf$variable, levels = p$x$layout$xaxis$ticktext)
    mdf$variable <- as.numeric(as.factor(mdf$variable))
    mdf$value <- factor(mdf$value)
    p <- p %>% add_trace(
      data = mdf,
      x = ~ variable,
      y = ~ `_row`,
      text = ~ value,
      inherit = FALSE,
      type = "scatter",
      mode = "text",
      textposition = cellnote_textposition,
      hoverinfo = "none",
      showlegend = FALSE,
      textfont = list(color = plotly::toRGB(cellnote_color), size = cellnote_size)
    )
  }

  if (!is.null(px) && !is.plotly(px)) {
    px <- ggplotly(px,
                   tooltip = if (dend_hoverinfo) "y" else "none",
                   dynamicTicks = dynamicTicks) %>%
      layout(showlegend = FALSE)
  }
  if (!is.null(py) && !is.plotly(py)) {
    py <- ggplotly(py,
                   tooltip = if (dend_hoverinfo) "y" else "none",
                   dynamicTicks = dynamicTicks) %>%
      layout(showlegend = FALSE)
  }

  # https://plot.ly/r/reference/#Layout_and_layout_style_objects
  p <- layout(
    p, # all of layout's properties: /r/reference/#layout
    title = main, # layout's title: /r/reference/#layout-title
    xaxis = list( # layout's xaxis is a named list. List of valid keys: /r/reference/#layout-xaxis
      title = xlab # xaxis's title: /r/reference/#layout-xaxis-title
      # showgrid = T        # xaxis's showgrid: /r/reference/#layout-xaxis-showgrid
    ),
    yaxis = list( # layout's yaxis is a named list. List of valid keys: /r/reference/#layout-yaxis
      title = ylab # yaxis's title: /r/reference/#layout-yaxis-title
    )
  )
  if (hide_colorbar) {
    p <- hide_colorbar(p)
  }

  # Adjust top based on whether main is empty or not.
  if (is.na(margins[3])) margins[3] <- ifelse(main == "", 0, 30)

  rn <- c(rownames(data_mat), colnames(x[["col_side_colors"]]))
  min_marg_row <- calc_margin(rn, fontsize = p$x$layout$yaxis$tickfont$size)
  if (row_dend_left && is.na(margins[4])) {
    margins[4] <- min_marg_row
  } else if (!row_dend_left && is.na(margins[2])) {
    margins[2] <- min_marg_row
  }
  if (is.na(margins[1])) {
    cn <- c(colnames(data_mat), colnames(x[["row_side_colors"]]))
    margins[1] <- calc_margin(cn, fontsize = p$x$layout$yaxis$tickfont$size)
  }

  # add a white grid
  if (grid_gap > 0) {
    p <- style(p, xgap = grid_gap, ygap = grid_gap, traces = 1)
    # doesn't seem to work for ggplot2.
    if (plot_method == "plotly") {
      if (!is.null(pr)) {
        pr <- style(pr, ygap = grid_gap)
      }
      if (!is.null(pc)) {
        pc <- style(pc, xgap = grid_gap)
      }
    }
  }


  if (!all(showticklabels)) {
    if (!showticklabels[[1]]) {
      p <- p %>%
        layout(xaxis = list(
          showticklabels = FALSE,
          ticklen = 0
        ))
    }
    if (!showticklabels[[2]]) {
      p <- p %>%
        layout(yaxis = list(
          showticklabels = FALSE,
          ticklen = 0
        ))
    }
    # ggplotly() %>%
    # layout(yaxis = list(tickmode='auto'),
    #        xaxis = list(tickmode='auto'))
  }

  heatmap_subplot <- heatmap_subplot_from_ggplotly(
    p = p,
    px = px,
    py = py,
    row_dend_left = row_dend_left,
    subplot_margin = subplot_margin,
    widths = subplot_widths,
    heights = subplot_heights,
    titleX = titleX,
    titleY = titleY,
    pr = pr,
    pc = pc,
    plot_method = plot_method,
    showticklabels = showticklabels
  )
  l <- layout(
    heatmap_subplot,
    margin = list(l = margins[2], b = margins[1], t = margins[3], r = margins[4]),
    legend = list(y = 1, yanchor = "top")
  )

  # keep only relevant plotly options
  l <- config(
    l, displaylogo = FALSE,
    modeBarButtonsToRemove = c("sendDataToCloud", "select2d", "lasso2d", "autoScale2d", "hoverClosestCartesian", "hoverCompareCartesian", "sendDataToCloud")
  )

  l
}






default_dims <- function(px, pr) {
  if (!is.null(px)) {
    if (is.null(pr)) {
      widths <- c(0.8, 0.2)
    } else {
      widths <- c(0.7, 0.1, 0.2)
    }
  } else {
    if (is.null(pr)) {
      widths <- 1
    } else {
      widths <- c(0.9, 0.1)
    }
  }
  widths
}


heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
                                          row_dend_left = FALSE, subplot_margin = 0,
                                          titleX = TRUE, titleY = TRUE,
                                          widths=NULL, heights=NULL,
                                          plot_method,
                                          showticklabels = c(TRUE, TRUE)) {
  widths <- widths %||% default_dims(px, pr)
  if (row_dend_left) {
    widths <- rev(widths)
  }
  heights <- heights %||% rev(default_dims(py, pc))


  # make different plots based on which dendrogram and sidecolors we have
  row1_list <- list(py, plotly_empty(), plotly_empty())
  row2_list <- list(pc, plotly_empty(), plotly_empty())
  row3_list <- list(p, pr, px)

  if (row_dend_left) {
    row3_list <- rev(row3_list)
    row2_list <- rev(row2_list)
    row1_list <- rev(row1_list)
  }
  plots <- c(
    row1_list,
    row2_list,
    row3_list
  )

  column_list <- list(py, pc, p)
  ind_null_col <- sapply(column_list, is.null)
  ## number of rows depends on vertically aligned components
  nrows <- sum(!ind_null_col)
  ind_remove_col <- rep(ind_null_col, each = length(plots) / 3)

  ind_null_row <- sapply(row3_list, is.null)
  ind_remove_row <- rep(ind_null_row, length.out = length(plots))

  if (sum(!ind_null_col) != length(heights)) {
    stop(paste(
      "Number of subplot_heights supplied is not correct; should be",
      sum(!ind_null_col), "but is", length(heights)
    ))
  }
  if (sum(!ind_null_row) != length(widths)) {
    stop(paste(
      "Number of subplot_widths supplied is not correct; should be",
      sum(!ind_null_row), "but is", length(widths)
    ))
  }

  ## Remove all null plots
  plots <- plots[!(ind_remove_row | ind_remove_col)]

  ## Interim solution before removing warnings in documented way
  suppressMessages(
    suppressWarnings(
      s <- subplot(
        plots,
        nrows = nrows,
        widths = widths,
        shareX = TRUE,
        shareY = TRUE,
        titleX = titleX,
        titleY = titleY,
        margin = subplot_margin,
        heights = heights
      )
    )
  )

  if (plot_method == "plotly") {
    if (row_dend_left) {
      num_rows <- sum(!ind_null_row)
      str <- ifelse(num_rows > 1, num_rows, "")
      l <- list(
        anchor = paste0("x", str),
        side = "right",
        showticklabels = showticklabels[[2]]
      )
      num_cols <- sum(!ind_null_col)
      if (num_cols == 1) {
        lay <- function(p) layout(p, yaxis = l)
      } else if (num_cols == 2) {
        lay <- function(p) layout(p, yaxis2 = l)
      } else if (num_cols == 3) {
        lay <- function(p) layout(p, yaxis3 = l)
      }
      s <- lay(s)
    }
  }


  # s <- subplot(plots,
  #   nrows = nrows,
  #   widths = if(row_dend_left) rev(widths) else widths,
  #   shareX = TRUE, shareY = TRUE,
  #   titleX = titleX, titleY = titleY,
  #   margin = subplot_margin,
  #   heights = heights)


  return(s)
}


## TODO: Better/safer estimation of total size, or use monospace.
calc_margin <- function(labels, fontsize) {
  max(nchar(labels) * fontsize, na.rm = TRUE) * 0.6
  # http://stackoverflow.com/questions/19113725/what-dependency-between-font-size-and-width-of-char-in-monospace-font
}

#' @title Checks if an object is of class plotly or not.
#' @export
#' @description
#' Helpful for the plot_method in link{heatmaply}.
#'
#' @param x an object to check
#'
#' @return
#' TRUE if the object inherits "plotly" as a class.
#'
is.plotly <- function(x) {
  inherits(x, "plotly")
}

context("heatmaply misc")

test_that("node argument works", {
  rrapply <- function(A, FUN, ...) mapply(function(a, B) lapply(
        B,
        function(x) FUN(a, x, ...)
      ), a = A, MoreArgs = list(B = A))
  cor.tests <- rrapply(mtcars, cor.test) # a matrix of cor.tests
  p <- apply(cor.tests, 1:2, function(x) x[[1]]$p.value) # and it's there
  r <- cor(mtcars)
  h <- heatmaply(
    r, node_type = "scatter",
    point_size_name = "p",
    point_size_mat = -log10(p)
  )
  expect_is(heatmaply(mtcars, node_type = "scatter", point_size_mat = mtcars), "plotly")
})

test_that("non-numerics moved to row_side_colors", {
  mtcars[, ncol(mtcars) + 1] <- "a"
  h <- heatmaply(mtcars, row_side_colors = mtcars[, 1:2])
  expect_is(h, "plotly")
  h <- heatmaply(mtcars)
  expect_is(h, "plotly")
})


test_that("heatmaply on matrix, and cexRow/Col", {
  mtcars <- as.matrix(mtcars)
  rownames(mtcars) <- NULL
  h <- heatmaply(mtcars, cexRow = 5, cexCol = 5)
  expect_warning(heatmaply(mtcars, cexRow = "b", cexCol = "a"))

  expect_is(h, "plotly")
})


test_that("grid_color and hide_colorbar", {
  h <- heatmaply(mtcars[1:5, 1:5], grid_color = "black", hide_colorbar = TRUE)
  expect_is(h, "plotly")
})

test_that("return plots", {
  h <- heatmaply(mtcars, return_ppxpy = TRUE)
  sapply(h, function(el) expect_true(is.null(el) | inherits(el, "gg")))
})

test_that("SideColors", {
  rs <- mtcars[, 1]
  rscolors <- colorRampPalette(c("red", "blue"))(length(unique(rs)))
  rs <- rscolors[as.character(rs)]
  h <- heatmaply(mtcars, RowSideColors = rs)
  expect_is(h, "plotly")
  h <- heatmaply(t(mtcars), ColSideColors = rs)
  expect_is(h, "plotly")
})

test_that("limits", {
  h <- heatmaply(t(mtcars), limits = range(as.matrix(mtcars)))
  expect_is(h, "plotly")
  expect_warning(h <- heatmaply(t(mtcars), limits = c(0, 0)))
  expect_is(h, "plotly")
  expect_error(heatmaply(t(mtcars), limits = c("a", "b")))
  expect_error(heatmaply(t(mtcars), limits = 1))
})

test_that("cellnote", {
  h <- heatmaply(mtcars, draw_cellnote = TRUE)
  expect_is(h, "plotly")
})

test_that("showticklabels", {
  h <- heatmaply(mtcars, showticklabels = TRUE)
  expect_is(h, "plotly")
  h <- heatmaply(mtcars, showticklabels = c(FALSE, FALSE))
  expect_is(h, "plotly")
  expect_error(
    heatmaply(mtcars, showticklabels = "a"),
    "showticklabels must be a logical vector of length 2 or 1"
  )
})

## This relies on PhantomJS... it's failing a lot so comment it
# test_that("file argument works", {
#   lapply(c("png", "jpeg", "pdf", "html"), function(type) {
#     file <- paste0("tmp.", type)
#     if (type=="pdf") {
#         expect_warning(heatmaply(mtcars, file=file))
#     } else {
#         heatmaply(mtcars, file=file)
#     }
#     expect_true(file.exists(file))
#   })
# })

test_that("correlation arguments work", {
  lapply(c("pearson", "spearman", "kendall"), function(method) {
    expect_is(heatmaply(mtcars, distfun = method), "plotly")
  })
})

test_that("long_data works", {
  mdf <- reshape2::melt(as.matrix(mtcars))
  colnames(mdf) <- c("name", "variable", "value")
  expect_is(heatmaply(long_data = mdf), "plotly")
  expect_error(heatmaply(mtcars, long_data = mdf),
    "x and long_data should not be used together")
})

test_that("heatmaply_na works", {
  m <- as.matrix(mtcars)
  m[1:5] <- NA
  expect_is(heatmaply_na(m), "plotly")
})

test_that("heatmaply_cor works", {
  m <- cor(as.matrix(mtcars))
  expect_is(heatmaply_cor(m), "plotly")
})


test_that("dengrogram=TRUE works", {
  expect_is(heatmaply(mtcars, dendrogram = TRUE), "plotly")
})

test_that("limits warning", {
  expect_warning(
    heatmaply(mtcars, limits = c(4, 5)),
    "Lower limit is not*"
  )
})

test_that("grid_gap works", {
  expect_is(heatmaply(mtcars, grid_gap = 1), "plotly")
})


test_that("subplot_* needs to be correct", {
  expect_error(heatmaply(mtcars, subplot_heights = rep(1, 10)))
  expect_error(heatmaply(mtcars, subplot_widths = rep(1, 10)))
})

test_that("custom_hovertext works", {
  mat <- as.matrix(mtcars)
  suppressWarnings(mat[] <- letters)
  for (plot_method in c("plotly", "ggplot")) {
    expect_error(heatmaply(mtcars, plot_method = plot_method, custom_hovertext = mat), NA)
  }
})

test_that("hclust_method=NA works", {
  expect_error(heatmaply(mtcars, hclust_method=NA), NA)
})

for (plot_method in c("ggplot", "plotly")) {
  for (bool in c(TRUE, FALSE)) {
    context(paste0(plot_method, ", row_dend_left=", bool))

    test_that("heatmaply mtcars (both dend)", {
      h <- heatmaply(
        mtcars, row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (no dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "none", row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (coldend only)", {
      h <- heatmaply(
        mtcars, dendrogram = "col", row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rowdend only)", {
      h <- heatmaply(
        mtcars, dendrogram = "row", row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })


    test_that("heatmaply mtcars (rscols, both dend)", {
      h <- heatmaply(
        mtcars,
        row_side_colors = mtcars[, 1:2], row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })


    test_that("heatmaply mtcars (rscols, row dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "none",
        row_side_colors = mtcars[, 1:2],
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rscols, row dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "col",
        row_side_colors = mtcars[, 1:2],
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rscols, no dends)", {
      h <- heatmaply(
        mtcars, dendrogram = "none",
        row_side_colors = mtcars[, 1:2], row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (cscols, both dend)", {
      h <- heatmaply(
        mtcars,
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (cscols, col dend)", {
      h <- heatmaply(
        mtcars, 
        dendrogram = "col",
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (cscols, row dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "row",
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (cscols, no dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "none",
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rcscols, both dend)", {
      h <- heatmaply(
        mtcars,
        row_side_colors = mtcars[, 1:2],
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rcscols, col dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "col",
        row_side_colors = mtcars[, 1:2],
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rcscols, row dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "row",
        row_side_colors = mtcars[, 1:2],
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })

    test_that("heatmaply mtcars (rcscols, no dend)", {
      h <- heatmaply(
        mtcars, dendrogram = "none",
        row_side_colors = mtcars[, 1:2],
        col_side_colors = data.frame(t(mtcars[1:2, ])),
        row_dend_left = bool,
        plot_method = plot_method
      )
      expect_is(h, "plotly")
    })
  }
}

#' @title Indicates which elements are missing (either 1 and 0)
#' @export
#' @description
#' is.na10 is a helper function for creating heatmaps to diagnose missing value patterns.
#' It is similar to \link{is.na} but instead of returning a logical TRUE/FALSE vector (or matrix) it
#' returns a numeric 1/0 output. This enables the \link{heatmaply} function to be used on the data.
#'
#' @param x a vector, matrix or data.frame.
#' @param ... not used.
#'
#' @seealso \link{is.na}, the grid_gap parameter in \link{heatmaply}.
#' @return
#' Returns a numeric (instead of a logical) variable/matrix of 1 (missing) or 0 (not missing) values (hence the name is.na10)
#' while still preserving the attributes resulted from running \link{is.na}.
#'
#' These are useful for funnelling into a heatmap (see the examples).
#'
#' @examples
#' \dontrun{
#' x <- mtcars
#' x <- data.frame(x)
#' x$am <- factor(x$am)
#' x$vs <- factor(x$vs)
#' set.seed(2017-01-19)
#' x[sample(nrow(x))[1:6],sample(ncol(x))[1:6]] <- NA
#'
#'
#' # nice grey colors from here: https://github.com/njtierney/visdat/blob/master/R/vis_miss_ly.R
#' x %>% is.na10 %>% heatmaply( colors = c("grey80", "grey20"), dendrogram = "none")
#' x %>% is.na10 %>% heatmaply( colors = c("grey80", "grey20"), k_col = 2, k_row = 2)
#'
#' heatmaply(is.na10(airquality), grid_gap = 1,
#'          colors = c("grey80", "grey20"), k_col = 2, k_row = 2)
#'
#' }
is.na10 <- function(x, ...) {
  # x %>% is.na %>% str
  # x %>% is.na %>% as.numeric

  # get NA
  x_na <- is.na(x)

  # turn numeric (whlie keeping the attributes!)
  mat_attr <- attributes(x_na)
  x_na <- as.integer(x_na)
  attributes(x_na) <- mat_attr

  # return
  x_na
}

---
title: "Introduction to heatmaply"
date: "`r Sys.Date()`"
author: "Tal Galili and Alan O'Callaghan"
output:
 html_document:
  fig_width: 12
  fig_height: 9
  self_contained: yes
  toc: true
editor_options: 
  chunk_output_type: console
---
<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Introduction to heatmaply}
-->


```{r, echo = FALSE, message = FALSE}
library("heatmaply")
library("knitr")
knitr::opts_chunk$set(
  warning=FALSE, # because of a bug in plotly: https://github.com/ropensci/plotly/issues/1670
  # see also: https://github.com/talgalili/heatmaply/issues/226
  # cache = TRUE,
  dpi = 60,
  comment = "#>",
  tidy = FALSE)

# http://stackoverflow.com/questions/24091735/why-pandoc-does-not-retrieve-the-image-file
# < ! -- rmarkdown v1 -->

```


**Author**: [Tal Galili](https://www.r-statistics.com/) (Tal.Galili@gmail.com)



Introduction
============

A heatmap is a popular graphical method for visualizing high-dimensional data,
in which a table of numbers are encoded as a grid of colored cells. The rows 
and columns of the matrix are ordered to highlight patterns and are often 
accompanied by dendrograms. Heatmaps are used in many fields for visualizing 
observations, correlations, missing values patterns, and more.

Interactive heatmaps allow the inspection of specific value by hovering the 
mouse over a cell, as well as zooming into a region of the heatmap by dragging 
a rectangle around the relevant area.

This work is based on ggplot2 and plotly.js engine. It produces similar 
heatmaps as d3heatmap, with the advantage of speed (plotly.js is able to 
handle larger size matrix), and the ability to zoom from the dendrogram.

heatmaply also provides an interface based around the 
[plotly R package](https://CRAN.R-project.org/package=plotly).
This interface can be used by choosing `plot_method = "plotly"` instead of
the default `plot_method = "ggplot"`. This interface can provide smaller
objects and faster rendering to disk in many cases and provides otherwise almost
identical features.

Documentation for this package is also available as a 
[pkgdown](https://cran.r-project.org/package=pkgdown)
site: <http://talgalili.github.io/heatmaply/>

Installation
============

To install the stable version on CRAN:

```{r, eval = FALSE}
install.packages('heatmaply')
```

To install the GitHub version:

```{r, eval = FALSE}
# You'll need devtools
install.packages.2 <- function (pkg) if (!require(pkg)) install.packages(pkg);
install.packages.2('remotes')

remotes::install_github("ropensci/plotly") 
remotes::install_github('talgalili/heatmaply')
```


And then you may load the package using:

```{r}
library("heatmaply")
```

Basic usage
===========


Default
-------

The default settings in heatmaply attempt to be both useful yet not too computationally intensive. Here is an example based on the `mtcars` dataset:

> The data was extracted from the 1974 Motor Trend US magazine, and comprises fuel consumption and 10 aspects of automobile design and performance for 32 automobiles (1973-74 models).


```{r}
library("heatmaply")
heatmaply(mtcars)
```

<!-- The margins parameter
-------------


By default heatmaply tries to fix the margins of the plot based on the length of the labels, but this system is not perfect. Hence, we sometimes need to manually fix the margins (hopefully this will be fixed in future versions of plotly). This is also helpful when including xlab/ylab/main texts.

```{r}
heatmaply(mtcars, xlab = "Features", ylab = "Cars", 
  main = "An example of title and xlab/ylab")
```
 -->

Correlation heatmaps
--------------------

We can use the margins parameter with correlation heatmaps. heatmaply includes
the `heatmaply_cor` function, which is a wrapper around `heatmaply` with 
arguments optimised for use with correlation matrices. Notice how we color 
the branches (see dendextend::color_branches for further detail on `k_row` and 
`k_col`):

```{r}
heatmaply_cor(
  cor(mtcars),
  xlab = "Features",
  ylab = "Features",
  k_col = 2,
  k_row = 2
)
```

We can also do a more advanced correlation heatmap where the p-value from
the correlation test is mapped to point size: 

```{r}

r <- cor(mtcars)
## We use this function to calculate a matrix of p-values from correlation tests
## https://stackoverflow.com/a/13112337/4747043
cor.test.p <- function(x){
    FUN <- function(x, y) cor.test(x, y)[["p.value"]]
    z <- outer(
      colnames(x), 
      colnames(x), 
      Vectorize(function(i,j) FUN(x[,i], x[,j]))
    )
    dimnames(z) <- list(colnames(x), colnames(x))
    z
}
p <- cor.test.p(mtcars)

heatmaply_cor(
  r,
  node_type = "scatter",
  point_size_mat = -log10(p), 
  point_size_name = "-log10(p-value)",
  label_names = c("x", "y", "Correlation")
)
```



Data transformation (scaling, normalize, and percentize)
========================================================

The variables in mtcars includes values that reflect different types of 
measurement, each with its own range (and meaning) of values. In such a case, 
it is best to transform the data so to have all the variables have comparable 
values.

scale
-----

If we would assume all variables come from some normal distribution, then 
scaling (i.e.: subtract the mean and divide by the standard deviation) would 
bring them all close to the standard normal distribution. In such a case, each 
value would reflect the distance from the mean in units of standard deviation. 
The "scale" parameter in heatmaply supports column and row scaling, and can be 
used as follows:

(not evaluated so to reduce vignette size)

```{r, eval = F}
heatmaply(
  mtcars, 
  xlab = "Features",
  ylab = "Cars", 
  scale = "column",
  main = "Data transformation using 'scale'"
)
```

normalize
---------



When variables in the data comes from possibly different (and non-normal)
distributions, other transformations may be in order. For example, scaling on 
a binary variable with many 0's and just a few 1's will lead to a column with a 
very extreme value that would cause the color legend to be very skewed by that 
variable, leading to a masking of the distribution of the rest of the variables.

Another possibility is to use the `normalize` function to brings data to the 0 
to 1 scale by subtracting the minimum and dividing by the maximum of all 
observations. This preserves the shape of each variable's distribution while 
making them easily comparable on the same "scale". Using the function on 
mtcars easily reveals columns with only two (`am`, `vs`) or three (`gear`, 
`cyl`) variables compared with variables that have a higher resolution of 
possible values:

(not evaluated so to reduce vignette size)

```{r, eval = F}
heatmaply(
  normalize(mtcars),
  xlab = "Features",
  ylab = "Cars", 
  main = "Data transformation using 'normalize'"
)
```

percentize
----------

An alternative to `normalize` is the `percentize` function. This is similar 
to ranking the variables, but instead of keeping the rank values, divide them 
by the maximal rank. This is done by using the `ecdf` of the variables on their 
own values, bringing each value to its  empirical percentile. The benefit of 
the percentize function is that each value has a relatively clear 
interpretation, it is the percent of observations with that value or below 
it.

```{r}
heatmaply(
  percentize(mtcars),
  xlab = "Features",
  ylab = "Cars", 
  main = "Data transformation using 'percentize'"
)
```

Notice that for binary variables (0 and 1), `percentize` will turn all 0 values
to their proportion and all 1 values will remain 1. This means the 
transformation is not symmatric for 0 and 1. Hence, if scaling for clustering, 
it might be better to use `rank` for dealing with tie values (if no ties 
are present, then `percentize` will perform similarly to `rank`).



is.na10 (missing values)
------------------------

Reviewing missing values can easily be done using the `is.na10` function. 
When using it with heatmaply, it is often helpful to use a non-zero `grid_gap`
to place gaps between cells of the heatmap. Similar to `heatmaply_cor`,
`heatmaply_na` is a wrapper around `heatmaply` with arguments optimised for the
visualisation of missingness:

```{r}
heatmaply_na(
  airquality[1:30, ],
  showticklabels = c(TRUE, FALSE),
  k_col = 3,
  k_row = 3
)


# warning - using grid_color cannot handle a large matrix! For example:
# airquality[1:10, ] %>% is.na10 %>% 
#   heatmaply(color = c("white", "black"), grid_color = "grey",
#             k_col =3, k_row = 3,
#             margins = c(40, 50)) 
# airquality %>% is.na10 %>% 
#   heatmaply(color = c("grey80", "grey20"), # grid_color = "grey",
#             k_col =3, k_row = 3,
#             margins = c(40, 50)) 
# 

```




Changing color palettes
-----------------------

We can use colors other than the default `viridis`. The packages
[`cetcolor`](https://CRAN.R-project.org/package=cetcolor) and 
[`RColorBrewer`](https://CRAN.R-project.org/package=RColorBrewer)
provide a number of excellent options for continuous and discrete colour 
palettes. These are generally designed to be perceptually uniform, and often
also colorblind-friendly.

For example, we may want to use other color palettes in order to get divergent colors for the correlations (these will, sadly, often be less useful for colorblind people). These come by default when using `heatmaply_cor`

(not evaluated so to reduce vignette size)

```{r, eval = F}
heatmaply_cor(
  cor(mtcars),
  k_col = 2, 
  k_row = 2
)

```


Another example for using colors:


```{r, eval = FALSE}
heatmaply(
  percentize(mtcars),
  colors = heat.colors(100)
)
```

Or even more customized colors using `scale_fill_gradient_fun`:

```{r}
heatmaply(
  mtcars,
  scale_fill_gradient_fun = ggplot2::scale_fill_gradient2(
    low = "blue", 
    high = "red", 
    midpoint = 200, 
    limits = c(0, 500)
  )
)

```



Customized dendrograms and annotation
=====================================


Various seriation options
-------------------------

heatmaply uses the `seriation` package to find an optimal ordering of rows 
and columns. Optimal means to optimize the Hamiltonian path length that 
is restricted by the dendrogram structure. This, in other words, means to 
rotate the branches so that the sum of distances between each adjacent leaf 
(label) will be minimized. This is related to a restricted version of the 
travelling salesman problem. 

The default options is `"OLO"` (Optimal leaf ordering) which optimizes the 
above criterion (in O(n^4)). Another option is `"GW"`
([Gruvaeus and Wainer](https://doi.org/10.1111/j.2044-8317.1972.tb00491.x)) 
which aims for the same goal but uses a potentially faster heuristic. The 
option `"mean"` gives the output we would get by default from heatmap functions 
in other packages such as `gplots::heatmap.2`. The option `"none"` gives us 
the dendrograms without any rotation that is based on the data matrix. 

```{r}
# The default of heatmaply:
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "OLO"
)
```

(not evaluated so to reduce vignette size)
```{r, eval = F}
# Similar to OLO but less optimal (since it is a heuristic)
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "GW"
)
```

(not evaluated so to reduce vignette size)
```{r, eval = F}
# the default by gplots::heatmaply.2
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "mean"
)
```


```{r, eval = F}
# the default output from hclust
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "none"
)
```

This works heavily relies on the `seriation` package (their 
[vignette](https://CRAN.R-project.org/package=seriation/vignettes/seriation.pdf)
is well worth the read), and also lightly on the `dendextend` package (see
[vignette](http://talgalili.github.io/dendextend/articles/dendextend.html) )


Customized dendrograms using dendextend
---------------------------------------

A user can supply their own dendrograms for the rows/columns of the heatmaply
using the `Rowv` and the `Colv` parameters:

```{r}

x  <- as.matrix(datasets::mtcars)

# now let's spice up the dendrograms a bit:
library("dendextend")

row_dend  <- x %>% 
  dist %>% 
  hclust %>% 
  as.dendrogram %>%
  set("branches_k_color", k = 3) %>% 
  set("branches_lwd", c(1, 3)) %>%
  ladderize
# rotate_DendSer(ser_weight = dist(x))
col_dend  <- x %>% 
  t %>% 
  dist %>% 
  hclust %>% 
  as.dendrogram %>%
  set("branches_k_color", k = 2) %>% 
  set("branches_lwd", c(1, 2)) %>%
  ladderize
#    rotate_DendSer(ser_weight = dist(t(x)))

heatmaply(
  percentize(x),
  Rowv = row_dend,
  Colv = col_dend
)


```



Replicating the dendrogram ordering of heatmap.2
------------------------------------------------

The following example shows how to get the same result in heatmaply as with 
`gplots::heatmap.2`:

```{r}
x  <- as.matrix(datasets::mtcars)
gplots::heatmap.2(
  x,
  trace = "none",
  col = viridis(100),
  key = FALSE
)
```

With heatmaply, the only difference is the side of the row dendrogram. 
This is because the `ggplotly` function from plotly does not (yet) handle 
axes placed in different locations than the default.

```{r, eval = FALSE}
heatmaply(
  x,
  seriate = "mean"
)
```

We can get a more similar version using:

```{r}
heatmaply(x,
  seriate = "mean", 
  row_dend_left = TRUE,
  plot_method = "plotly"
)
```

Some aspects of heatmaply may not function identically when using 
`plot_method = "plotly"` relative to how they function when using
`plot_method = "ggplot"` (the default), however in the majority of cases
the output is equivalent. Using this option results in faster heatmaps capable 
of handling larger matrices.



Adding annotation based on additional factors using `RowSideColors`
-------------------------------------------------------------------

With heatmap.2

```{r}
# Example for using RowSideColors

x  <- as.matrix(datasets::mtcars)
rc <- colorspace::rainbow_hcl(nrow(x))

library("gplots")
library("viridis")
heatmap.2(
  x,
  trace = "none",
  col = viridis(100),
  RowSideColors = rc,
  key = FALSE
)

```

With `heatmaply`:

```{r, eval = FALSE}
heatmaply(
  x,
  seriate = "mean",
  RowSideColors = rc
)
```

A more sophisticated heatmap:

```{r}
heatmaply(
  x[, -c(8, 9)],
  seriate = "mean",
  col_side_colors = c(rep(0, 5), rep(1, 4)),
  row_side_colors = x[, 8:9]
)

```

Text annotations
----------------

`heatmaply` includes the `cellnote` argument, which allows us to display
character values overlaid on the heatmap. By default, the colour of the text
on each cell is chosen to ensure legibility, with black text shown over light
cells and white text shown over dark cells.

```{r}
heatmaply(
  mtcars,
  cellnote = mtcars
)
```


Hover text
----------

As hovertext is one of the most useful aspects of interactive heatmaps,
`heatmaply` allows users to append custom hovertext:

```{r}
mat <- mtcars
mat[] <- paste("This cell is", rownames(mat))
mat[] <- lapply(colnames(mat), function(colname) {
    paste0(mat[, colname], ", ", colname)
})
heatmaply(
  mtcars,
  custom_hovertext = mat
)
```


ggheatmap
---------

`heatmaply` also has the option to produce (static) vector graphic equivalents using
the excellent [`egg`](https://CRAN.R-project.org/package=egg) 
package. This feature is somewhat experimental, and all feedback and 
contributions to improve it are highly appreciated.

```{r}
ggheatmap(
  mtcars,
  scale = "column",
  row_side_colors = mtcars[, c("cyl", "gear")]
)
```


Saving your heatmaply into a file
=================================

You can save an interactive version of your `heatmaply` into an HTML file using
the following code:

```{r, eval = FALSE}
dir.create("folder")
heatmaply(mtcars, file = "folder/heatmaply_plot.html")
browseURL("folder/heatmaply_plot.html")
```


Similar code can be used for saving a static file (png/jpeg/pdf)

```{r, eval = FALSE}
dir.create("folder")
# Before the first time using this code you may need to first run:
# webshot::install_phantomjs() or to install 
# [plotly's orca](https://github.com/plotly/orca) program.
heatmaply(mtcars, file = "folder/heatmaply_plot.png")
browseURL("folder/heatmaply_plot.png")
```

If you only wish to save the file, without plotting it in the console,
you can assign the output to a temporary object name:

```{r, eval = FALSE}
# This saves the file, but does not plot it in the RStudio viewer
tmp <- heatmaply(mtcars, file = "folder/heatmaply_plot.png")
rm(tmp)
```


Sidenotes
=========

We considered using the fastcluster package for clustering, but the time gain 
was too small compared to the rest of the bottlenecks in the package.

```{r, eval = FALSE}

library("microbenchmark")


library("heatmaply")
x <- matrix(1:1000, 500, 2)

microbenchmark(
  heatmaply(x, hclustfun = stats::hclust),
  heatmaply(x, hclustfun = fastcluster::hclust),
  times = 10
)

x <- matrix(1:1000, 1000, 2)
microbenchmark(
  stats::hclust(dist(x)),
  fastcluster::hclust(dist(x)),
  times = 10
)


```








Credit
======

This package is thanks to the amazing work done by many people in the open 
source community. Beyond the many people working on the pipeline of R, thanks 
should go to the plotly team, and especially to Carson Sievert and others 
working on the R package of plotly. Also, many of the design elements were 
inspired by the work done on heatmap, heatmap.2 and d3heatmap, so special 
thanks goes to the R core team, Gregory R. Warnes, and Joe Cheng from RStudio. 
The dendrogram side of the package is based on the work in dendextend, in which
special thanks should go to Andrie de Vries for his original work on the 
ggdendro package was the first to bring dendrograms to ggplot2 (this later 
evolved into the richer ggdend objects, as implemented in dendextend). 
Thanks should also go to Alan O'Callaghan for his many contributions to getting 
the package to work better with plotly, as well as for Jonathan Sidi for his 
work on the shinyHeatmply package. Lastely, my thanks goes to Yoav Benjamini 
for his support and helpful comments on this work.


Contact
=======

You are welcome to:

* submit suggestions and bug-reports at: 
  <https://github.com/talgalili/heatmaply/issues>
* send a pull request on: <https://github.com/talgalili/heatmaply/>
* compose a friendly e-mail to: <tal.galili@gmail.com>


Latest news
===========

You can see the most recent changes to the package in the [NEWS.md file](https://talgalili.github.io/heatmaply/news/)



Session info
============

```{r, cache = FALSE}
sessionInfo()
```

---
title: "Introduction to heatmaply"
date: "`r Sys.Date()`"
author: "Tal Galili and Alan O'Callaghan"
output:
 html_document:
  fig_width: 12
  fig_height: 9
  self_contained: yes
  toc: true
editor_options: 
  chunk_output_type: console
---
<!--
%\VignetteEngine{knitr::rmarkdown}
%\VignetteIndexEntry{Introduction to heatmaply}
-->


```{r, echo = FALSE, message = FALSE}
library("heatmaply")
library("knitr")
knitr::opts_chunk$set(
  warning=FALSE, # because of a bug in plotly: https://github.com/ropensci/plotly/issues/1670
  # see also: https://github.com/talgalili/heatmaply/issues/226
  # cache = TRUE,
  dpi = 60,
  comment = "#>",
  tidy = FALSE)

# http://stackoverflow.com/questions/24091735/why-pandoc-does-not-retrieve-the-image-file
# < ! -- rmarkdown v1 -->

```


**Author**: [Tal Galili](https://www.r-statistics.com/) (Tal.Galili@gmail.com)



Introduction
============

A heatmap is a popular graphical method for visualizing high-dimensional data,
in which a table of numbers are encoded as a grid of colored cells. The rows 
and columns of the matrix are ordered to highlight patterns and are often 
accompanied by dendrograms. Heatmaps are used in many fields for visualizing 
observations, correlations, missing values patterns, and more.

Interactive heatmaps allow the inspection of specific value by hovering the 
mouse over a cell, as well as zooming into a region of the heatmap by dragging 
a rectangle around the relevant area.

This work is based on ggplot2 and plotly.js engine. It produces similar 
heatmaps as d3heatmap, with the advantage of speed (plotly.js is able to 
handle larger size matrix), and the ability to zoom from the dendrogram.

heatmaply also provides an interface based around the 
[plotly R package](https://CRAN.R-project.org/package=plotly).
This interface can be used by choosing `plot_method = "plotly"` instead of
the default `plot_method = "ggplot"`. This interface can provide smaller
objects and faster rendering to disk in many cases and provides otherwise almost
identical features.

Documentation for this package is also available as a 
[pkgdown](https://cran.r-project.org/package=pkgdown)
site: <http://talgalili.github.io/heatmaply/>

Installation
============

To install the stable version on CRAN:

```{r, eval = FALSE}
install.packages('heatmaply')
```

To install the GitHub version:

```{r, eval = FALSE}
# You'll need devtools
install.packages.2 <- function (pkg) if (!require(pkg)) install.packages(pkg);
install.packages.2('remotes')

remotes::install_github("ropensci/plotly") 
remotes::install_github('talgalili/heatmaply')
```


And then you may load the package using:

```{r}
library("heatmaply")
```

Basic usage
===========


Default
-------

The default settings in heatmaply attempt to be both useful yet not too computationally intensive. Here is an example based on the `mtcars` dataset:

> The data was extracted from the 1974 Motor Trend US magazine, and comprises fuel consumption and 10 aspects of automobile design and performance for 32 automobiles (1973-74 models).


```{r}
library("heatmaply")
heatmaply(mtcars)
```

<!-- The margins parameter
-------------


By default heatmaply tries to fix the margins of the plot based on the length of the labels, but this system is not perfect. Hence, we sometimes need to manually fix the margins (hopefully this will be fixed in future versions of plotly). This is also helpful when including xlab/ylab/main texts.

```{r}
heatmaply(mtcars, xlab = "Features", ylab = "Cars", 
  main = "An example of title and xlab/ylab")
```
 -->

Correlation heatmaps
--------------------

We can use the margins parameter with correlation heatmaps. heatmaply includes
the `heatmaply_cor` function, which is a wrapper around `heatmaply` with 
arguments optimised for use with correlation matrices. Notice how we color 
the branches (see dendextend::color_branches for further detail on `k_row` and 
`k_col`):

```{r}
heatmaply_cor(
  cor(mtcars),
  xlab = "Features",
  ylab = "Features",
  k_col = 2,
  k_row = 2
)
```

We can also do a more advanced correlation heatmap where the p-value from
the correlation test is mapped to point size: 

```{r}

r <- cor(mtcars)
## We use this function to calculate a matrix of p-values from correlation tests
## https://stackoverflow.com/a/13112337/4747043
cor.test.p <- function(x){
    FUN <- function(x, y) cor.test(x, y)[["p.value"]]
    z <- outer(
      colnames(x), 
      colnames(x), 
      Vectorize(function(i,j) FUN(x[,i], x[,j]))
    )
    dimnames(z) <- list(colnames(x), colnames(x))
    z
}
p <- cor.test.p(mtcars)

heatmaply_cor(
  r,
  node_type = "scatter",
  point_size_mat = -log10(p), 
  point_size_name = "-log10(p-value)",
  label_names = c("x", "y", "Correlation")
)
```



Data transformation (scaling, normalize, and percentize)
========================================================

The variables in mtcars includes values that reflect different types of 
measurement, each with its own range (and meaning) of values. In such a case, 
it is best to transform the data so to have all the variables have comparable 
values.

scale
-----

If we would assume all variables come from some normal distribution, then 
scaling (i.e.: subtract the mean and divide by the standard deviation) would 
bring them all close to the standard normal distribution. In such a case, each 
value would reflect the distance from the mean in units of standard deviation. 
The "scale" parameter in heatmaply supports column and row scaling, and can be 
used as follows:

(not evaluated so to reduce vignette size)

```{r, eval = F}
heatmaply(
  mtcars, 
  xlab = "Features",
  ylab = "Cars", 
  scale = "column",
  main = "Data transformation using 'scale'"
)
```

normalize
---------



When variables in the data comes from possibly different (and non-normal)
distributions, other transformations may be in order. For example, scaling on 
a binary variable with many 0's and just a few 1's will lead to a column with a 
very extreme value that would cause the color legend to be very skewed by that 
variable, leading to a masking of the distribution of the rest of the variables.

Another possibility is to use the `normalize` function to brings data to the 0 
to 1 scale by subtracting the minimum and dividing by the maximum of all 
observations. This preserves the shape of each variable's distribution while 
making them easily comparable on the same "scale". Using the function on 
mtcars easily reveals columns with only two (`am`, `vs`) or three (`gear`, 
`cyl`) variables compared with variables that have a higher resolution of 
possible values:

(not evaluated so to reduce vignette size)

```{r, eval = F}
heatmaply(
  normalize(mtcars),
  xlab = "Features",
  ylab = "Cars", 
  main = "Data transformation using 'normalize'"
)
```

percentize
----------

An alternative to `normalize` is the `percentize` function. This is similar 
to ranking the variables, but instead of keeping the rank values, divide them 
by the maximal rank. This is done by using the `ecdf` of the variables on their 
own values, bringing each value to its  empirical percentile. The benefit of 
the percentize function is that each value has a relatively clear 
interpretation, it is the percent of observations with that value or below 
it.

```{r}
heatmaply(
  percentize(mtcars),
  xlab = "Features",
  ylab = "Cars", 
  main = "Data transformation using 'percentize'"
)
```

Notice that for binary variables (0 and 1), `percentize` will turn all 0 values
to their proportion and all 1 values will remain 1. This means the 
transformation is not symmatric for 0 and 1. Hence, if scaling for clustering, 
it might be better to use `rank` for dealing with tie values (if no ties 
are present, then `percentize` will perform similarly to `rank`).



is.na10 (missing values)
------------------------

Reviewing missing values can easily be done using the `is.na10` function. 
When using it with heatmaply, it is often helpful to use a non-zero `grid_gap`
to place gaps between cells of the heatmap. Similar to `heatmaply_cor`,
`heatmaply_na` is a wrapper around `heatmaply` with arguments optimised for the
visualisation of missingness:

```{r}
heatmaply_na(
  airquality[1:30, ],
  showticklabels = c(TRUE, FALSE),
  k_col = 3,
  k_row = 3
)


# warning - using grid_color cannot handle a large matrix! For example:
# airquality[1:10, ] %>% is.na10 %>% 
#   heatmaply(color = c("white", "black"), grid_color = "grey",
#             k_col =3, k_row = 3,
#             margins = c(40, 50)) 
# airquality %>% is.na10 %>% 
#   heatmaply(color = c("grey80", "grey20"), # grid_color = "grey",
#             k_col =3, k_row = 3,
#             margins = c(40, 50)) 
# 

```




Changing color palettes
-----------------------

We can use colors other than the default `viridis`. The packages
[`cetcolor`](https://CRAN.R-project.org/package=cetcolor) and 
[`RColorBrewer`](https://CRAN.R-project.org/package=RColorBrewer)
provide a number of excellent options for continuous and discrete colour 
palettes. These are generally designed to be perceptually uniform, and often
also colorblind-friendly.

For example, we may want to use other color palettes in order to get divergent colors for the correlations (these will, sadly, often be less useful for colorblind people). These come by default when using `heatmaply_cor`

(not evaluated so to reduce vignette size)

```{r, eval = F}
heatmaply_cor(
  cor(mtcars),
  k_col = 2, 
  k_row = 2
)

```


Another example for using colors:


```{r, eval = FALSE}
heatmaply(
  percentize(mtcars),
  colors = heat.colors(100)
)
```

Or even more customized colors using `scale_fill_gradient_fun`:

```{r}
heatmaply(
  mtcars,
  scale_fill_gradient_fun = ggplot2::scale_fill_gradient2(
    low = "blue", 
    high = "red", 
    midpoint = 200, 
    limits = c(0, 500)
  )
)

```



Customized dendrograms and annotation
=====================================


Various seriation options
-------------------------

heatmaply uses the `seriation` package to find an optimal ordering of rows 
and columns. Optimal means to optimize the Hamiltonian path length that 
is restricted by the dendrogram structure. This, in other words, means to 
rotate the branches so that the sum of distances between each adjacent leaf 
(label) will be minimized. This is related to a restricted version of the 
travelling salesman problem. 

The default options is `"OLO"` (Optimal leaf ordering) which optimizes the 
above criterion (in O(n^4)). Another option is `"GW"`
([Gruvaeus and Wainer](https://www.researchgate.net/publication/230266994_Two_additions_to_hierarchical_cluster_analysis)) 
which aims for the same goal but uses a potentially faster heuristic. The 
option `"mean"` gives the output we would get by default from heatmap functions 
in other packages such as `gplots::heatmap.2`. The option `"none"` gives us 
the dendrograms without any rotation that is based on the data matrix. 

```{r}
# The default of heatmaply:
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "OLO"
)
```

(not evaluated so to reduce vignette size)
```{r, eval = F}
# Similar to OLO but less optimal (since it is a heuristic)
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "GW"
)
```

(not evaluated so to reduce vignette size)
```{r, eval = F}
# the default by gplots::heatmaply.2
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "mean"
)
```


```{r, eval = F}
# the default output from hclust
heatmaply(
  percentize(mtcars)[1:10, ],
  seriate = "none"
)
```

This works heavily relies on the `seriation` package (their 
[vignette](https://CRAN.R-project.org/package=seriation/vignettes/seriation.pdf)
is well worth the read), and also lightly on the `dendextend` package (see
[vignette](http://talgalili.github.io/dendextend/articles/dendextend.html) )


Customized dendrograms using dendextend
---------------------------------------

A user can supply their own dendrograms for the rows/columns of the heatmaply
using the `Rowv` and the `Colv` parameters:

```{r}

x  <- as.matrix(datasets::mtcars)

# now let's spice up the dendrograms a bit:
library("dendextend")

row_dend  <- x %>% 
  dist %>% 
  hclust %>% 
  as.dendrogram %>%
  set("branches_k_color", k = 3) %>% 
  set("branches_lwd", c(1, 3)) %>%
  ladderize
# rotate_DendSer(ser_weight = dist(x))
col_dend  <- x %>% 
  t %>% 
  dist %>% 
  hclust %>% 
  as.dendrogram %>%
  set("branches_k_color", k = 2) %>% 
  set("branches_lwd", c(1, 2)) %>%
  ladderize
#    rotate_DendSer(ser_weight = dist(t(x)))

heatmaply(
  percentize(x),
  Rowv = row_dend,
  Colv = col_dend
)


```



Replicating the dendrogram ordering of heatmap.2
------------------------------------------------

The following example shows how to get the same result in heatmaply as with 
`gplots::heatmap.2`:

```{r}
x  <- as.matrix(datasets::mtcars)
gplots::heatmap.2(
  x,
  trace = "none",
  col = viridis(100),
  key = FALSE
)
```

With heatmaply, the only difference is the side of the row dendrogram. 
This is because the `ggplotly` function from plotly does not (yet) handle 
axes placed in different locations than the default.

```{r, eval = FALSE}
heatmaply(
  x,
  seriate = "mean"
)
```

We can get a more similar version using:

```{r}
heatmaply(x,
  seriate = "mean", 
  row_dend_left = TRUE,
  plot_method = "plotly"
)
```

Some aspects of heatmaply may not function identically when using 
`plot_method = "plotly"` relative to how they function when using
`plot_method = "ggplot"` (the default), however in the majority of cases
the output is equivalent. Using this option results in faster heatmaps capable 
of handling larger matrices.



Adding annotation based on additional factors using `RowSideColors`
-------------------------------------------------------------------

With heatmap.2

```{r}
# Example for using RowSideColors

x  <- as.matrix(datasets::mtcars)
rc <- colorspace::rainbow_hcl(nrow(x))

library("gplots")
library("viridis")
heatmap.2(
  x,
  trace = "none",
  col = viridis(100),
  RowSideColors = rc,
  key = FALSE
)

```

With `heatmaply`:

```{r, eval = FALSE}
heatmaply(
  x,
  seriate = "mean",
  RowSideColors = rc
)
```

A more sophisticated heatmap:

```{r}
heatmaply(
  x[, -c(8, 9)],
  seriate = "mean",
  col_side_colors = c(rep(0, 5), rep(1, 4)),
  row_side_colors = x[, 8:9]
)

```

Text annotations
----------------

`heatmaply` includes the `cellnote` argument, which allows us to display
character values overlaid on the heatmap. By default, the colour of the text
on each cell is chosen to ensure legibility, with black text shown over light
cells and white text shown over dark cells.

```{r}
heatmaply(
  mtcars,
  cellnote = mtcars
)
```


Hover text
----------

As hovertext is one of the most useful aspects of interactive heatmaps,
`heatmaply` allows users to append custom hovertext:

```{r}
mat <- mtcars
mat[] <- paste("This cell is", rownames(mat))
mat[] <- lapply(colnames(mat), function(colname) {
    paste0(mat[, colname], ", ", colname)
})
heatmaply(
  mtcars,
  custom_hovertext = mat
)
```


ggheatmap
---------

`heatmaply` also has the option to produce (static) vector graphic equivalents using
the excellent [`egg`](https://CRAN.R-project.org/package=egg) 
package. This feature is somewhat experimental, and all feedback and 
contributions to improve it are highly appreciated.

```{r}
ggheatmap(
  mtcars,
  scale = "column",
  row_side_colors = mtcars[, c("cyl", "gear")]
)
```


Saving your heatmaply into a file
=================================

You can save an interactive version of your `heatmaply` into an HTML file using
the following code:

```{r, eval = FALSE}
dir.create("folder")
heatmaply(mtcars, file = "folder/heatmaply_plot.html")
browseURL("folder/heatmaply_plot.html")
```


Similar code can be used for saving a static file (png/jpeg/pdf)

```{r, eval = FALSE}
dir.create("folder")
# Before the first time using this code you may need to first run:
# webshot::install_phantomjs() or to install 
# [plotly's orca](https://github.com/plotly/orca) program.
heatmaply(mtcars, file = "folder/heatmaply_plot.png")
browseURL("folder/heatmaply_plot.png")
```

If you only wish to save the file, without plotting it in the console,
you can assign the output to a temporary object name:

```{r, eval = FALSE}
# This saves the file, but does not plot it in the RStudio viewer
tmp <- heatmaply(mtcars, file = "folder/heatmaply_plot.png")
rm(tmp)
```


Sidenotes
=========

We considered using the fastcluster package for clustering, but the time gain 
was too small compared to the rest of the bottlenecks in the package.

```{r, eval = FALSE}

library("microbenchmark")


library("heatmaply")
x <- matrix(1:1000, 500, 2)

microbenchmark(
  heatmaply(x, hclustfun = stats::hclust),
  heatmaply(x, hclustfun = fastcluster::hclust),
  times = 10
)

x <- matrix(1:1000, 1000, 2)
microbenchmark(
  stats::hclust(dist(x)),
  fastcluster::hclust(dist(x)),
  times = 10
)


```








Credit
======

This package is thanks to the amazing work done by many people in the open 
source community. Beyond the many people working on the pipeline of R, thanks 
should go to the plotly team, and especially to Carson Sievert and others 
working on the R package of plotly. Also, many of the design elements were 
inspired by the work done on heatmap, heatmap.2 and d3heatmap, so special 
thanks goes to the R core team, Gregory R. Warnes, and Joe Cheng from RStudio. 
The dendrogram side of the package is based on the work in dendextend, in which
special thanks should go to Andrie de Vries for his original work on the 
ggdendro package was the first to bring dendrograms to ggplot2 (this later 
evolved into the richer ggdend objects, as implemented in dendextend). 
Thanks should also go to Alan O'Callaghan for his many contributions to getting 
the package to work better with plotly, as well as for Jonathan Sidi for his 
work on the shinyHeatmply package. Lastely, my thanks goes to Yoav Benjamini 
for his support and helpful comments on this work.


Contact
=======

You are welcome to:

* submit suggestions and bug-reports at: 
  <https://github.com/talgalili/heatmaply/issues>
* send a pull request on: <https://github.com/talgalili/heatmaply/>
* compose a friendly e-mail to: <tal.galili@gmail.com>


Latest news
===========

You can see the most recent changes to the package in the [NEWS.md file](https://talgalili.github.io/heatmaply/news/)



Session info
============

```{r, cache = FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/percentize.R
\name{percentize}
\alias{percentize}
\alias{percentize.default}
\alias{percentize.data.frame}
\alias{percentize.matrix}
\title{Empirical Percentile Transformation}
\usage{
percentize(x, ...)
}
\arguments{
\item{x}{a vector or a data.frame.}

\item{...}{Currently ignored.}
}
\value{
A vector (or data.frame) after \link{ecdf} was used on that vector.
If x is a \link{data.frame} then only the numeric variables are transformed.
}
\description{
An Empirical Percentile Transformation (percentize) is similar to taking the rank
of a variable. The difference is that it is simpler to compare and interpret
the transformed variables.

This is helpful for comparing several variables in a heatmap (e.g.: \link{heatmaply}).
}
\examples{
\dontrun{
x <- mtcars
x <- data.frame(x)
x$am <- factor(x$am)
x$vs <- factor(x$vs)
heatmaply(percentize(x))


x <- data.frame(a = 1:10, b = 11:20)
x[4:6, 1:2] <- NA
percentize(x)
percentize(x[,1])

}
}
\seealso{
\link{normalize}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmaply.R
\name{heatmaply}
\alias{heatmaply}
\alias{heatmaply_na}
\alias{heatmaply_cor}
\alias{heatmaply.default}
\alias{heatmaply.heatmapr}
\title{Cluster heatmap based on plotly}
\usage{
heatmaply(x, ...)

heatmaply_na(x, grid_gap = 1, colors = c("grey80", "grey20"), ...)

heatmaply_cor(x, limits = c(-1, 1), colors = cool_warm, ...)

\method{heatmaply}{default}(
  x,
  colors = viridis(n = 256, alpha = 1, begin = 0, end = 1, option = "viridis"),
  limits = NULL,
  na.value = "grey50",
  row_text_angle = 0,
  column_text_angle = 45,
  subplot_margin = 0,
  cellnote = NULL,
  draw_cellnote = !is.null(cellnote),
  cellnote_color = "auto",
  cellnote_textposition = "middle right",
  cellnote_size = 12,
  Rowv,
  Colv,
  distfun = dist,
  hclustfun = hclust,
  dist_method = NULL,
  hclust_method = NULL,
  distfun_row,
  hclustfun_row,
  distfun_col,
  hclustfun_col,
  dendrogram = c("both", "row", "column", "none"),
  show_dendrogram = c(TRUE, TRUE),
  reorderfun = function(d, w) reorder(d, w),
  k_row = 1,
  k_col = 1,
  symm = FALSE,
  revC,
  scale = c("none", "row", "column"),
  na.rm = TRUE,
  row_dend_left = FALSE,
  margins = c(NA, NA, NA, NA),
  ...,
  scale_fill_gradient_fun = NULL,
  grid_color = NA,
  grid_gap = 0,
  srtRow,
  srtCol,
  xlab = "",
  ylab = "",
  main = "",
  titleX = TRUE,
  titleY = TRUE,
  hide_colorbar = FALSE,
  key.title = NULL,
  return_ppxpy = FALSE,
  row_side_colors,
  row_side_palette = NULL,
  col_side_colors,
  col_side_palette = NULL,
  ColSideColors = NULL,
  RowSideColors = NULL,
  seriate = c("OLO", "mean", "none", "GW"),
  heatmap_layers = NULL,
  side_color_layers = NULL,
  dendrogram_layers = NULL,
  branches_lwd = 0.6,
  file,
  width = NULL,
  height = NULL,
  long_data,
  plot_method = c("ggplot", "plotly"),
  label_names = NULL,
  fontsize_row = 10,
  fontsize_col = 10,
  cexRow,
  cexCol,
  subplot_widths = NULL,
  subplot_heights = NULL,
  colorbar_len = 0.3,
  colorbar_thickness = 30,
  colorbar_xanchor = if (row_dend_left) "right" else "left",
  colorbar_yanchor = "bottom",
  colorbar_xpos = if (row_dend_left) -0.1 else 1.1,
  colorbar_ypos = 0,
  showticklabels = c(TRUE, TRUE),
  dynamicTicks = FALSE,
  grid_size = 0.1,
  node_type = "heatmap",
  point_size_mat = NULL,
  point_size_name = "Point size",
  label_format_fun = function(...) format(..., digits = 4),
  labRow = NULL,
  labCol = NULL,
  custom_hovertext = NULL,
  col = NULL,
  dend_hoverinfo = TRUE,
  side_color_colorbar_len = 0.3
)

\method{heatmaply}{heatmapr}(
  x,
  colors = viridis(n = 256, alpha = 1, begin = 0, end = 1, option = "viridis"),
  limits = NULL,
  na.value = "grey50",
  row_text_angle = 0,
  column_text_angle = 45,
  subplot_margin = 0,
  row_dend_left = FALSE,
  margins = c(NA, NA, NA, NA),
  ...,
  scale_fill_gradient_fun = scale_fill_gradientn(colors = if (is.function(colors))
    colors(256) else colors, na.value = na.value, limits = limits),
  grid_color = NA,
  grid_gap = 0,
  srtRow,
  srtCol,
  xlab = "",
  ylab = "",
  main = "",
  titleX = TRUE,
  titleY = TRUE,
  hide_colorbar = FALSE,
  key.title = NULL,
  return_ppxpy = FALSE,
  draw_cellnote = FALSE,
  cellnote_color = "auto",
  cellnote_textposition = "middle right",
  cellnote_size = 12,
  row_side_colors = x[["row_side_colors"]],
  row_side_palette = NULL,
  col_side_colors = x[["col_side_colors"]],
  col_side_palette = NULL,
  plot_method = c("ggplot", "plotly"),
  ColSideColors = NULL,
  RowSideColors = NULL,
  heatmap_layers = NULL,
  side_color_layers = NULL,
  dendrogram_layers = NULL,
  branches_lwd = 0.6,
  label_names = c("row", "column", "value"),
  fontsize_row = 10,
  fontsize_col = 10,
  subplot_widths = NULL,
  subplot_heights = NULL,
  colorbar_xanchor = if (row_dend_left) "right" else "left",
  colorbar_yanchor = "bottom",
  colorbar_xpos = if (row_dend_left) -0.1 else 1.1,
  colorbar_ypos = 0,
  colorbar_len = 0.3,
  colorbar_thickness = 30,
  showticklabels = c(TRUE, TRUE),
  dynamicTicks = FALSE,
  node_type = c("scatter", "heatmap"),
  grid_size = 0.1,
  point_size_mat = x[["matrix"]][["point_size_mat"]],
  point_size_name = "Point size",
  label_format_fun = function(...) format(..., digits = 4),
  custom_hovertext = x[["matrix"]][["custom_hovertext"]],
  dend_hoverinfo = TRUE,
  side_color_colorbar_len = 0.3
)
}
\arguments{
\item{x}{can either be a heatmapr object, or a numeric matrix
Defaults to \code{TRUE} unless \code{x} contains any \code{NA}s.}

\item{...}{other parameters passed to \link{heatmapr} (currently, various
parameters may be ignored.}

\item{grid_gap}{this is a fast alternative to grid_color. The default is 0, but if a larger value
is used (for example, 1), then the resulting heatmap will have a white grid which can
help identify different cells. This is implemented using \link[plotly]{style} (with xgap and ygap).}

\item{colors, col}{a vector of colors to use for heatmap color.
The default uses
\code{\link[viridis]{viridis}(n=256, alpha = 1, begin = 0, end = 1, option = "viridis")}
It is passed to \link[ggplot2]{scale_fill_gradientn}.
If colors is a color function (with the first argument being `n` = the number of colors),
it will be used to create 256 colors from that function.
(col is there to stay compatible with \link[gplots]{heatmap.2})}

\item{limits}{a two dimensional numeric vector specifying the data range for the scale.}

\item{na.value}{color to use for missing values (default is "grey50").}

\item{row_text_angle}{numeric (Default is 0), the angle of the text of the
rows. (this is called srtRow in \link[gplots]{heatmap.2})}

\item{column_text_angle}{numeric (Default is 45), the angle of the text of
the columns. (this is called srtCol in \link[gplots]{heatmap.2})}

\item{subplot_margin}{Currently not well implemented. It is passed to
\link[plotly]{subplot}. Default is 0. Either a single value or
 four values (all between 0 and 1). If four values are provided,
 the first is used as the left margin, the second is used as the right margin,
 the third is used as the top margin, and the fourth is used as the bottom margin.
 If a single value is provided, it will be used as all four margins.}

\item{cellnote}{Values to be shown as annotations atop the heatmap cells.}

\item{draw_cellnote}{Should the cellnote annotations be drawn? Defaults is FALSE,
if cellnote is not supplied, TRUE if cellnote is supplied. If TRUE and
cellnote is not supplied, x will be used for cellnote.}

\item{cellnote_color}{The color of the cellnote text to be used.}

\item{cellnote_textposition}{The text positioning/centering of the cellnote.
Default is "middle right". Options are
"top left", "top center", "top right", "middle left", "middle center",
"middle right", "bottom left", "bottom center", "bottom right"}

\item{cellnote_size}{The font size (HTML/CSS) of the cellnote. Default is 12.}

\item{Rowv}{determines if and how the row dendrogram should be reordered.
By default, it is TRUE, which implies dendrogram is computed and reordered
based on row means. If NULL or FALSE, then no dendrogram is computed and
no reordering is done. If a \link{dendrogram} (or \link{hclust}),
then it is used "as-is", ie without any reordering. If a vector of integers,
then dendrogram is computed and reordered based on the order of the vector.}

\item{Colv}{determines if and how the column dendrogram should be reordered.
Has the options as the Rowv argument above and additionally when x is a
square matrix, Colv = "Rowv" means that columns should be treated
identically to the rows.}

\item{distfun}{function used to compute the distance (dissimilarity)
between both rows and columns. Defaults to dist.
The options "pearson", "spearman" and "kendall" can be used to
use correlation-based clustering, which uses \code{as.dist(1 - cor(t(x)))}
as the distance metric (using the specified correlation method).}

\item{hclustfun}{function used to compute the hierarchical clustering
when Rowv or Colv are not dendrograms. Defaults to hclust.}

\item{dist_method}{default is NULL (which results in "euclidean" to be used).
Can accept alternative character strings indicating the
method to be passed to distfun. By default distfun. is \link{dist} hence
this can be one of "euclidean", "maximum", "manhattan", "canberra", "binary"
or "minkowski".}

\item{hclust_method}{default is NULL (which results in "complete" to be used).
Can accept alternative character strings indicating the
method to be passed to hclustfun By default hclustfun is \link{hclust} hence
this can be one of "ward.D", "ward.D2", "single", "complete", "average"
(= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).
Specifying hclust_method=NA causes heatmaply to use
\code{\link[dendextend]{find_dend}} to find the "optimal" dendrogram for
the data.}

\item{distfun_row}{distfun for row dendrogram only.}

\item{hclustfun_row}{hclustfun for col dendrogram only.}

\item{distfun_col}{distfun for row dendrogram only.}

\item{hclustfun_col}{hclustfun for col dendrogram only.}

\item{dendrogram}{character string indicating whether to compute 'none',
'row', 'column' or 'both' dendrograms. Defaults to 'both'.
However, if Rowv (or Colv) is FALSE or NULL and dendrogram is 'both',
then a warning is issued and Rowv (or Colv) arguments are honoured.
It also accepts TRUE/FALSE as synonyms for "both"/"none".}

\item{show_dendrogram}{Logical vector of length two, controlling whether
the row and/or column dendrograms are displayed. If a logical scalar is
provided, it is repeated to become a logical vector of length two.}

\item{reorderfun}{function(d, w) of dendrogram and weights for reordering the
row and column dendrograms. The default uses stats{reorder.dendrogram}}

\item{k_row}{an integer scalar with the desired number of groups by which to
color the dendrogram's branches in the rows (uses \link[dendextend]{color_branches})
If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.}

\item{k_col}{an integer scalar with the desired number of groups by which to
color the dendrogram's branches in the columns (uses \link[dendextend]{color_branches})
If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.}

\item{symm}{logical indicating if x should be treated symmetrically; can only
be true when x is a square matrix.}

\item{revC}{logical indicating if the column order should be reversed for plotting.
Default (when missing) - is FALSE, unless symm is TRUE.
This is useful for cor matrix.}

\item{scale}{character indicating if the values should be centered and scaled
in either the row direction or the column direction, or none. The default is
"none".}

\item{na.rm}{logical (default is TRUE) indicating whether NA's should be
removed when scaling (i.e.: when using rowMeans/colMeans). Generally it
should always be kept as TRUE, and is included here mainly to stay backward
compatible with gplots::heatmap.2. This argument does not effect the presence
of NA values in the matrix itself. For removing rows/columns with NAs you
should pre-process your matrix using na.omit (or some form of imputation).}

\item{row_dend_left}{logical (default is FALSE). Should the row dendrogram be
plotted on the left side of the heatmap. If false then it will be plotted on
the right side.}

\item{margins}{numeric vector of length 4 (default is c(50,50,NA,0))
containing the margins (see \link[plotly]{layout}) for column, row and main
title names, respectively. The top margin is NA by default. If main==""
then the top margin will be set to 0, otherwise it will get 30.
For a multiline title a larger default for the 3rd element should be set.
The right margin is NA by default, meaning it will be zero if row_dend_left
is FALSE, or 100 if row_dend_left is TRUE.}

\item{scale_fill_gradient_fun}{A function that creates a smooth gradient for the heatmap.
The default uses \link[ggplot2]{scale_fill_gradientn} with the values of colors, limits, and
na.value that are supplied by the user. The user can input a customized function, such as
\link{scale_color_gradient}() in order to get other results (although the virids default
is quite recommended)}

\item{grid_color}{control the color of the heatmap grid. Default is NA.
Value passed to \link[ggplot2]{geom_tile}. Do not use this parameter on
larger matrix sizes, as it can dramatically prolong the build time of the heatmap.
(another parameter, grid_color, will be added in the future - once it is implemented in plotly)
In the meantime it is MUCH better to use the grid_gap argument.}

\item{srtRow}{if supplied, this overrides row_text_angle (this is to stay compatible with \link[gplots]{heatmap.2})}

\item{srtCol}{if supplied, this overrides column_text_angle (this is to stay compatible with \link[gplots]{heatmap.2})}

\item{xlab}{A character title for the x axis.}

\item{ylab}{A character title for the y axis.}

\item{main}{A character title for the heatmap.}

\item{titleX}{logical (TRUE). should x-axis titles be retained? (passed to \link[plotly]{subplot}).}

\item{titleY}{logical (TRUE). should y-axis titles be retained? (passed to \link[plotly]{subplot}).}

\item{hide_colorbar}{logical (FALSE). If TRUE, then the color bar (i.e.: the legend) is hidden.}

\item{key.title}{(character) main title of the color key. If set to NULL (default) no title will be plotted.}

\item{return_ppxpy}{logical (FALSE). If TRUE, then no plotting is done and the p, px and py objects are
returned (before turning into plotly objects). This is a temporary option which might be removed in the
future just to make it easy to create a ggplot heatmaps.}

\item{row_side_colors, col_side_colors}{data.frame of factors to produce
row/column side colors in the style of heatmap.2/heatmap.3.
When a data.frame is provided, the column names are used as the label names for each of the newly added row_side_colors.
When a vector is provided it is coerced into a data.frame and the name of the side color will be just row_side_colors.}

\item{row_side_palette, col_side_palette}{Color palette functions to be
used for row_side_colors and col_side_colors respectively.}

\item{ColSideColors, RowSideColors}{passed to row_side_colors,col_side_colors in order
to keep compatibility with \link[gplots]{heatmap.2}}

\item{seriate}{character indicating the method of matrix sorting (default: "OLO").
Implemented options include:
"OLO" (Optimal leaf ordering, optimizes the Hamiltonian path length that is
restricted by the dendrogram structure - works in O(n^4) )
"mean" (sorts the matrix based on the reorderfun using marginal means of
the matrix. This is the default used by \link[gplots]{heatmap.2}),
"none" (the default order produced by the dendrogram),
"GW" (Gruvaeus and Wainer heuristic to optimize the Hamiltonian path length
that is restricted by the dendrogram structure)}

\item{heatmap_layers}{ggplot object(s) (eg, list(theme_bw())) to be added to
the heatmap before conversion to a plotly object.}

\item{side_color_layers}{ggplot2 objects to be added to side color plots,
similar to heatmap_layers.}

\item{dendrogram_layers}{ggplot2 objects to be added to dendrograms,
similar to heatmap_layers and side_color_layers.}

\item{branches_lwd}{numeric (default is 0.6). The width of the dendrograms' branches.
If NULL then it is ignored. If the "lwd" is already defined in Rowv/Colv then this
parameter is ignored (it is checked using \link[dendextend]{has_edgePar}("lwd")).}

\item{file}{name of the file(s) into which to save the heatmaply output.
Should be a character vector of strings ending with ".html" for a dynamic output,
or ".png", ".jpeg", ".pdf" for a static output.

For example: heatmaply(x, file = "heatmaply_plot.html") or
dir.create("folder");
heatmaply(x, file = "folder/heatmaply_plot.html")

This is based on \link[htmlwidgets]{saveWidget}, and \link[webshot]{webshot} for the static files.
For more refined control over the static file output, you should save the heatmaply object using \link[plotly]{export} and pass the
arguments you want based on the ones in \link[webshot]{webshot}.

Another example: heatmaply(x, file = c("heatmaply_plot.html", "heatmaply_plot.png"))}

\item{width, height}{The width and height of the image saved. Default is
which uses 800 x 500 pixels.}

\item{long_data}{Data in long format. Replaces x, so both should not be used.
Colnames must be c("name", "variable", "value"). If you do not have a names
column you can simply use a sequence of numbers from 1 to the number of "rows"
in the data.}

\item{plot_method}{Use "ggplot" or "plotly" to choose which library produces heatmap
and dendrogram plots}

\item{label_names}{Names for labels of x, y and value/fill mouseover.}

\item{fontsize_row, fontsize_col, cexRow, cexCol}{Font size for row and column labels.}

\item{subplot_widths, subplot_heights}{The relative widths and heights of each
subplot. The length of these vectors will vary depending on the number of
plots involved.}

\item{colorbar_len}{The length of the colorbar/color key relative to the total
plot height. Only used if plot_method = "plotly"}

\item{colorbar_thickness}{The thickness (width) of the colorbar/color key
in pixels. Only used if plot_method = "plotly".}

\item{colorbar_xanchor, colorbar_yanchor}{The x and y anchoring points of the
colorbar/color legend. Can be "left", "middle" or "right" for colorbar_xanchor,
and "top", "middle" or "bottom" for colorbar_yanchor.
See \code{\link[plotly]{colorbar}} for more details.}

\item{colorbar_xpos, colorbar_ypos}{The x and y co-ordinates (in proportion of the plot window)
of the colorbar/color legend. See \code{\link[plotly]{colorbar}} for more details.}

\item{showticklabels}{A logical vector of length two (default is TRUE).
If FALSE, then the ticks are removed from the sides of the plot. The first location refers to
the x axis and the second to the y axis. If only one value is supplied (TRUE/FALSE) then it is
replicated to get to length 2. When using this parameter, it might be worth also adjusting
margins.
This option should be used when working with medium to large matrix size as it
makes the heatmap much faster (and the hover still works).}

\item{dynamicTicks}{(default: FALSE). passed to \link[plotly]{ggplotly}:
should plotly.js dynamically generate axis tick labels?
Dynamic ticks are useful for updating ticks in response to zoom/pan interactions; however,
they can not always reproduce labels as they would appear in the static ggplot2 image.}

\item{grid_size}{When node_type is "scatter", this controls point size. When
node_type is "heatmap", this controls the size of the grid between heatmap cells.}

\item{node_type}{For plot_method = "ggplot", should the heatmap be rendered as
a x-y scatter plot (node_type = "scatter") or a heatmap (node_type = "heatmap").
Default is node_type = "heatmap".}

\item{point_size_mat}{Matrix to map to point size}

\item{point_size_name}{Name of point size mapping (for hovertext/legend)}

\item{label_format_fun}{Function to format hovertext (eg,
\code{function(...) round(..., digits=3)} or
\code{function(...) format(..., digits=3)}}

\item{labRow, labCol}{character vectors with row and column labels to use;
these default to rownames(x) or colnames(x), respectively.
if set to NA, they change the value in showticklabels to be FALSE. This is mainly to keep
backward compatibility with gplots::heatmap.2.}

\item{custom_hovertext}{Custom hovertext matrix (the same dimensions as the input).
If plot_method is "plotly" then just this text is displayed; if plot_method
if "ggplot" then it is appended to the existing text.}

\item{dend_hoverinfo}{Boolean value which controls whether mouseover text
is shown for the row and column dendrograms.}

\item{side_color_colorbar_len}{As with colorbar_len, this controls the
length of the colorbar/color key relative to the total plot height.
This argument controls the colorbar_len of the side colour plots.
Only used if plot_method = "plotly".}
}
\description{
An object of class heatmapr includes all the needed information
for producing a heatmap. The goal is to separate the pre-processing of the
heatmap elements from the graphical rendering of the object, which could be done

(Please submit an issue on github if you have a feature that you wish to have added)

heatmaply_na is a wrapper for `heatmaply` which comes with defaults that are better
for exploring missing value (NA) patterns. Specifically, the grid_gap is set to 1, and the
colors include two shades of grey. It also calculates the \link{is.na10} automatically.

heatmaply_cor is a wrapper for `heatmaply` which comes with defaults that are better
for correlation matrixes. Specifically, the limits are set from -1 to 1, and the color palette is \link{RdBu}.
}
\examples{

\dontrun{

# mtcars
# x <- heatmapr(mtcars)
library(heatmaply)
heatmaply(iris[,-5], k_row = 3, k_col = 2)
heatmaply(cor(iris[,-5]))
heatmaply(cor(iris[,-5]), limits = c(-1,1))
heatmaply(mtcars, k_row = 3, k_col = 2)
# heatmaply(mtcars, k_row = 3, k_col = 2, grid_color = "white")
heatmaply(mtcars, k_row = 3, k_col = 2, grid_gap = 1)

# make sure there is enough room for the labels:
heatmaply(mtcars, margins = c(40, 130))
# this is the same as using:
heatmaply(mtcars) \%>\% layout(margin = list(l = 130, b = 40))

# control text angle
heatmaply(mtcars, column_text_angle = 90, margins = c(40, 130))
# the same as using srtCol:
# heatmaply(mtcars, srtCol = 90) \%>\% layout(margin = list(l = 130, b = 40))



x <- mtcars
# different colors
heatmaply(x, colors = heat.colors(200))
# using special scale_fill_gradient_fun colors
heatmaply(x, scale_fill_gradient_fun = scale_color_gradient())


# We can join two heatmaps together:
library(heatmaply)
hm1 <- heatmaply(mtcars, margins = c(40, 130))
hm2 <- heatmaply(mtcars, scale = "col", margins = c(40, 130))
subplot(hm1, hm2, margin = .2)

# If we want to share the Y axis, then it is risky to keep any of the dendrograms:
library(heatmaply)
hm1 <- heatmaply(mtcars, Colv = FALSE, Rowv = FALSE, margins = c(40, 130))
hm2 <- heatmaply(mtcars, scale = "col" , Colv = FALSE, Rowv = FALSE,
             margins = c(40, 130))
subplot(hm1, hm2, margin = .02, shareY = TRUE)


# We can save heatmaply as an HTML file by using:
heatmaply(iris[,-5], file = "heatmaply_iris.html")
# or a png/pdf/jpeg file using:
heatmaply(iris[,-5], file = "heatmaply_iris.png")
# or just doing it in one go:
heatmaply(iris[,-5], file = c("heatmaply_iris.html", "heatmaply_iris.png") )



# If we don't want the HTML to be selfcontained, we can use the following:
library(heatmaply)
library(htmlwidgets)
heatmaply(iris[,-5]) \%>\%
   saveWidget(file="heatmaply_iris.html",selfcontained = FALSE)


# Example for using RowSideColors

x  <- as.matrix(datasets::mtcars)
rc <- colorspace::rainbow_hcl(nrow(x))

library(gplots)
library(viridis)
heatmap.2(x, trace = "none", col = viridis(100),
          RowSideColors=rc)

heatmaply(x, seriate = "mean",
          RowSideColors=rc)


heatmaply(x[,-c(8,9)], seriate = "mean",
          col_side_colors = c(rep(0,5), rep(1,4)),
          row_side_colors = x[,8:9])
heatmaply(x[,-c(8,9)], seriate = "mean",
          col_side_colors = data.frame(a=c(rep(0,5), rep(1,4))),
          row_side_colors = x[,8:9])


## Example of using Rowv And Colv for custumized dendrograms.


x  <- as.matrix(datasets::mtcars)

# now let's spice up the dendrograms a bit:
library(dendextend)

row_dend  <- x \%>\% dist \%>\% hclust \%>\% as.dendrogram \%>\%
  set("branches_k_color", k = 3) \%>\% set("branches_lwd", 4) \%>\%
  ladderize
#    rotate_DendSer(ser_weight = dist(x))
col_dend  <- x \%>\% t \%>\% dist \%>\% hclust \%>\% as.dendrogram \%>\%
  set("branches_k_color", k = 2) \%>\% set("branches_lwd", 4) \%>\%
  ladderize
#    rotate_DendSer(ser_weight = dist(t(x)))

heatmaply(x, Rowv = row_dend, Colv = col_dend)


heatmaply(is.na10(airquality))
heatmaply(is.na10(airquality), grid_gap = 1)

# grid_gap can handle quite large data matrix
heatmaply(matrix(1:10000,100,100), k_row = 3, k_col = 3, grid_gap = 1)

# Examples of playing with font size:
heatmaply(mtcars, fontsize_col = 20, fontsize_row = 5, margin = c(100,90))



# Example for using subplot_width/subplot_height

heatmaply(percentize(mtcars),
     subplot_widths=c(0.6, 0.4),
     subplot_heights=c(0.05, 0.95))



# Example of removing labels and thus making the plot faster
heatmaply(iris, showticklabels = c(T,F), margins = c(80,10))

# this is what allows for a much larger matrix to be printed:
set.seed(2017-05-18)
large_x <- matrix(rnorm(19), 1000,100)
heatmaply(large_x, dendrogram = F, showticklabels = F, margins = c(1,1))


}
\dontrun{
heatmaply_na(airquality)
}
\dontrun{
heatmaply_cor(cor(mtcars))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.na10.R
\name{is.na10}
\alias{is.na10}
\title{Indicates which elements are missing (either 1 and 0)}
\usage{
is.na10(x, ...)
}
\arguments{
\item{x}{a vector, matrix or data.frame.}

\item{...}{not used.}
}
\value{
Returns a numeric (instead of a logical) variable/matrix of 1 (missing) or 0 (not missing) values (hence the name is.na10)
while still preserving the attributes resulted from running \link{is.na}.

These are useful for funnelling into a heatmap (see the examples).
}
\description{
is.na10 is a helper function for creating heatmaps to diagnose missing value patterns.
It is similar to \link{is.na} but instead of returning a logical TRUE/FALSE vector (or matrix) it
returns a numeric 1/0 output. This enables the \link{heatmaply} function to be used on the data.
}
\examples{
\dontrun{
x <- mtcars
x <- data.frame(x)
x$am <- factor(x$am)
x$vs <- factor(x$vs)
set.seed(2017-01-19)
x[sample(nrow(x))[1:6],sample(ncol(x))[1:6]] <- NA


# nice grey colors from here: https://github.com/njtierney/visdat/blob/master/R/vis_miss_ly.R
x \%>\% is.na10 \%>\% heatmaply( colors = c("grey80", "grey20"), dendrogram = "none")
x \%>\% is.na10 \%>\% heatmaply( colors = c("grey80", "grey20"), k_col = 2, k_row = 2)

heatmaply(is.na10(airquality), grid_gap = 1,
         colors = c("grey80", "grey20"), k_col = 2, k_row = 2)

}
}
\seealso{
\link{is.na}, the grid_gap parameter in \link{heatmaply}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmapr.R
\name{heatmapr}
\alias{heatmapr}
\title{Creates a heatmapr object}
\source{
The interface was designed based on \link{heatmap}, \link[gplots]{heatmap.2}, and (the also d3heatmap).
}
\usage{
heatmapr(
  x,
  Rowv,
  Colv,
  distfun = dist,
  hclustfun = hclust,
  dist_method = NULL,
  hclust_method = NULL,
  distfun_row,
  hclustfun_row,
  distfun_col,
  hclustfun_col,
  dendrogram = c("both", "row", "column", "none"),
  show_dendrogram = c(TRUE, TRUE),
  reorderfun = function(d, w) reorder(d, w),
  k_row = 1,
  k_col = 1,
  symm = FALSE,
  revC,
  scale = c("none", "row", "column"),
  na.rm = TRUE,
  labRow = rownames(x),
  labCol = colnames(x),
  cexRow,
  cexCol,
  digits = 3L,
  cellnote = NULL,
  theme = NULL,
  colors = "RdYlBu",
  width = NULL,
  height = NULL,
  xaxis_height = 80,
  yaxis_width = 120,
  xaxis_font_size = NULL,
  yaxis_font_size = NULL,
  brush_color = "#0000FF",
  show_grid = TRUE,
  anim_duration = 500,
  row_side_colors,
  col_side_colors,
  seriate = c("OLO", "mean", "none", "GW"),
  point_size_mat = NULL,
  custom_hovertext = NULL,
  ...
)
}
\arguments{
\item{x}{A numeric matrix
Defaults to \code{TRUE} unless \code{x} contains any \code{NA}s.}

\item{Rowv}{determines if and how the row dendrogram should be reordered.    By default, it is TRUE, which implies dendrogram is computed and reordered based on row means. If NULL or FALSE, then no dendrogram is computed and no reordering is done. If a \link{dendrogram} (or \link{hclust}), then it is used "as-is", ie without any reordering. If a vector of integers, then dendrogram is computed and reordered based on the order of the vector.}

\item{Colv}{determines if and how the column dendrogram should be reordered.    Has the options as the Rowv argument above and additionally when x is a square matrix, Colv = "Rowv" means that columns should be treated identically to the rows.}

\item{distfun}{function used to compute the distance (dissimilarity) between both rows and columns. Defaults to dist.}

\item{hclustfun}{function used to compute the hierarchical clustering when Rowv or Colv are not dendrograms. Defaults to hclust.}

\item{dist_method}{default is NULL (which results in "euclidean" to be used). Can accept alternative character strings indicating the
method to be passed to distfun. By default distfun. is \link{dist} hence
this can be one of "euclidean", "maximum", "manhattan", "canberra", "binary" or "minkowski".}

\item{hclust_method}{default is NULL (which results in "complete" to be used). Can accept alternative character strings indicating the
method to be passed to hclustfun By default hclustfun is \link{hclust} hence
this can be one of "ward.D", "ward.D2", "single", "complete", "average" (= UPGMA), "mcquitty" (= WPGMA), "median" (= WPGMC) or "centroid" (= UPGMC).}

\item{distfun_row}{distfun for row dendrogram only.}

\item{hclustfun_row}{hclustfun for col dendrogram only.}

\item{distfun_col}{distfun for row dendrogram only.}

\item{hclustfun_col}{hclustfun for col dendrogram only.}

\item{dendrogram}{character string indicating whether to compute 'none',
'row', 'column' or 'both' dendrograms. Defaults to 'both'. However, if Rowv
(or Colv) is FALSE or NULL and dendrogram is 'both', then a warning is issued
and Rowv (or Colv) arguments are honoured.}

\item{show_dendrogram}{Logical vector of length controlling whether the row
and column dendrograms are displayed. If a logical scalar is
provided, it is repeated to become a logical vector of length two.}

\item{reorderfun}{function(d, w) of dendrogram and weights for reordering the row and column dendrograms. The default uses stats{reorder.dendrogram}}

\item{k_row}{an integer scalar with the desired number of groups by which to color the dendrogram's branches in the rows (uses \link[dendextend]{color_branches})
If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.}

\item{k_col}{an integer scalar with the desired number of groups by which to color the dendrogram's branches in the columns (uses \link[dendextend]{color_branches})
If NA then \link[dendextend]{find_k} is used to deduce the optimal number of clusters.}

\item{symm}{logical indicating if x should be treated symmetrically; can only be true when x is a square matrix.}

\item{revC}{logical indicating if the column order should be reversed for plotting.
Default (when missing) - is FALSE, unless symm is TRUE.
This is useful for cor matrix.}

\item{scale}{character indicating if the values should be centered and scaled in either the row direction or the column direction, or none. The default is "none".}

\item{na.rm}{logical indicating whether NA's should be removed.}

\item{labRow}{character vectors with row labels to use (from top to bottom); default to rownames(x).}

\item{labCol}{character vectors with column labels to use (from left to right); default to colnames(x).}

\item{cexRow}{positive numbers. If not missing, it will override \code{xaxis_font_size}
and will give it a value cexRow*14}

\item{cexCol}{positive numbers. If not missing, it will override \code{yaxis_font_size}
and will give it a value cexCol*14}

\item{digits}{integer indicating the number of decimal places to be used by \link{round} for 'label'.}

\item{cellnote}{(optional) matrix of the same dimensions as \code{x}
that has the human-readable version of each value,
for displaying on top of the heatmap cells.}

\item{theme}{A custom CSS theme to use. Currently the only valid values are
\code{""} and \code{"dark"}. \code{"dark"} is primarily intended for
standalone visualizations, not R Markdown or Shiny.}

\item{colors}{Either a colorbrewer2.org palette name (e.g. \code{"YlOrRd"} or
\code{"Blues"}), or a vector of colors to interpolate in hexadecimal
\code{"#RRGGBB"} format, or a color interpolation function like
\code{\link[grDevices]{colorRamp}}.}

\item{width}{Width in pixels (optional, defaults to automatic sizing).}

\item{height}{Height in pixels (optional, defaults to automatic sizing).}

\item{xaxis_height}{Size of axes, in pixels.}

\item{yaxis_width}{Size of axes, in pixels.}

\item{xaxis_font_size}{Font size of axis labels, as a CSS size (e.g. "14px" or "12pt").}

\item{yaxis_font_size}{Font size of axis labels, as a CSS size (e.g. "14px" or "12pt").}

\item{brush_color}{The base color to be used for the brush. The brush will be
filled with a low-opacity version of this color. \code{"#RRGGBB"} format
expected.}

\item{show_grid}{\code{TRUE} to show gridlines, \code{FALSE} to hide them, or
a numeric value to specify the gridline thickness in pixels (can be a
non-integer).}

\item{anim_duration}{Number of milliseconds to animate zooming in and out.
For large \code{x} it may help performance to set this value to \code{0}.}

\item{row_side_colors, col_side_colors}{data.frame of factors to produce
row/column side colors in the style of heatmap.2/heatmap.3.
col_side_colors should be "wide", ie be the same dimensions
as the column side colors it will produce.}

\item{seriate}{character indicating the method of matrix sorting (default: "OLO").
Implemented options include:
"OLO" (Optimal leaf ordering, optimizes the Hamiltonian path length that is restricted by the dendrogram structure - works in O(n^4) )
"mean" (sorts the matrix based on the reorderfun using marginal means of the matrix. This is the default used by \link[gplots]{heatmap.2}),
"none" (the default order produced by the dendrogram),
"GW" (Gruvaeus and Wainer heuristic to optimize the Hamiltonian path length that is restricted by the dendrogram structure)}

\item{point_size_mat}{A matrix of values which can be mapped to point size}

\item{custom_hovertext}{Custom hovertext matrix (the same dimensions as the input).}

\item{...}{currently ignored}
}
\description{
An object of class heatmapr includes all the needed information
for producing a heatmap. The goal is to separate the pre-processing of the
heatmap elements from the graphical rendering of the object, which could be done
using plotly (but potentially also with other graphical devices).
}
\examples{

\dontrun{
library(heatmaply)
hm <- heatmapr(mtcars, scale = "column", colors = "Blues")
heatmaply(hm)
}

}
\seealso{
\link{heatmap}, \link[gplots]{heatmap.2}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/percentize.R
\name{normalize}
\alias{normalize}
\alias{normalize.default}
\alias{normalize.data.frame}
\alias{normalize.matrix}
\title{Normalization transformation (0-1)}
\usage{
normalize(x, ...)
}
\arguments{
\item{x}{a vector or a data.frame.}

\item{...}{Currently ignored.}
}
\value{
A vector (or data.frame) after normalizing the numeric variables.
}
\description{
An Empirical Normalization Transformation brings data to the 0 to 1 scale
by substracting the minimum and dividing by the maximum of all observations.
This is similar to \link{percentize} in that it allows to compare variables
of different scales, but it also keeps the shape of the distribution.
}
\examples{
\dontrun{
x <- mtcars
x <- data.frame(x)
x$am <- factor(x$am)
x$vs <- factor(x$vs)
heatmaply(percentize(x))
heatmaply(normalize(x))


x <- data.frame(a = 1:10, b = 11:20)
x[4:6, 1:2] <- NA
normalize(x)
normalize(x[,1])

}
}
\seealso{
\link{percentize}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RColorBrewer.R
\name{RColorBrewer_colors}
\alias{RColorBrewer_colors}
\alias{BrBG}
\alias{PiYG}
\alias{PRGn}
\alias{PuOr}
\alias{RdBu}
\alias{RdGy}
\alias{RdYlBu}
\alias{RdYlGn}
\alias{Spectral}
\alias{Blues}
\alias{BuGn}
\alias{BuPu}
\alias{GnBu}
\alias{Greens}
\alias{Greys}
\alias{Oranges}
\alias{OrRd}
\alias{PuBu}
\alias{PuBuGn}
\alias{PuRd}
\alias{Purples}
\alias{RdPu}
\alias{Reds}
\alias{YlGn}
\alias{YlGnBu}
\alias{YlOrBr}
\alias{YlOrRd}
\alias{cool_warm}
\title{RColorBrewer color Ramp Palette}
\usage{
BrBG(n)

PiYG(n)

PRGn(n)

PuOr(n)

RdBu(n)

RdGy(n)

RdYlBu(n)

RdYlGn(n)

Spectral(n)

Blues(n)

BuGn(n)

BuPu(n)

GnBu(n)

Greens(n)

Greys(n)

Oranges(n)

OrRd(n)

PuBu(n)

PuBuGn(n)

PuRd(n)

Purples(n)

RdPu(n)

Reds(n)

YlGn(n)

YlGnBu(n)

YlOrBr(n)

YlOrRd(n)

cool_warm(n)
}
\arguments{
\item{n}{the number of colors (>= 1) to be in the palette.}
}
\value{
A character vector of color names.
}
\description{
Functions for getting the colors of RColorBrewer (i.e.: \link{brewer.pal}) without the limitation of only 9/11
color values, based on \link{colorRampPalette}.

For sequential palettes this is not essential since we have \link{viridis}. But for diverging palettes
this is quit essential.

The sequential palettes names are
Blues BuGn BuPu GnBu Greens Greys Oranges OrRd PuBu PuBuGn PuRd Purples RdPu Reds YlGn YlGnBu YlOrBr YlOrRd

The diverging palettes are
BrBG PiYG PRGn PuOr RdBu RdGy RdYlBu RdYlGn Spectral
And also cool_warm. The cool_warm palette is based on Kenneth Moreland's proposal (see ref).
It goes from blue (cool) to ref (warm), based on well thought-out design elements.
}
\examples{
\dontrun{

library(RColorBrewer)
display.brewer.all(n=11,type="div"); title(main = "Divergent color palette")
display.brewer.all(n=9,type=c("seq")); title(main = "Sequential color palette")



img <- function(obj, nam) {
  image(1:length(obj), 1, as.matrix(1:length(obj)), col=obj,
        main = nam, ylab = "", xaxt = "n", yaxt = "n",  bty = "n")
}

par(mfrow = c(10,1))
img(rev(cool_warm(500)), "cool_warm, (Moreland 2009)")
img(RdBu(500), "RdBu")
img(BrBG(500), "BrBG")
img(PiYG(500), "PiYG")
img(PRGn(500), "PRGn")
img(PuOr(500), "PuOr")
img(RdGy(500), "RdGy")
img(RdYlBu(500), "RdYlBu")
img(RdYlGn(500), "RdYlGn")
img(Spectral(500), "Spectral")


library(heatmaply)
heatmaply(cor(mtcars), colors = PiYG, limits = c(-1,1))
heatmaply(cor(mtcars), colors = RdBu, limits = c(-1,1))

}
}
\references{
* Moreland, Kenneth. "Diverging color maps for scientific visualization." Advances in Visual Computing (2009): 92-103.
url: http://www.kennethmoreland.com/color-maps/
The code was provided here: http://stackoverflow.com/a/44073011/256662
Thanks to the user YAK, who relied on the code from the Rgnuplot package
(which is duplicated here, in order to save the need to import the entire package)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plots.R
\name{ggplot_side_color_plot}
\alias{ggplot_side_color_plot}
\title{Side color plots for heatmaps}
\usage{
ggplot_side_color_plot(
  df,
  palette = NULL,
  scale_title = paste(type, "side colors"),
  type = c("column", "row"),
  text_angle = if (type == "column") 0 else 90,
  is_colors = FALSE,
  fontsize = 10,
  label_name = NULL
)
}
\arguments{
\item{df}{A "molten" data.frame as produced by (eg) reshape2::melt}

\item{palette}{A function which can return colors to be used in the sidebar
plot}

\item{scale_title}{Title of the color scale. Not currently used.}

\item{type}{Horizontal or vertical plot? Valid values are "column" and "row"}

\item{text_angle}{the angle of the text of the rows/columns.}

\item{is_colors}{Use if the values in df are valid colours and should not be mapped
to a color scheme, and instead should be plotted directly.}

\item{fontsize}{Font size (currently unused)}

\item{label_name}{Name for the mouseover label, usually "row" or "column"}
}
\value{
A ggplot geom_tile object
}
\description{
Important for creating annotation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmapr.R
\name{is.heatmapr}
\alias{is.heatmapr}
\title{Is the object of class heatmapr}
\usage{
is.heatmapr(x)
}
\arguments{
\item{x}{an object.}
}
\value{
logical - is the object of class heatmapr.
}
\description{
Is the object of class heatmapr.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggheatmap.R
\name{ggheatmap}
\alias{ggheatmap}
\title{ggplot heatmap equivalent to heatmaply}
\usage{
ggheatmap(
  ...,
  widths = NULL,
  heights = NULL,
  row_dend_left = FALSE,
  hide_colorbar = FALSE
)
}
\arguments{
\item{...}{Passed to \link{heatmaply}}

\item{widths, heights}{Relative widths and heights of plots.}

\item{row_dend_left}{Logical argument controlling whether the row
dendrogram is placed on the left of the plot.}

\item{hide_colorbar}{Logical argument controlling whether the color bar (i.e.:
the legend) is hidden.}
}
\description{
This function produces a ggplot analogue of heatmaply figures
using \link[egg]{ggarrange}. This function may not always support the same
set of features as , and exporting the heatmaply object with, for example,
\link[plotly]{orca} or \code{heatmaply(mtcars, file = "foo.png")}.
}
\examples{
ggheatmap(mtcars)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmaply.R
\name{is.plotly}
\alias{is.plotly}
\title{Checks if an object is of class plotly or not.}
\usage{
is.plotly(x)
}
\arguments{
\item{x}{an object to check}
}
\value{
TRUE if the object inherits "plotly" as a class.
}
\description{
Helpful for the plot_method in link{heatmaply}.
}
